
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

#ifndef __DYNAMIC_LIB_H
#define __DYNAMIC_LIB_H

#include <dlfcn.h>

@begin
   include "basic.h"
@end

/*
 * definition of structure dynamic_lib_s
 */

@begin
   define dynamic_lib_s dynamic
@end

struct dynamic_lib_s
{
   void *lib_ptr;

   inline void init();
   inline void clear();
   inline void flush_all() {}
   inline void swap(dynamic_lib_s &a_second);
   inline dynamic_lib_s &operator=(dynamic_lib_s &a_src) {cassert(0); return *this;}
   inline bool operator==(dynamic_lib_s &a_second) {cassert(0); return false;}

   /*!
    * \brief open dynamic library
    * \param a_file - file containing dynamic library object
    * \return true if successfully opened
    */
   inline bool open(const char *a_file);

   /*!
    * \brief return addres to which was loaded symbol from library
    * \param a_addr_ptr - pointer to pointer to which is stored address of object
    * \return true if is symbol successfully resolved
    */
   inline bool get_symbol_addr(const char *a_symbol_name,void **a_addr_ptr);

   /*!
    * \brief test if is dynamic library loaded
    * \return true if is loaded some dynamic library
    */
   inline bool is_loaded();
};

/*
 * inline methods of structure dynamic_lib_s
 */

inline void dynamic_lib_s::init()
{/*{{{*/
   lib_ptr = NULL;
}/*}}}*/

inline void dynamic_lib_s::clear()
{/*{{{*/
   if (lib_ptr != NULL) {
      dlclose(lib_ptr);
   }

   init();
}/*}}}*/

inline void dynamic_lib_s::swap(dynamic_lib_s &a_second)
{/*{{{*/
   void *tmp_lib_ptr = lib_ptr;
   lib_ptr = a_second.lib_ptr;
   a_second.lib_ptr = tmp_lib_ptr;
}/*}}}*/

inline bool dynamic_lib_s::open(const char *a_file)
{/*{{{*/
   clear();

   lib_ptr = dlopen(a_file,RTLD_LAZY);
   return lib_ptr != NULL;
}/*}}}*/

inline bool dynamic_lib_s::get_symbol_addr(const char *a_symbol_name,void **a_addr_ptr)
{/*{{{*/
   *a_addr_ptr = dlsym(lib_ptr,a_symbol_name);
   return *a_addr_ptr != NULL;
}/*}}}*/

inline bool dynamic_lib_s::is_loaded()
{/*{{{*/
   return lib_ptr != NULL;
}/*}}}*/

#endif

