
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

@begin
   include "graph.h"
@end

/*
 * constants and definitions
 */

// - strings of graph type dot codes -
const char *c_graph_type_dot_codes[c_graph_type_cnt] = 
{
   "--",
   "->",
};

// - graph functions for precreated types -

// - character functions -
int gf_char_compare(void *a_first,void *a_second)
{/*{{{*/
   register char f_char = (char)(t_ps_uint)a_first;
   register char s_char = (char)(t_ps_uint)a_second;

   return f_char < s_char?-1:f_char > s_char;
}/*}}}*/

void gf_char_operator_equal(void **a_trg,void *a_src)
{/*{{{*/
   *a_trg = a_src;
}/*}}}*/

void gf_char_get_string(void *a_value,string_s &a_string)
{/*{{{*/
   a_string.setf("'%c'",(char)(t_ps_uint)a_value);
}/*}}}*/

void gf_char_serialize(void *a_value,string_s &a_string)
{/*{{{*/
   a_string.set(1,(char *)&a_value);
}/*}}}*/

void gf_char_deserialize(string_s &a_string,void **a_value)
{/*{{{*/
   *a_value = (pointer)(t_ps_uint)a_string.data[0];
}/*}}}*/

// - integer functions -
int gf_int_compare(void *a_first,void *a_second)
{/*{{{*/
   register int f_int = (int)(t_ps_uint)a_first;
   register int s_int = (int)(t_ps_uint)a_second;

   return f_int < s_int?-1:f_int > s_int;
}/*}}}*/

void gf_int_operator_equal(void **a_trg,void *a_src)
{/*{{{*/
   *a_trg = a_src;
}/*}}}*/

void gf_int_get_string(void *a_value,string_s &a_string)
{/*{{{*/
   a_string.setf("%d",(int)(t_ps_uint)a_value);
}/*}}}*/

void gf_int_serialize(void *a_value,string_s &a_string)
{/*{{{*/
   a_string.setf("%d",(int)(t_ps_uint)a_value);
}/*}}}*/

void gf_int_deserialize(string_s &a_string,void **a_value)
{/*{{{*/
   *a_value = (void *)(t_ps_uint)(int)strtol(a_string.data,NULL,10);
}/*}}}*/

// - unsigned integer functions -
int gf_unsigned_compare(void *a_first,void *a_second)
{/*{{{*/
   register unsigned f_unsigned = (unsigned)(t_ps_uint)a_first;
   register unsigned s_unsigned = (unsigned)(t_ps_uint)a_second;

   return f_unsigned < s_unsigned?-1:f_unsigned > s_unsigned;
}/*}}}*/

void gf_unsigned_operator_equal(void **a_trg,void *a_src)
{/*{{{*/
   *a_trg = a_src;
}/*}}}*/

void gf_unsigned_get_string(void *a_value,string_s &a_string)
{/*{{{*/
   a_string.setf("%u",(unsigned)(t_ps_uint)a_value);
}/*}}}*/

void gf_unsigned_serialize(void *a_value,string_s &a_string)
{/*{{{*/
   a_string.setf("%u",(unsigned)(t_ps_uint)a_value);
}/*}}}*/

void gf_unsigned_deserialize(string_s &a_string,void **a_value)
{/*{{{*/
   *a_value = (void *)(t_ps_uint)(unsigned)strtol(a_string.data,NULL,10);
}/*}}}*/

// - float functions -
int gf_float_compare(void *a_first,void *a_second)
{/*{{{*/
   register float f_float = *((float *)&a_first);
   register float s_float = *((float *)&a_second);

   return f_float < s_float?-1:f_float > s_float;
}/*}}}*/

void gf_float_operator_equal(void **a_trg,void *a_src)
{/*{{{*/
   *a_trg = a_src;
}/*}}}*/

void gf_float_get_string(void *a_value,string_s &a_string)
{/*{{{*/
   a_string.setf("%f",*((float *)&a_value));
}/*}}}*/

void gf_float_serialize(void *a_value,string_s &a_string)
{/*{{{*/
   a_string.setf("%f",*((float *)&a_value));
}/*}}}*/

void gf_float_deserialize(string_s &a_string,void **a_value)
{/*{{{*/
   *((float *)a_value) = (float)strtod(a_string.data,NULL);
}/*}}}*/

// - string functions -
void gf_string_init(void **a_value_ptr)
{/*{{{*/
   string_s *str_ptr = (string_s *)cmalloc(sizeof(string_s));
   str_ptr->init();

   *a_value_ptr = (void *)str_ptr;
}/*}}}*/

int gf_string_compare(void *a_first,void *a_second)
{/*{{{*/
   string_s &f_str = *(string_s *)a_first;
   string_s &s_str = *(string_s *)a_second;

   return strcmp(f_str.data,s_str.data);
}/*}}}*/

void gf_string_operator_equal(void **a_trg,void *a_src)
{/*{{{*/
   string_s &t_str = *(string_s *)*a_trg;
   string_s &s_str = *(string_s *)a_src;

   t_str = s_str;
}/*}}}*/

void gf_string_get_string(void *a_value,string_s &a_string)
{/*{{{*/
   string_s &str = *(string_s *)a_value;

   a_string.setf("\"%s\"",str.data);
}/*}}}*/

void gf_string_clear(void *a_value)
{/*{{{*/
   string_s &str = *(string_s *)a_value;

   str.clear();
   cfree(a_value);
}/*}}}*/

void gf_string_serialize(void *a_value,string_s &a_string)
{/*{{{*/
   a_string = *(string_s *)a_value;
}/*}}}*/

void gf_string_deserialize(string_s &a_string,void **a_value)
{/*{{{*/
   *((string_s *)*a_value) = a_string;
}/*}}}*/

/*
 * arrays of graph functions for precreated types
 */

const void *graph_blank_functions[] = 
{/*{{{*/
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
   NULL,
};/*}}}*/

const void *graph_char_functions[] =
{/*{{{*/
   NULL,
   (void *)gf_char_compare,
   (void *)gf_char_operator_equal,
   (void *)gf_char_get_string,
   NULL,
   (void *)gf_char_serialize,
   (void *)gf_char_deserialize,
};/*}}}*/

const void *graph_int_functions[] = 
{/*{{{*/
   NULL,
   (void *)gf_int_compare,
   (void *)gf_int_operator_equal,
   (void *)gf_int_get_string,
   NULL,
   (void *)gf_int_serialize,
   (void *)gf_int_deserialize,
};/*}}}*/

const void *graph_unsigned_functions[] = 
{/*{{{*/
   NULL,
   (void *)gf_unsigned_compare,
   (void *)gf_unsigned_operator_equal,
   (void *)gf_unsigned_get_string,
   NULL,
   (void *)gf_unsigned_serialize,
   (void *)gf_unsigned_deserialize,
};/*}}}*/

const void *graph_float_functions[] = 
{/*{{{*/
   NULL,
   (void *)gf_float_compare,
   (void *)gf_float_operator_equal,
   (void *)gf_float_get_string,
   NULL,
   (void *)gf_float_serialize,
   (void *)gf_float_deserialize,
};/*}}}*/

extern const void *graph_string_functions[] =
{/*{{{*/
   (void *)gf_string_init,
   (void *)gf_string_compare,
   (void *)gf_string_operator_equal,
   (void *)gf_string_get_string,
   (void *)gf_string_clear,
   (void *)gf_string_serialize,
   (void *)gf_string_deserialize,
};/*}}}*/

/*
 * methods of generated structures
 */

// -- gg_vertex_map_s --
@begin
   methods gg_vertex_map_s
@end

// -- gg_vertex_map_set_s --
@begin
   methods gg_vertex_map_set_s
@end

// -- gg_vertex_map_sets_s --
@begin
   methods gg_vertex_map_sets_s
@end

unsigned gg_vertex_map_set_s::get_idx_by_sub_vertex(unsigned a_idx,unsigned a_sv_idx)
{/*{{{*/
   unsigned vm_idx = a_idx;
   do {
      gg_vertex_map_s &vmap = data[vm_idx].object;

      if (vmap.sv_idx == a_sv_idx) {
	 return vm_idx;
      }

      vm_idx = vmap.prev_map_idx;
   } while(vm_idx != c_idx_not_exist);

   return c_idx_not_exist;
}/*}}}*/

unsigned gg_vertex_map_set_s::get_idx_by_mother_vertex(unsigned a_idx,unsigned a_mv_idx)
{/*{{{*/
   unsigned vm_idx = a_idx;
   do {
      gg_vertex_map_s &vmap = data[vm_idx].object;

      if (vmap.mv_idx == a_mv_idx) {
	 return vm_idx;
      }

      vm_idx = vmap.prev_map_idx;
   } while(vm_idx != c_idx_not_exist);

   return c_idx_not_exist;
}/*}}}*/

unsigned gg_vertex_map_set_s::get_map_with_unused_edges(unsigned sg_vertex_cnt,unsigned a_idx)
{/*{{{*/
   bool tested[sg_vertex_cnt];
   memset(tested,false,sg_vertex_cnt*sizeof(bool));

   unsigned vm_idx = a_idx;
   do {
      gg_vertex_map_s &vmap = data[vm_idx].object;

      if (!tested[vmap.sv_idx]) {
	 if (vmap.next_edge_idx != c_idx_not_exist) {
	    return vm_idx;
	 }

	 tested[vmap.sv_idx] = true;
      }

      vm_idx = vmap.prev_map_idx;
   } while(vm_idx != c_idx_not_exist);

   return c_idx_not_exist;
}/*}}}*/

void gg_vertex_map_set_s::retrieve_mapping_sub_to_mother(unsigned a_idx,ui_array_s &a_mapping)
{/*{{{*/
   unsigned vm_idx = a_idx;
   do {
      gg_vertex_map_s &vmap = data[vm_idx].object;

      a_mapping[vmap.sv_idx] = vmap.mv_idx;

      vm_idx = vmap.prev_map_idx;
   } while(vm_idx != c_idx_not_exist);
}/*}}}*/

void gg_vertex_map_set_s::release_vmap_string(unsigned a_idx)
{/*{{{*/
   unsigned vm_idx = a_idx;
   do {
      gg_vertex_map_s &vmap = data[vm_idx].object;

      if (--vmap.reference_cnt == 0) {

	 // - remove gg_vertex_map from set -
	 remove(vm_idx);

	 vm_idx = vmap.prev_map_idx;
      }
      else {
	 break;
      }
   } while(vm_idx != c_idx_not_exist);
}/*}}}*/

void gg_vertex_map_set_s::DEBUG_print_string(unsigned a_idx)
{/*{{{*/
   fprintf(stderr,"mapping %u: ",a_idx);

   unsigned vm_idx = a_idx;
   do {
      gg_vertex_map_s &vmap = data[vm_idx].object;

      fprintf(stderr,"%u(%u)->%u, ",vmap.sv_idx,vmap.next_edge_idx,vmap.mv_idx);

      vm_idx = vmap.prev_map_idx;
   } while(vm_idx != c_idx_not_exist);

   fputc('\n',stderr);
}/*}}}*/

// -- edge_s --
@begin
   methods edge_s
@end

// -- edge_rb_tree_s --
@begin
   methods edge_rb_tree_s
@end

// -- edge_idxs_s --
@begin
   methods edge_idxs_s
@end

void edge_idxs_s::get_edge_idxs_by_edge(edge_s &a_edge,ui_array_s &a_edge_idxs,unsigned &a_edge_cnt)
{/*{{{*/
   if (root_idx == c_idx_not_exist) {
      return;
   }

   edge_rb_tree_s &edges = ((graph_s *)graph_ptr)->edges;

   unsigned stack[get_descent_stack_size()];
   unsigned *stack_ptr = stack;

   *(stack_ptr++) = root_idx;
   do {
      unsigned node_idx = *(--stack_ptr);
      edge_idxs_s_node &node = data[node_idx];

      int comp_result = compare_edges(a_edge,edges[node.object]);
      if (comp_result < 0) {
         if (node.left_idx != leaf_idx) {
            *(stack_ptr++) = node.left_idx;
         }
      }
      else {
         if (comp_result == 0) {
            a_edge_idxs.push(data[node_idx].object);
            a_edge_cnt++;

            if (node.left_idx != leaf_idx) {
               *(stack_ptr++) = node.left_idx;
            }
         }

         if (node.right_idx != leaf_idx) {
            *(stack_ptr++) = node.right_idx;
         }
      }
   } while(stack_ptr > stack);
}/*}}}*/

// -- vertex_s --
@begin
   methods vertex_s
@end

// -- vertex_rb_tree_s --
@begin
   methods vertex_rb_tree_s
@end

// -- graph_s --
@begin
   methods graph_s
@end

void graph_s::clear()
{/*{{{*/
   if (vertices.root_idx != c_idx_not_exist) {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vertices.vf_clear(vertices[v_idx].value);

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   if (edges.root_idx != c_idx_not_exist) {
      unsigned stack[edges.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned e_idx = edges.get_stack_min_value_idx(edges.root_idx,&stack_ptr);
      do {
         edges.ef_clear(edges[e_idx].value);

         e_idx = edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
      } while(e_idx != c_idx_not_exist);
   }

   vertex_cnt = 0;
   vertices.clear();

   edge_cnt = 0;
   edges.clear();

   init();
}/*}}}*/

void graph_s::swap(graph_s &second)
{/*{{{*/
   unsigned tmp_type = type;
   type = second.type;
   second.type = tmp_type;

   unsigned tmp_vertex_cnt = vertex_cnt;
   vertex_cnt = second.vertex_cnt;
   second.vertex_cnt = tmp_vertex_cnt;

   vertices.swap(second.vertices);

   // - remap edge_idxs graph_ptr -
   if (vertices.root_idx != c_idx_not_exist) {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vertices[v_idx].edge_idxs.graph_ptr = (pointer)this;
         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   // - remap second edge_idxs graph_ptr -
   if (second.vertices.root_idx != c_idx_not_exist) {
      vertex_rb_tree_s &s_vertices = second.vertices;

      unsigned stack[s_vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned v_idx = s_vertices.get_stack_min_value_idx(s_vertices.root_idx,&stack_ptr);
      do {
         s_vertices[v_idx].edge_idxs.graph_ptr = (pointer)&second;
         v_idx = s_vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   unsigned tmp_edge_cnt = edge_cnt;
   edge_cnt = second.edge_cnt;
   second.edge_cnt = tmp_edge_cnt;

   edges.swap(second.edges);
}/*}}}*/

graph_s &graph_s::operator=(graph_s &second)
{/*{{{*/
   clear();

   type = second.type;
   vertex_cnt = second.vertex_cnt;
   vertices = second.vertices;
   edge_cnt = second.edge_cnt;
   edges = second.edges;

   // - remap edge_idxs graph_ptr, and copy vertex evaluation -
   if (vertices.root_idx != c_idx_not_exist) {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vertex_s &vertex = vertices[v_idx];

         vertices.vf_init(&vertex.value);
         vertices.vf_operator_equal(&vertex.value,second.vertices[v_idx].value);
         vertex.edge_idxs.graph_ptr = (pointer)this;

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   // - copy edge evaluation -
   if (edges.root_idx != c_idx_not_exist) {
      unsigned stack[edges.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned e_idx = edges.get_stack_min_value_idx(edges.root_idx,&stack_ptr);
      do {
         edge_s &edge = edges[e_idx];

         edges.ef_init(&edge.value);
         edges.ef_operator_equal(&edge.value,second.edges[e_idx].value);

         e_idx = edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
      } while(e_idx != c_idx_not_exist);
   }

   return *this;
}/*}}}*/

void graph_s::deserialize_evaluation(void **a_vertices_value_fps,void **a_edges_value_fps)
{/*{{{*/
   cassert(vertices.value_fps == graph_string_functions && edges.value_fps == graph_string_functions);

   // - retrieve deserialize function pointers -
   void (*vf_init_ptr)(void **) = (void (*)(void **))a_vertices_value_fps[c_graph_function_init];
   void (*vf_deserialize_ptr)(string_s &,void **) = (void (*)(string_s &,void **))a_vertices_value_fps[c_graph_function_deserialize];

   void (*ef_init_ptr)(void **) = (void (*)(void **))a_edges_value_fps[c_graph_function_init];
   void (*ef_deserialize_ptr)(string_s &,void **) = (void (*)(string_s &,void **))a_edges_value_fps[c_graph_function_deserialize];

   // - peocess vertexes -
   if (vertices.root_idx != c_idx_not_exist) {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         void *&value = vertices[v_idx].value;

         // - initialize new evaluation -
         void *new_value;
         if (vf_init_ptr != NULL) (*vf_init_ptr)(&new_value);
         else new_value = NULL;

         // - deserialize string to new value -
         (*vf_deserialize_ptr)(*(string_s *)value,&new_value);

         // - clear old evaluation (string) -
         vertices.vf_clear(value);

         // - replace original value by new value -
         value = new_value;

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   vertices.value_fps = a_vertices_value_fps;

   // - process edges -
   if (edges.root_idx != c_idx_not_exist) {
      unsigned stack[edges.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned e_idx = edges.get_stack_min_value_idx(edges.root_idx,&stack_ptr);
      do {
         void *&value = edges[e_idx].value;

         // - initialize new evaluation -
         void *new_value;
         if (ef_init_ptr != NULL) (*ef_init_ptr)(&new_value);
         else new_value = NULL;

         // - deserialize string to new value -
         (*ef_deserialize_ptr)(*(string_s *)value,&new_value);

         // - clear old evaluation (string) -
         edges.ef_clear(value);

         // - replace original value by new value -
         value = new_value;
         
         e_idx = edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
      } while(e_idx != c_idx_not_exist);
   }

   edges.value_fps = a_edges_value_fps;

   // - rehash all red black trees -
   {
      vertices.rehash_tree();
      edges.rehash_tree();

      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vertices[v_idx].edge_idxs.rehash_tree();
         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   // FIXME debug red black tree properties assertion
   /*{
      cassert(vertices.check_rb_tree_properties());
      cassert(edges.check_rb_tree_properties());

      {
         unsigned stack[vertices.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
         do {
            cassert(vertices[v_idx].edge_idxs.check_rb_tree_properties());
            v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
         } while(v_idx != c_idx_not_exist);
      }
   }*/

}/*}}}*/

bool graph_s::is_dipole_graph()
{/*{{{*/

   // - vertex count must be 2 -
   if (vertex_cnt != 2 || edge_cnt == 0) {
      return false;
   }

   // - all edges hase no same end vertices -
   unsigned stack[edges.get_descent_stack_size()];
   unsigned *stack_ptr = stack;

   unsigned e_idx = edges.get_stack_min_value_idx(edges.root_idx,&stack_ptr);
   do {
      edge_s &edge = edges[e_idx];
      if (edge.src_vertex_idx == edge.trg_vertex_idx) {
         return false;
      }

      e_idx = edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
   } while(e_idx != c_idx_not_exist);

   return true;
}/*}}}*/

bool graph_s::is_connected_graph()
{/*{{{*/
   if (vertex_cnt <= 1) {
      return true;
   }

   bool v_map[vertices.used];
   memset(v_map,false,vertices.used*sizeof(bool));

   bool e_map[edges.used];
   memset(e_map,false,edges.used*sizeof(bool));

   // - traverse throught all edges, closure incident to vertices of first edge -
   if (edges.root_idx != c_idx_not_exist) {
      ui_array_s edge_stack;
      edge_stack.init();

      // - push first edge on stack and mark it -
      e_map[edges.root_idx] = true;
      edge_stack.push(edges.root_idx);

      do {
         edge_s &edge = edges[edge_stack.pop()];

         // - if edge source vertex was not processed -
         if (!v_map[edge.src_vertex_idx]) {
            
            // - traverse throught its incident edges -
            edge_idxs_s &edge_idxs = vertices[edge.src_vertex_idx].edge_idxs;
            if (edge_idxs.root_idx != c_idx_not_exist) {
               unsigned stack[edge_idxs.get_descent_stack_size()];
               unsigned *stack_ptr = stack;

               unsigned ei_idx = edge_idxs.get_stack_min_value_idx(edge_idxs.root_idx,&stack_ptr);
               do {
                  // - if edge was not processed push it on the stack and process it -
                  if (!e_map[edge_idxs[ei_idx]]) {
                     edge_stack.push(edge_idxs[ei_idx]);
                     e_map[edge_idxs[ei_idx]] = true;
                  }

                  ei_idx = edge_idxs.get_stack_next_idx(ei_idx,&stack_ptr,stack);
               } while(ei_idx != c_idx_not_exist);
            }

            // - mark vertex as processed -
            v_map[edge.src_vertex_idx] = true;
         }

         // - if edge target vertex was not processed -
         if (!v_map[edge.trg_vertex_idx]) {
            
            // - traverse throught its incident edges -
            edge_idxs_s &edge_idxs = vertices[edge.trg_vertex_idx].edge_idxs;
            if (edge_idxs.root_idx != c_idx_not_exist) {
               unsigned stack[edge_idxs.get_descent_stack_size()];
               unsigned *stack_ptr = stack;

               unsigned ei_idx = edge_idxs.get_stack_min_value_idx(edge_idxs.root_idx,&stack_ptr);
               do {
                  // - if edge was not processed push it on the stack and process it -
                  if (!e_map[edge_idxs[ei_idx]]) {
                     edge_stack.push(edge_idxs[ei_idx]);
                     e_map[edge_idxs[ei_idx]] = true;
                  }

                  ei_idx = edge_idxs.get_stack_next_idx(ei_idx,&stack_ptr,stack);
               } while(ei_idx != c_idx_not_exist);
            }

            // - mark vertex as processed -
            v_map[edge.trg_vertex_idx] = true;
         }

      } while(edge_stack.used > 0);

      edge_stack.clear();
   }

   // - test if all vertices was accesed (if not graph is not connected) -
   {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         if (!v_map[v_idx]) {
            return false;
         }

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);

      return true;
   }
}/*}}}*/

bool graph_s::is_bipartite_graph()
{/*{{{*/
   if (vertex_cnt <= 1) {
      return false;
   }

   unsigned v_mark[vertices.used];
   memset(v_mark,c_idx_not_exist,vertices.used*sizeof(unsigned));

   ui_array_s vertex_stack;
   vertex_stack.init();

   // - for each vertex find mark from {0,1} such that there is no edge between vertices with same mark -
   unsigned v_stack[vertices.get_descent_stack_size()];
   unsigned *v_stack_ptr = v_stack;

   unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&v_stack_ptr);
   do {
      
      // - if vertex was not marked yet (new component of graph) -
      if (v_mark[v_idx] == c_idx_not_exist) {
         
         // - mark it with first symbol and push it to stack -
         v_mark[v_idx] = 0;
         vertex_stack.push(v_idx);

         do {
            unsigned sv_idx = vertex_stack.pop();
            edge_idxs_s &edge_idxs = vertices[sv_idx].edge_idxs;

            // - traverse throught its incident edges -
            if (edge_idxs.root_idx != c_idx_not_exist) {
               unsigned stack[edge_idxs.get_descent_stack_size()];
               unsigned *stack_ptr = stack;

               unsigned ei_idx = edge_idxs.get_stack_min_value_idx(edge_idxs.root_idx,&stack_ptr);
               do {
                  edge_s &edge = edges[edge_idxs[ei_idx]];

                  // - process vertex of edge which is not vertex poped from stack -

                  if (edge.src_vertex_idx != sv_idx) {

                     // - if mark of vertex equals to poped vertex mark => graph is not bipartite -
                     if (v_mark[edge.src_vertex_idx] == v_mark[sv_idx]) {
                        vertex_stack.clear();
                        return false;
                     }
                     // - if vertex is not marked mark it with oposite mark to poped vertex mark -
                     else if (v_mark[edge.src_vertex_idx] == c_idx_not_exist) {
                        v_mark[edge.src_vertex_idx] = 1 - v_mark[sv_idx];
                        vertex_stack.push(edge.src_vertex_idx);
                     }
                  }
                  else if (edge.trg_vertex_idx != sv_idx) {

                     // - if mark of vertex equals to poped vertex mark => graph is not bipartite -
                     if (v_mark[edge.trg_vertex_idx] == v_mark[sv_idx]) {
                        vertex_stack.clear();
                        return false;
                     }
                     // - if vertex is not marked mark it with oposite mark to poped vertex mark -
                     else if (v_mark[edge.trg_vertex_idx] == c_idx_not_exist) {
                        v_mark[edge.trg_vertex_idx] = 1 - v_mark[sv_idx];
                        vertex_stack.push(edge.trg_vertex_idx);
                     }
                  }
                  else {
                     vertex_stack.clear();
                     return false;
                  }

                  // - move to next diferend edge (ignore multi edges with same value, source vertex and target vertex) -
                  unsigned old_edge_idx = ei_idx;
                  do {
                     ei_idx = edge_idxs.get_next_idx(ei_idx);
                  } while(ei_idx != c_idx_not_exist && edge_idxs.compare_edges(edges[old_edge_idx],edges[edge_idxs[ei_idx]]) == 0);
               } while(ei_idx != c_idx_not_exist);
            }
         } while(vertex_stack.used > 0);
      }

      v_idx = vertices.get_stack_next_idx(v_idx,&v_stack_ptr,v_stack_ptr);
   } while(v_idx != c_idx_not_exist);

   vertex_stack.clear();

   return true;
}/*}}}*/

bool graph_s::is_regular_graph()
{/*{{{*/
   if (vertex_cnt == 0) return true;

   unsigned stack[vertices.get_descent_stack_size()];
   unsigned *stack_ptr = stack;

   unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
   unsigned regular_degree = vertices[v_idx].get_degree();

   // - if graph is regular all vertices has same (regular) degree -
   v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
   if (v_idx != c_idx_not_exist) {
      do {
         if (vertices[v_idx].get_degree() != regular_degree) {
            return false;
         }

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   return true;
}/*}}}*/

void graph_s::get_adjacent_vertices(unsigned a_vertex_idx,ui_array_s &a_vertex_idxs)
{/*{{{*/
   a_vertex_idxs.used = 0;

   // - set of edges incident to vertex -
   edge_idxs_s &edge_idxs = vertices[a_vertex_idx].edge_idxs;
   if (edge_idxs.root_idx != c_idx_not_exist) {
      unsigned stack[edge_idxs.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned ei_idx = edge_idxs.get_stack_min_value_idx(edge_idxs.root_idx,&stack_ptr);
      do {
         edge_s &edge = edges[edge_idxs[ei_idx]];

         // - all vertices which are not same as a_vertex_idx insert to result array -
         if (a_vertex_idx != edge.src_vertex_idx) {
            a_vertex_idxs.push(edge.src_vertex_idx);
         }

         if (a_vertex_idx != edge.trg_vertex_idx) {
            a_vertex_idxs.push(edge.trg_vertex_idx);
         }

         // - move to next diferend edge (ignore multi edges with same value, source vertex and target vertex) -
         unsigned old_edge_idx = ei_idx;
         do {
            ei_idx = edge_idxs.get_next_idx(ei_idx);
         } while(ei_idx != c_idx_not_exist && edge_idxs.compare_edges(edges[old_edge_idx],edges[edge_idxs[ei_idx]]) == 0);
      } while(ei_idx != c_idx_not_exist);
   }
}/*}}}*/

void graph_s::get_adjacent_edges(unsigned a_vertex_idx,ui_array_s &a_edge_idxs)
{/*{{{*/
   a_edge_idxs.used = 0;

   edge_idxs_s &edge_idxs = vertices[a_vertex_idx].edge_idxs;

   // - insert all edges in incident set to result array -
   if (edge_idxs.root_idx != c_idx_not_exist) {
      unsigned stack[edge_idxs.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned ei_idx = edge_idxs.get_stack_min_value_idx(edge_idxs.root_idx,&stack_ptr);
      do {
         a_edge_idxs.push(edge_idxs[ei_idx]);
         ei_idx = edge_idxs.get_stack_next_idx(ei_idx,&stack_ptr,stack);
      } while(ei_idx != c_idx_not_exist);
   }
}/*}}}*/

inline int comp_int(const void *first,const void *second)
{/*{{{*/
   register unsigned f_val = *((unsigned *)first);
   register unsigned s_val = *((unsigned *)second);

   return (f_val > s_val?-1:(f_val < s_val)?1:0);
}/*}}}*/

void graph_s::get_degree_sequence(ui_array_s &a_degree_sequence)
{/*{{{*/
   a_degree_sequence.used = 0;

   if (vertices.root_idx == c_idx_not_exist) {
      return;
   }

   // - retrieve array of degrees -
   unsigned stack[vertices.get_descent_stack_size()];
   unsigned *stack_ptr = stack;

   unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
   do {
      a_degree_sequence.push(vertices[v_idx].get_degree());
      v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
   } while(v_idx != c_idx_not_exist);

   // - sort degrees in non decreasing order -
   qsort(a_degree_sequence.data,a_degree_sequence.used,sizeof(unsigned),comp_int);
}/*}}}*/

bool edge_src_trg_vertex_compare(edge_s &a_first,edge_s &a_second)
{/*{{{*/
   return a_first.src_vertex_idx == a_second.src_vertex_idx && a_first.trg_vertex_idx == a_second.trg_vertex_idx;
}/*}}}*/

unsigned graph_s::find_source_node()
{/*{{{*/
   cassert(type == c_graph_type_digraph);

   if (vertex_cnt == 0) {
      return c_idx_not_exist;
   }

   uc_array_s im;
   im.init_size(vertices.used*vertices.used);
   im.used = im.size;
   memset(im.data,0,im.used*sizeof(unsigned char));

   uc_array_s aim;
   aim.init_size(vertices.used*vertices.used);
   aim.used = aim.size;
   memset(aim.data,0,aim.used*sizeof(unsigned char));

   uc_array_s taim;
   taim.init_size(vertices.used*vertices.used);
   taim.used = taim.size;

   ui_array_s rim;
   rim.init_size(vertices.used*vertices.used);
   rim.used = rim.size;
   memset(rim.data,0,rim.used*sizeof(unsigned));

   unsigned vi_array[vertex_cnt];

   // - set vertex index array -
   {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned i_idx = 0;
      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vi_array[i_idx] = v_idx;

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(++i_idx,v_idx != c_idx_not_exist);
   }

   // - set incident and actual incident matrices -
   {
      unsigned i_idx = 0;
      do {
         unsigned v_idx = vi_array[i_idx];
         vertex_s &vertex = vertices[v_idx];
         edge_idxs_s &edge_idxs = vertex.edge_idxs;

         if (edge_idxs.root_idx != c_idx_not_exist) {
            unsigned stack[edge_idxs.get_descent_stack_size()];
            unsigned *stack_ptr = stack;

            unsigned ei_idx = edge_idxs.get_stack_min_value_idx(edge_idxs.root_idx,&stack_ptr);
            do {
               edge_s &edge = edges[edge_idxs[ei_idx]];

               if (edge.src_vertex_idx == v_idx) {
                  unsigned arr_idx = v_idx*vertices.used + edge.trg_vertex_idx;

                  im[arr_idx] = 1;
                  aim[arr_idx] = 1;
               }

               ei_idx = edge_idxs.get_stack_next_idx(ei_idx,&stack_ptr,stack);
            } while(ei_idx != c_idx_not_exist);
         }
      } while(++i_idx < vertex_cnt);
   }

   unsigned i_idx = 0;
   do {
      
      // - add aim matrix to rim matrix -
      {
         unsigned ri_idx = 0;
         do {
            unsigned rv_idx = vi_array[ri_idx];
            unsigned ci_idx = 0;
            do {
               unsigned arr_idx = rv_idx*vertices.used + vi_array[ci_idx];

               rim[arr_idx] += aim[arr_idx];
            } while(++ci_idx < vertex_cnt);
         } while(++ri_idx < vertex_cnt);
      }

      // - clear target actual incident matrix -
      memset(taim.data,0,taim.used*sizeof(unsigned char));

      // - multiple aim with im -> taim -
      unsigned ri_idx = 0;
      do {
         unsigned rv_idx = vi_array[ri_idx];
         unsigned char *aim_r_ptr = aim.data + rv_idx*vertices.used;
         unsigned char *taim_r_ptr = taim.data + rv_idx*vertices.used;

         unsigned ci_idx = 0;
         do {
            unsigned cv_idx = vi_array[ci_idx];

            if (aim_r_ptr[cv_idx]) {
               unsigned char *im_r_ptr = im.data + cv_idx*vertices.used;

               unsigned i_ci_idx = 0;
               do {
                  unsigned i_cv_idx = vi_array[i_ci_idx];
                  
                  taim_r_ptr[i_cv_idx] |= im_r_ptr[i_cv_idx];
               } while(++i_ci_idx < vertex_cnt);
            }
         } while(++ci_idx < vertex_cnt);
      } while(++ri_idx < vertex_cnt);

      // - swap aim and taim matrices -
      aim.swap(taim);

   } while(++i_idx < vertex_cnt);

   // - find minimal vertex incidence count -
   unsigned min_ic = UINT_MAX;
   unsigned min_ic_v_idx = c_idx_not_exist;

   unsigned ci_idx = 0;
   do {
      unsigned cv_idx = vi_array[ci_idx];
      unsigned ic = 0;

      unsigned *rim_c_ptr = rim.data + cv_idx;
      unsigned ri_idx = 0;
      do {
         ic += rim_c_ptr[vi_array[ri_idx]*vertices.used];
      } while(++ri_idx < vertex_cnt);

      if (ic < min_ic) {
         min_ic = ic;
         min_ic_v_idx = cv_idx;
      }

   } while(++ci_idx < vertex_cnt);

   rim.clear();
   taim.clear();
   aim.clear();
   im.clear();

   return min_ic_v_idx;
}/*}}}*/

unsigned graph_s::find_source_node_optimalized()
{/*{{{*/
   cassert(type == c_graph_type_digraph);

   if (vertex_cnt == 0) {
      return c_idx_not_exist;
   }

   unsigned row_b_length = (vertices.used & 0x1f)?(vertices.used & 0xffffffe0) + 32:vertices.used;
   unsigned row_length = row_b_length >> 5;

   unsigned vi_array[vertex_cnt];

   // - set vertex index array -
   {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned i_idx = 0;
      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vi_array[i_idx] = v_idx;

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(++i_idx,v_idx != c_idx_not_exist);
   }

   // - create incident matrix -
   ui_array_s im;
   im.init_size(row_b_length*row_length);
   im.used = im.size;
   memset(im.data,0,im.used*sizeof(unsigned));

   // - set incident matrix -
   {
      unsigned i_idx = 0;
      do {
         unsigned v_idx = vi_array[i_idx];
         vertex_s &vertex = vertices[v_idx];
         edge_idxs_s &edge_idxs = vertex.edge_idxs;

         if (edge_idxs.root_idx != c_idx_not_exist) {
            unsigned stack[edge_idxs.get_descent_stack_size()];
            unsigned *stack_ptr = stack;

            unsigned ei_idx = edge_idxs.get_stack_min_value_idx(edge_idxs.root_idx,&stack_ptr);
            do {
               edge_s &edge = edges[edge_idxs[ei_idx]];

               if (edge.src_vertex_idx == v_idx) {
                  unsigned arr_idx = v_idx*row_b_length + edge.trg_vertex_idx;

                  im[arr_idx/32] |= 0x01 << arr_idx%32;
               }

               ei_idx = edge_idxs.get_stack_next_idx(ei_idx,&stack_ptr,stack);
            } while(ei_idx != c_idx_not_exist);
         }
      } while(++i_idx < vertex_cnt);
   }

   // - initialize bit source positions -
   ui_array_s src_pos;
   src_pos.init_size(row_length);
   src_pos.used = src_pos.size;
   memset(src_pos.data,0xff,src_pos.used*sizeof(unsigned));

   // - initialize bit target positions -
   ui_array_s trg_pos;
   trg_pos.init_size(row_length);
   trg_pos.used = trg_pos.size;

   // - create result flow array -
   ui_array_s flow_array;
   flow_array.init_size(row_b_length);
   flow_array.used = flow_array.size;
   memset(flow_array.data,0,flow_array.used*sizeof(unsigned));

   // - perform flow iterations -
   unsigned i_idx = 0;
   do {
      memset(trg_pos.data,0,trg_pos.used*sizeof(unsigned));

      unsigned si_idx = 0;
      do {
         unsigned s_uint = src_pos[si_idx];
         unsigned b_idx = 0;
         do {
            if (s_uint & (0x01 << b_idx)) {
               
               // - move from state -
               unsigned *t_ptr = trg_pos.data;
               unsigned *t_ptr_end = t_ptr + trg_pos.used;
               unsigned *im_ptr = im.data + ((si_idx << 5) + b_idx)*row_length;
               //unsigned *f_ptr = flow_array.data;
               do {
                  //unsigned b_pos = 0x01;
                  //do {
                  //   *f_ptr += (*im_ptr & b_pos) != 0;
                  //} while(++f_ptr,(b_pos <<= 1) > 0);

                  *t_ptr = *t_ptr | *im_ptr;
               } while(++im_ptr,++t_ptr < t_ptr_end);
            }
         } while(++b_idx < 32);
      } while(++si_idx < row_length);

      src_pos.swap(trg_pos);

      // - add src_pos to flow_array -
      unsigned *s_ptr = src_pos.data;
      unsigned *s_ptr_end = s_ptr + src_pos.used;
      unsigned *f_ptr = flow_array.data;
      do {
         unsigned b_pos = 0x01;
         do {
            *f_ptr += (*s_ptr & b_pos) != 0;
         } while(++f_ptr,(b_pos <<= 1) > 0);
      } while(++s_ptr < s_ptr_end);

   } while(++i_idx < vertex_cnt);

   // - find minimal flow vertex -
   unsigned mf_value = UINT_MAX;
   unsigned mf_idx = c_idx_not_exist;

   unsigned vi_idx = 0;
   do {
      unsigned v_idx = vi_array[vi_idx];
      
      if (flow_array[v_idx] < mf_value) {
         mf_value = flow_array[v_idx];
         mf_idx = v_idx;
      }
   } while(++vi_idx < vertex_cnt);

   flow_array.clear();
   trg_pos.clear();
   src_pos.clear();
   im.clear();

   return mf_idx;
}/*}}}*/

void graph_s::find_isomorphic_subdigraphs(graph_s &a_subgraph,bb_array_s &a_join_vertex_map,ui_arrays_s &a_ret_vertex_maps,ui_arrays_s &a_ret_edge_maps,bool a_create_edge_mapping)
{/*{{{*/
   // - subgraph must be connected -
   // - both graphs must be digraphs -
   // - both graphs vertex_rb_trees and edge_rb_trees must have same value_fps function pointers -

   cassert(vertices.root_idx != c_idx_not_exist && a_subgraph.vertices.root_idx != c_idx_not_exist 
   && type == c_graph_type_digraph && a_subgraph.type == c_graph_type_digraph 
   && vertices.value_fps == a_subgraph.vertices.value_fps && edges.value_fps == a_subgraph.edges.value_fps);

   a_ret_vertex_maps.used = 0;
   a_ret_edge_maps.used = 0;

   // - mappings of subgraph -
   gg_vertex_map_set_s vertex_map;
   vertex_map.init();

   // - stack of mapping ends -
   ui_array_s vm_stack;
   vm_stack.init();

   // - array of subgraphs mapping ends -
   ui_array_s subgraph_array;
   subgraph_array.init();

   // - initial insert of vertices mapping -
   {
      {
         vertex_s &s_vertex = a_subgraph.vertices[a_subgraph.vertices.root_idx];
         edge_idxs_s &edge_idxs = a_subgraph.vertices[a_subgraph.vertices.root_idx].edge_idxs;

         // - get first vertex edge index -
         unsigned first_edge_idx;
         if (edge_idxs.root_idx != c_idx_not_exist) {
            first_edge_idx = edge_idxs.get_min_value_idx(edge_idxs.root_idx);
         }
         else {
            first_edge_idx = c_idx_not_exist;
         }

         ui_array_s found_v_idxs;
         found_v_idxs.init();

         // - find host graph vertices by evaluation -
         vertices.get_idxs(s_vertex,found_v_idxs);

         // - create initial vertex mappings -
         if (found_v_idxs.used != 0) {
            unsigned *v_ptr = found_v_idxs.data;
            unsigned *v_ptr_end = v_ptr + found_v_idxs.used;
            do {
               unsigned vmap_idx = vertex_map.insert_blank();
               vertex_map[vmap_idx].set(a_subgraph.vertices.root_idx,*v_ptr,c_idx_not_exist,1,first_edge_idx,1);
               vm_stack.push(vmap_idx);
            } while(++v_ptr < v_ptr_end);
         }

         found_v_idxs.clear();
      }
   }

   // - process vmaps on stack -
   if (vm_stack.used > 0) {
      do {
         unsigned vmap_idx = vm_stack.pop();
         gg_vertex_map_s *vmap_ptr = &vertex_map[vmap_idx];

         debug_message_5(
            fprintf(stderr,"find_isomorphic_subdigraphs: process: ");
            vertex_map.DEBUG_print_string(vmap_idx);
         );

         {
            vertex_s &s_vertex = a_subgraph.vertices[vmap_ptr->sv_idx];
            vertex_s &m_vertex = vertices[vmap_ptr->mv_idx];

            if (a_join_vertex_map[vmap_ptr->sv_idx]) {
               
               // - process join vertex -
               if (m_vertex.input_degree < s_vertex.input_degree || m_vertex.output_degree < s_vertex.output_degree) {

                  debug_message_5(
                     fprintf(stderr,"find_isomorphic_subdigraphs: failed\n");
                  );

                  vertex_map.release_vmap_string(vmap_idx);
                  continue;
               }
            }
            else {

               // - process standard vertex -
               if (m_vertex.input_degree != s_vertex.input_degree || m_vertex.output_degree != s_vertex.output_degree) {

                  debug_message_5(
                     fprintf(stderr,"find_isomorphic_subdigraphs: failed\n");
                  );

                  vertex_map.release_vmap_string(vmap_idx);
                  continue;
               }
            }
         }

         // - test if subgraph is complete -
         if (vmap_ptr->length == a_subgraph.vertex_cnt) {
            
            debug_message_5(
               fprintf(stderr,"find_isomorphic_subdigraphs: well done\n");
            );

            subgraph_array.push(vmap_idx);
            continue;
         }

         unsigned actual_vm_stack_size = vm_stack.used;
         edge_idxs_s &vmap_sv_edge_idxs = a_subgraph.vertices[vmap_ptr->sv_idx].edge_idxs;
         edge_rb_tree_s &s_edges = a_subgraph.edges;

         // - process unprocessed edge indexes -
         if (vmap_ptr->next_edge_idx != c_idx_not_exist) {
            do {

               // - subgraph edge incident with vmap_ptr->sv_idx vertex -
               edge_s &sv_edge = s_edges[vmap_sv_edge_idxs[vmap_ptr->next_edge_idx]];

               // - next possible processed subgraph vertex -
               unsigned sv_idx = (sv_edge.src_vertex_idx != vmap_ptr->sv_idx)?sv_edge.src_vertex_idx:sv_edge.trg_vertex_idx;

               // - if vertex is not used in mapping string -
               if (vertex_map.get_idx_by_sub_vertex(vmap_idx,sv_idx) == c_idx_not_exist) {
                  
                  // - is subgraph vertex source or target -
                  bool vmap_sv_idx_in_first_edge_position = sv_edge.src_vertex_idx == vmap_ptr->sv_idx;

                  // - indexes of edges incident to sv_idx vertex -
                  edge_idxs_s &sv_edge_idxs = a_subgraph.vertices[sv_idx].edge_idxs;
                  unsigned sv_edge_idxs_first_idx = sv_edge_idxs.get_min_value_idx(sv_edge_idxs.root_idx);

                  // - indexes of edged incident with vmap_ptr->mv_idx vertex -
                  edge_idxs_s &vmap_mv_edge_idxs = vertices[vmap_ptr->mv_idx].edge_idxs;

                  // - cycle throught mother graph vertex edges -
                  if (vmap_mv_edge_idxs.root_idx != c_idx_not_exist) {
                     unsigned stack[vmap_mv_edge_idxs.get_descent_stack_size()];
                     unsigned *stack_ptr = stack;

                     unsigned me_edge_idx_idx = vmap_mv_edge_idxs.get_stack_min_value_idx(vmap_mv_edge_idxs.root_idx,&stack_ptr);
                     do {
                        
                        // - mother graph edge incident with vmap_ptr->mv_idx vertex -
                        edge_s &mv_edge = edges[vmap_mv_edge_idxs[me_edge_idx_idx]];

                        // - test orientation of edge in vertex + edge evaluation test -
                        if (vmap_ptr->mv_idx == (vmap_sv_idx_in_first_edge_position?mv_edge.src_vertex_idx:mv_edge.trg_vertex_idx)
                           && edges.ef_compare(sv_edge.value,mv_edge.value) == 0) {

                           // - next possible processed mother graph vertex -
                           unsigned mv_idx = (mv_edge.src_vertex_idx != vmap_ptr->mv_idx)?mv_edge.src_vertex_idx:mv_edge.trg_vertex_idx;

                           // - if vertex is not used in mapping string -
                           if (vertex_map.get_idx_by_mother_vertex(vmap_idx,mv_idx) == c_idx_not_exist) {

                              vertex_s &s_vertex = a_subgraph.vertices[sv_idx];
                              vertex_s &m_vertex = vertices[mv_idx];

                              // -  testing evaluation of vertices -
                              if (vertices.vf_compare(s_vertex.value,m_vertex.value) == 0) {

                                 // - insert new mapping to string -
                                 unsigned new_vmap_idx = vertex_map.insert_blank();

                                 // - refresh vmap_ptr -
                                 vmap_ptr = &vertex_map[vmap_idx];

                                 vertex_map[new_vmap_idx].set(sv_idx,mv_idx,vmap_idx,1,sv_edge_idxs_first_idx,vmap_ptr->length + 1);
                                 vm_stack.push(new_vmap_idx);

                                 debug_message_5(
                                    fprintf(stderr,"find_isomorphic_subdigraphs: normal mapping add: ");
                                    vertex_map.DEBUG_print_string(new_vmap_idx);
                                 );

                                 // - increase reference count of previous mapping in mapping string -
                                 vmap_ptr->reference_cnt++;
                              }
                           }
                        }

                        // - move to next diferend edge (ignore multi edges with same value, source vertex and target vertex) -
                        unsigned old_edge_idx = vmap_mv_edge_idxs[me_edge_idx_idx];
                        do {
                           me_edge_idx_idx = vmap_mv_edge_idxs.get_stack_next_idx(me_edge_idx_idx,&stack_ptr,stack);
                        } while(me_edge_idx_idx != c_idx_not_exist && vmap_mv_edge_idxs.compare_edges(edges[old_edge_idx],edges[vmap_mv_edge_idxs[me_edge_idx_idx]]) == 0);
                     } while(me_edge_idx_idx != c_idx_not_exist);
                  }
               }

               // - move to next diferend edge (ignore multi edges with same value, source vertex and target vertex) -
               unsigned old_edge_idx = vmap_sv_edge_idxs[vmap_ptr->next_edge_idx];
               do {
                  vmap_ptr->next_edge_idx = vmap_sv_edge_idxs.get_next_idx(vmap_ptr->next_edge_idx);
               } while(vmap_ptr->next_edge_idx != c_idx_not_exist && vmap_sv_edge_idxs.compare_edges(s_edges[old_edge_idx],s_edges[vmap_sv_edge_idxs[vmap_ptr->next_edge_idx]]) == 0);

               if (actual_vm_stack_size != vm_stack.used) {
                  break;
               }

            } while(vmap_ptr->next_edge_idx != c_idx_not_exist);
         }

         if (actual_vm_stack_size == vm_stack.used) {
            
            // - search for mappings in map string with "unused" edges -
            unsigned m_idx = vertex_map.get_map_with_unused_edges(a_subgraph.vertices.used,vmap_idx);
            if (m_idx == c_idx_not_exist) {
               vertex_map.release_vmap_string(vmap_idx);

               debug_message_5(
                  fprintf(stderr,"find_isomorphic_subdigraphs: failed backsearch\n");
               );
            }
            else {

               // - insert new map by copy from m_idx with increased next_edge_idx -
               unsigned new_vmap_idx = vertex_map.insert_blank();

               // - refresh vmap_ptr -
               vmap_ptr = &vertex_map[vmap_idx];

               // - retrieve mapping to duplicate -
               gg_vertex_map_s &split_vmap = vertex_map[m_idx];

               vertex_map[new_vmap_idx].set(split_vmap.sv_idx,split_vmap.mv_idx,vmap_idx,1,split_vmap.next_edge_idx,vmap_ptr->length);
               vm_stack.push(new_vmap_idx);

               // - increase vmap_ptr reference count -
               vmap_ptr->reference_cnt++;

               debug_message_5(
                  fprintf(stderr,"find_isomorphic_subdigraphs: backsearch mapping add: ");
                  vertex_map.DEBUG_print_string(new_vmap_idx);
               );
            }
        }
      } while(vm_stack.used > 0);
   }

   // - creation of ret_vertex_maps from mapping string -
   if (subgraph_array.used != 0) {

      // - basic vertex map arrays -
      ui_arrays_s basic_vertex_maps;
      basic_vertex_maps.init();

      basic_vertex_maps.copy_resize(subgraph_array.used);
      basic_vertex_maps.used = subgraph_array.used;

      unsigned *sa_ptr = subgraph_array.data;
      unsigned *sa_ptr_end = sa_ptr + subgraph_array.used;
      unsigned rvm_idx = 0;
      do {
	 ui_array_s &basic_vertex_map = basic_vertex_maps[rvm_idx];

	 basic_vertex_map.copy_resize(a_subgraph.vertices.used);
	 basic_vertex_map.used = a_subgraph.vertices.used;

         vertex_map.retrieve_mapping_sub_to_mother(*sa_ptr,basic_vertex_map);
      } while(++rvm_idx,++sa_ptr < sa_ptr_end);

      debug_message_5
      (/*{{{*/
         fprintf(stderr,"find_isomorphic_subdigraphs: result vertex mappings: \nmapping: ");

         if (basic_vertex_maps.used > 0) {
            fprintf(stderr,"                ");

            unsigned idx = 0;
            do {
               fprintf(stderr,"%4u ",idx);
            } while(++idx < basic_vertex_maps.used);
            fputc('\n',stderr);

            // - reference on subgraph vertices -
            vertex_rb_tree_s &s_vertices = a_subgraph.vertices;

            unsigned stack[s_vertices.get_descent_stack_size()];
            unsigned *stack_ptr = stack;

            unsigned v_idx = s_vertices.get_stack_min_value_idx(s_vertices.root_idx,&stack_ptr);
            do {
               fprintf(stderr,"subgraph vertex: %4u -> ",v_idx);

               // - cycle throught all founded vertex maps -
               ui_array_s *ptr = basic_vertex_maps.data;
               ui_array_s *ptr_end = ptr + basic_vertex_maps.used;
               do {
                  fprintf(stderr,"%4u|",(*ptr)[v_idx]);
               } while(++ptr < ptr_end);
               fputc('\n',stderr);

               v_idx = s_vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
            } while(v_idx != c_idx_not_exist);
         }
         else {
            fprintf(stderr,"not found\n");
         }

      );/*}}}*/

      // - search and test edges in mother graph -
      if (a_subgraph.edge_cnt > 0) {
         
         // - create array of same edges -
         ui_arrays_s edge_arrays;
         edge_arrays.init();

         {
            // - create reference on subgraph edges -
            vertex_rb_tree_s &s_vertices = a_subgraph.vertices;
            edge_rb_tree_s &s_edges = a_subgraph.edges;

            unsigned stack[s_edges.get_descent_stack_size()];
            unsigned *stack_ptr = stack;

            unsigned e_idx = s_edges.get_stack_min_value_idx(s_edges.root_idx,&stack_ptr);
            do {

               // - find all same edges -
               //s_edges.__get_idxs(s_edges[e_idx],tmp_edge_idxs,edge_src_trg_vertex_compare);

               // - push new edge array -
               edge_arrays.push_blank();

               // - optimized find all matching edges -
               unsigned edge_cnt = 0;
               s_vertices[s_edges[e_idx].src_vertex_idx].edge_idxs.get_edge_idxs_by_edge(s_edges[e_idx],edge_arrays.last(),edge_cnt);

               // - find next differend edge -
               edge_s &old_edge = s_edges[e_idx];
               do {
                  e_idx = s_edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
               } while(e_idx != c_idx_not_exist && (s_edges.__compare_value(old_edge,s_edges[e_idx]) == 0 && edge_src_trg_vertex_compare(old_edge,s_edges[e_idx])));
            } while(e_idx != c_idx_not_exist);
         }

         ui_array_s tmp_edge_idxs;
         tmp_edge_idxs.init();

         // - create graph edge mapping arrays -
         ui_arrays_array_s graph_edge_mapping_arrays;
         graph_edge_mapping_arrays.init_size(subgraph_array.used);

         ui_arrays_s graph_edge_arrays;
         graph_edge_arrays.init();

         // - test vertex mapping, and create final edge mapping -
         unsigned sg_idx = 0;
         do {
            
            // - insert next graph edge mapping array -
            graph_edge_mapping_arrays.push_blank();
            ui_arrays_s &graph_edge_mapping_array = graph_edge_mapping_arrays.last();
            graph_edge_mapping_array.used = 0;

            ui_array_s &ret_vertex_map = basic_vertex_maps[sg_idx];
            
            ui_array_s *ea_ptr = edge_arrays.data;
            ui_array_s *ea_ptr_end = ea_ptr + edge_arrays.used;
            do {

               // - clear graph_edge_arrays -
               graph_edge_arrays.used = 0;

               // - get reference on first edge of set -
               edge_s &s_edge = a_subgraph.edges[(*ea_ptr)[0]];

               edge_s m_edge;
               m_edge.set(s_edge.value,ret_vertex_map[s_edge.src_vertex_idx],ret_vertex_map[s_edge.trg_vertex_idx]);

               // - find all matching edges -
               //edges.__get_idxs(m_edge,tmp_edge_idxs,edge_src_trg_vertex_compare);

               {
                  tmp_edge_idxs.used = 0;
                  unsigned edge_cnt = 0;

                  vertices[m_edge.src_vertex_idx].edge_idxs.get_edge_idxs_by_edge(m_edge,tmp_edge_idxs,edge_cnt);
               }

               if (tmp_edge_idxs.used >= ea_ptr->used) {
                  debug_message_5
                  (/*{{{*/
                     fprintf(stderr,"find_isomorphic_subdigraphs: edge_mapping: (%4u",tmp_edge_idxs[0]);

                     if (tmp_edge_idxs.used > 1) {
                        unsigned *ptr = tmp_edge_idxs.data + 1;
                        unsigned *ptr_end = tmp_edge_idxs.data + tmp_edge_idxs.used;
                        do {
                           fprintf(stderr,",%4u",*ptr);
                        } while(++ptr < ptr_end);
                     }

                     fprintf(stderr,") -> (%u",(*ea_ptr)[0]);

                     if (ea_ptr->used > 1) {
                        unsigned *ptr = ea_ptr->data + 1;
                        unsigned *ptr_end = ea_ptr->data + ea_ptr->used;
                        do {
                           fprintf(stderr,",%u",*ptr);
                        } while(++ptr < ptr_end);
                     }

                     fprintf(stderr,")\n");
                  );/*}}}*/

                  unsigned p_map[ea_ptr->used];
                  unsigned p_iter[ea_ptr->used];
                  unsigned perm[tmp_edge_idxs.used];

                  memset(p_map,c_idx_not_exist,ea_ptr->used*sizeof(unsigned));
                  memset(p_iter,0,ea_ptr->used*sizeof(unsigned));
                  memset(perm,c_idx_not_exist,tmp_edge_idxs.used*sizeof(unsigned));

                  // - find all posible permutations of matching edges mapping -
                  unsigned e_idx = 0;
                  do {
                     
                     // - clear mapping from actual permutaion -
                     if (p_map[e_idx] != c_idx_not_exist) {
                        perm[p_map[e_idx]] = c_idx_not_exist;
                        p_map[e_idx] = c_idx_not_exist;
                     }

                     // - find next possible position -
                     while(p_iter[e_idx] < tmp_edge_idxs.used && perm[p_iter[e_idx]] != c_idx_not_exist) {
                        p_iter[e_idx]++;
                     }

                     // - if next position is OK -
                     if (p_iter[e_idx] >= tmp_edge_idxs.used) {
                        if (e_idx == 0) {
                           break;
                        }
                        else {
                           p_iter[e_idx] = 0;
                           e_idx--;
                           continue;
                        }
                     }

                     p_map[e_idx] = p_iter[e_idx];
                     perm[p_iter[e_idx]] = e_idx;

                     p_iter[e_idx]++;

                     if (++e_idx >= ea_ptr->used) {
                        debug_message_5
                        (/*{{{*/
                           fprintf(stderr,"find_isomorphic_subdigraphs: permutation:  |");
                           
                           unsigned *ptr = perm;
                           unsigned *ptr_end = ptr + tmp_edge_idxs.used;
                           do {
                              if (*ptr == c_idx_not_exist) {
                                 fprintf(stderr,"   X|");
                              }
                              else {
                                 fprintf(stderr,"%4u|",(*ea_ptr)[*ptr]);
                              }
                           } while(++ptr < ptr_end);

                           fputc('\n',stderr);
                        );/*}}}*/

                        // - insert edge array to graph edge arrays -
                        graph_edge_arrays.push_blank();
                        graph_edge_arrays.last().set(ea_ptr->used,p_map);

                        p_iter[e_idx] = 0;
                        e_idx--;
                     }
                  } while(1);
               }
               else {
                  debug_message_5(fprintf(stderr,"find_isomorphic_subdigraphs: cannot find subgraph edges mapping -> cancel vertex mapping\n"););

                  graph_edge_mapping_array.used = 0;
                  break;
               }

               // - create (multiply mappings of edges), result: final mapping of edges on given vertex mapping -
               if (graph_edge_mapping_array.used == 0) {
                  graph_edge_mapping_array.copy_resize(graph_edge_arrays.used);
                  graph_edge_mapping_array.used = graph_edge_arrays.used;

                  ui_array_s *m_ptr = graph_edge_arrays.data;
                  ui_array_s *m_ptr_end = m_ptr + graph_edge_arrays.used;
                  ui_array_s *gem_ptr = graph_edge_mapping_array.data;
                  do {
                     gem_ptr->copy_resize(a_subgraph.edges.used);
                     gem_ptr->used = a_subgraph.edges.used;

                     unsigned idx = 0;
                     do {
                        gem_ptr->data[ea_ptr->data[idx]] = tmp_edge_idxs[m_ptr->data[idx]];
                     } while(++idx < ea_ptr->used);
                  } while(++gem_ptr,++m_ptr < m_ptr_end);
               }
               else {
                  unsigned old_graph_edge_mapping_array_used = graph_edge_mapping_array.used;

                  if (graph_edge_arrays.used > 1) {
                     graph_edge_mapping_array.copy_resize(graph_edge_arrays.used*graph_edge_mapping_array.used);
                     graph_edge_mapping_array.used = graph_edge_arrays.used*graph_edge_mapping_array.used;

                     unsigned idx = old_graph_edge_mapping_array_used;
                     do {
                        graph_edge_mapping_array[idx] = graph_edge_mapping_array[idx%old_graph_edge_mapping_array_used];
                     } while(++idx < graph_edge_mapping_array.used);
                  }

                  unsigned idx = 0;
                  do {
                     ui_array_s &edge_map = graph_edge_mapping_array[idx];
                     ui_array_s &graph_edge_array = graph_edge_arrays[idx/old_graph_edge_mapping_array_used];

                     unsigned idx = 0;
                     do {
                        edge_map[ea_ptr->data[idx]] = tmp_edge_idxs[graph_edge_array[idx]];
                     } while(++idx < ea_ptr->used);
                  } while(++idx < graph_edge_mapping_array.used);
               }
            } while(++ea_ptr < ea_ptr_end);
         } while(++sg_idx < subgraph_array.used);

         graph_edge_arrays.clear();

         // - create final vertex and edge mappings -
         if (a_create_edge_mapping) {
            ui_arrays_s *aa_ptr = graph_edge_mapping_arrays.data;
            ui_arrays_s *aa_ptr_end = aa_ptr + graph_edge_mapping_arrays.used;
            ui_array_s *vm_ptr = basic_vertex_maps.data;
            do {

               // - if exist some edge mapping for this vertex map -
               if (aa_ptr->used != 0) {
                  ui_array_s *a_ptr = aa_ptr->data;
                  ui_array_s *a_ptr_end = a_ptr + aa_ptr->used;
                  do {
                     
                     // - copy vertex map -
                     a_ret_vertex_maps.push(*vm_ptr);

                     // - swap edge map -
                     a_ret_edge_maps.push_blank();
                     a_ret_edge_maps.last().swap(*a_ptr);
                  } while(++a_ptr < a_ptr_end);
               }
            } while(++vm_ptr,++aa_ptr < aa_ptr_end);
         }
         else {
            ui_arrays_s *aa_ptr = graph_edge_mapping_arrays.data;
            ui_arrays_s *aa_ptr_end = aa_ptr + graph_edge_mapping_arrays.used;
            ui_array_s *vm_ptr = basic_vertex_maps.data;
            do {
               
               // - if exist some edge mapping for vertex map -
               if (aa_ptr->used != 0) {
                  a_ret_vertex_maps.push(*vm_ptr);
               }
            } while(++vm_ptr,++aa_ptr < aa_ptr_end);
         }

         graph_edge_mapping_arrays.clear();

         tmp_edge_idxs.clear();
         edge_arrays.clear();
      }
      else {
         a_ret_vertex_maps.swap(basic_vertex_maps);
      }

      basic_vertex_maps.clear();
   }

   subgraph_array.clear();
   vm_stack.clear();
   vertex_map.clear();
}/*}}}*/

bool graph_s::load_ARG_database_graph(const char *a_file_name)
{/*{{{*/
   cassert(a_file_name != NULL);

   unsigned labeled = false;

   // - find file name in path -
   char *ls_ptr = (char *)a_file_name;

   {
      char *p_ptr = ls_ptr;
      char *p_ptr_end = p_ptr + strlen(a_file_name);
      do {
         if (*p_ptr == '/') {
            ls_ptr = p_ptr + 1;
         }
      } while(++p_ptr < p_ptr_end);
   }

   // - test if graph is labeled by file name -
   if (strstr(ls_ptr,"mcs") == ls_ptr) {
      labeled = true;
   }

   // - clear target graph -
   clear();

   // - set graph type to digraph -
   type = c_graph_type_digraph;

   // - if ARG source is directed graph set evaluation type to integer  -
   if (labeled) {
      vertices.value_fps = (void **)graph_int_functions;
      edges.value_fps = (void **)graph_int_functions;
   }

   FILE *f = fopen(a_file_name,"rb");
   if (f == NULL) {
      return false;
   }

   fseek(f,0,SEEK_END);
   unsigned buffer_size = ftell(f);
   fseek(f,0,SEEK_SET);

   unsigned char buffer[buffer_size];
   fread(buffer,buffer_size,1,f);
   fclose(f);

   unsigned char *b_ptr = buffer;

   // - reading file buffer -
   unsigned node_cnt = *(unsigned short *)b_ptr;b_ptr += sizeof(unsigned short);
   debug_message_6(fprintf(stderr,"load_ARG_database_graph: node_cnt: %u\n",node_cnt));

   if (!labeled) {

      ui_array_s node_idxs;
      node_idxs.init_size(node_cnt);
      node_idxs.used = node_cnt;

      // - insert nodes into graph -
      //unsigned node_idxs[node_cnt];
      unsigned n_idx = 0;
      do {
         node_idxs[n_idx] = insert_vertex(NULL);
      } while(++n_idx < node_cnt); 

      // - load graph edges -
      n_idx = 0;
      do {
         unsigned n_edge_cnt = *(unsigned short *)b_ptr;b_ptr += sizeof(unsigned short);

         debug_message_6
         (/*{{{*/
            fprintf(stderr,"load_ARG_database_graph: node %u: edge cnt: %u",n_idx,n_edge_cnt);
            
            if (n_edge_cnt > 0) {
               fprintf(stderr," -> ");

               unsigned char *old_b_ptr = b_ptr;
               unsigned e_idx = 0;
               do {
                  unsigned edge_trg = *(unsigned short *)b_ptr;b_ptr += sizeof(unsigned short);
                  fprintf(stderr,"%u, ",edge_trg);
               } while(++e_idx < n_edge_cnt);
               fputc('\n',stderr);

               b_ptr = old_b_ptr;
            }
            else {
               fputc('\n',stderr);
            }
         );/*}}}*/

         if (n_edge_cnt > 0) {
            unsigned e_idx = 0;
            do {
               unsigned edge_trg = *(unsigned short *)b_ptr;b_ptr += sizeof(unsigned short);
               insert_edge(node_idxs[n_idx],node_idxs[edge_trg],NULL);
            } while(++e_idx < n_edge_cnt);
         }
      } while(++n_idx < node_cnt);

      node_idxs.clear();
   }
   else {
      
      // - insert nodes into graph -
      unsigned node_idxs[node_cnt];
      unsigned n_idx = 0;
      do {
         int node_label = *(unsigned short *)b_ptr;b_ptr += sizeof(unsigned short);
         //node_idxs[n_idx] = insert_vertex((void *)node_label);

         // FIXME
         //node_idxs[n_idx] = insert_vertex((void *)0);
         node_idxs[n_idx] = insert_vertex((void *)(node_label % 5));
         //node_idxs[n_idx] = insert_vertex((void *)(rand() % 5));
      } while(++n_idx < node_cnt);

      // - load graph edges -
      n_idx = 0;
      do {
         unsigned n_edge_cnt = *(unsigned short *)b_ptr;b_ptr += sizeof(unsigned short);

         if (n_edge_cnt > 0) {
            unsigned e_idx = 0;
            do {
               unsigned edge_trg = *(unsigned short *)b_ptr;b_ptr += sizeof(unsigned short);
               unsigned edge_label = *(unsigned short *)b_ptr;b_ptr += sizeof(unsigned short);
               //insert_edge(node_idxs[n_idx],node_idxs[edge_trg],(void *)edge_label);

               // FIXME
               //insert_edge(node_idxs[n_idx],node_idxs[edge_trg],(void *)0);
               insert_edge(node_idxs[n_idx],node_idxs[edge_trg],(void *)(edge_label % 5));
               //insert_edge(node_idxs[n_idx],node_idxs[edge_trg],(void *)(rand() % 5));
            } while(++e_idx < n_edge_cnt);
         }
      } while(++n_idx < node_cnt);
   }

   return true;
}/*}}}*/

bool graph_s::save_ARG_database_graph(const char *a_file_name)
{/*{{{*/
   cassert(vertices.value_fps == graph_blank_functions && edges.value_fps == graph_blank_functions && vertex_cnt <= USHRT_MAX);

   unsigned buffer_size = sizeof(unsigned short) + vertex_cnt*sizeof(unsigned short) + edge_cnt*sizeof(unsigned short);
   unsigned char buffer[buffer_size];
   unsigned char *b_ptr = buffer;

   // - store count of vertices -
   *(unsigned short *)b_ptr = (unsigned short)vertex_cnt; b_ptr += sizeof(unsigned short);

   // - vertex map array -
   unsigned short vertex_map[vertices.used];

   {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      // - create vertex_map -
      unsigned short vm_value = 0;
      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vertex_map[v_idx] = vm_value++;
         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      // - process vertices -
      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vertex_s &vertex = vertices[v_idx];
         edge_idxs_s &edge_idxs = vertex.edge_idxs;

         *(unsigned short *)b_ptr = (unsigned short)vertex.output_degree; b_ptr += sizeof(unsigned short);

         // - process vertex edge indexes -
         if (edge_idxs.root_idx != c_idx_not_exist) {
            unsigned ei_stack[edge_idxs.get_descent_stack_size()];
            unsigned *ei_stack_ptr = ei_stack;

            // - process edges -
            unsigned ei_idx = edge_idxs.get_stack_min_value_idx(edge_idxs.root_idx,&ei_stack_ptr);
            do {
               edge_s &edge = edges[edge_idxs[ei_idx]];

               if (edge.src_vertex_idx == v_idx) {
                  *(unsigned short *)b_ptr = vertex_map[edge.trg_vertex_idx]; b_ptr += sizeof(unsigned short);
               }

               ei_idx = edge_idxs.get_stack_next_idx(ei_idx,&ei_stack_ptr,ei_stack);
            } while(ei_idx != c_idx_not_exist);
         }

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   cassert(b_ptr == buffer + buffer_size);

   // - save buffer to target file -
   FILE *f = fopen(a_file_name,"wb");
   cassert(f != NULL);
   fwrite(buffer,buffer_size,1,f);
   fclose(f);

   return true;
}/*}}}*/

void graph_s::generate_connected_graph(unsigned a_vertex_cnt,unsigned a_edge_cnt,unsigned a_graph_type)
{/*{{{*/
   cassert(a_vertex_cnt > 1 && a_edge_cnt >= a_vertex_cnt - 1);

   clear();

   // - set graph type -
   type = a_graph_type;

   // - create vertex map array -
   unsigned vertex_map[a_vertex_cnt];

   // - insert vertices to graph -
   {
      unsigned idx = 0;
      do {
         vertex_map[idx] = insert_vertex(NULL);
      } while(++idx < a_vertex_cnt);
   }

   // - no loops and multiple edges -
   bb_array_s imat;
   imat.init_size(vertices.used*vertices.used);
   imat.used = imat.size;
   memset(imat.data,false,imat.used*sizeof(bool));

   // - create unused vertices array -
   bool used_vertices[a_vertex_cnt];
   memset(used_vertices,false,a_vertex_cnt*sizeof(bool));

   // - create graph spanning tree -
   unsigned unused_vertex_cnt = a_vertex_cnt;
   unsigned v_idx = rand()%(unused_vertex_cnt--);
   used_vertices[v_idx] = true;

   do {
      unsigned last_v_idx = v_idx;
      v_idx = rand()%(unused_vertex_cnt--);

      unsigned idx = 0;
      do {
         if (!used_vertices[idx]) {
            if (v_idx-- == 0) {
               v_idx = idx;
               break;
            }
         }
      } while(++idx);

      used_vertices[v_idx] = true;

      unsigned sv_idx = vertex_map[last_v_idx];
      unsigned tv_idx = vertex_map[v_idx];

      // - insert edge between vertices -
      insert_edge(sv_idx,tv_idx,NULL);
      imat[sv_idx*vertices.used + tv_idx] = true;

   } while(unused_vertex_cnt > 0);

   unsigned edge_cnt = a_edge_cnt - (a_vertex_cnt - 1);
   if (edge_cnt > 0) {
      do {

         // - insert edge between random vertices -
         unsigned sv_idx = vertex_map[rand()%a_vertex_cnt];
         unsigned tv_idx = vertex_map[rand()%a_vertex_cnt];

         if (!imat[sv_idx*vertices.used + tv_idx]) {
            insert_edge(sv_idx,tv_idx,NULL);
            imat[sv_idx*vertices.used + tv_idx] = true;
            --edge_cnt;
         }
      } while(edge_cnt > 0);
   }

   imat.clear();
}/*}}}*/

void graph_s::create_isomorphic_graph(graph_s &a_graph)
{/*{{{*/
   clear();

   // - copy graph properties -
   type = a_graph.type;
   vertices.value_fps = a_graph.vertices.value_fps;
   edges.value_fps = a_graph.edges.value_fps;

   if (a_graph.vertices.root_idx != c_idx_not_exist) {

      // - map from idx 0 .. a_graph.vertex_cnt to a_graph.vertices -
      unsigned vertex_map[a_graph.vertex_cnt];

      // - fill vertex map array -
      {
         vertex_rb_tree_s &g_vertices = a_graph.vertices;

         unsigned stack[g_vertices.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned idx = 0;
         unsigned v_idx = g_vertices.get_stack_min_value_idx(g_vertices.root_idx,&stack_ptr);
         do {
            vertex_map[idx] = v_idx;
            v_idx = g_vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
         } while(++idx,v_idx != c_idx_not_exist);
      }

      // - map from a_graph.vertices to vertices -
      unsigned gn_vertex_map[a_graph.vertices.used];

      // - insert vertices to new graph -
      {
         unsigned g_vertex_cnt = a_graph.vertex_cnt;
         do {
            unsigned v_idx = rand()%(g_vertex_cnt--);

            unsigned idx = 0;
            do {
               if (vertex_map[idx] != c_idx_not_exist) {
                  if (v_idx-- == 0) {
                     v_idx = idx;
                     break;
                  }
               }
            } while(++idx);

            // - index and reference on a_graph vertex -
            unsigned g_vertex_idx = vertex_map[v_idx];
            vertex_s &g_vertex = a_graph.vertices[g_vertex_idx];

            // - insert new vertex to isomorphic graph -
            gn_vertex_map[g_vertex_idx] = insert_vertex(g_vertex.value);
            vertex_map[v_idx] = c_idx_not_exist;

         } while(g_vertex_cnt > 0);
      }

      // - insert edges to new graph -
      if (a_graph.edges.root_idx != c_idx_not_exist) {
         edge_rb_tree_s &g_edges = a_graph.edges;
         
         unsigned stack[g_edges.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned e_idx = g_edges.get_stack_min_value_idx(g_edges.root_idx,&stack_ptr);
         do {
            edge_s &g_edge = g_edges[e_idx];

            // - insert edge into isomorphic graph -
            insert_edge(gn_vertex_map[g_edge.src_vertex_idx],gn_vertex_map[g_edge.trg_vertex_idx],g_edge.value);

            e_idx = g_edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
         } while(e_idx != c_idx_not_exist);
      }
   }

}/*}}}*/

void graph_s::DEBUG_print_dot_code_params(FILE *a_file,const char *a_header,const char *a_node_prefix,const char *a_indent,void **a_vertices_value_fps,void **a_edges_value_fps)
{/*{{{*/

   void **store_vertices_value_fps = vertices.value_fps;
   vertices.value_fps = a_vertices_value_fps;

   void **store_edges_value_fps = edges.value_fps;
   edges.value_fps = a_edges_value_fps;

   fprintf(a_file,a_header);

   string_s value_str;
   value_str.init();

   // - traverse throught vertices and print their values to output -
   if (vertices.root_idx != c_idx_not_exist) {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vertices.vf_get_string(vertices[v_idx].value,value_str);

         if (value_str.size > 1) {
            fprintf(a_file,"%s   %snode_%u [ label = %s ]\n",a_indent,a_node_prefix,v_idx,value_str.data);
         }
         else {
            fprintf(a_file,"%s   %snode_%u [ label = %u ]\n",a_indent,a_node_prefix,v_idx,v_idx);
         }

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   // - traverse throught edges and print their values to output -
   if (edges.root_idx != c_idx_not_exist) {
      unsigned stack[edges.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned e_idx = edges.get_stack_min_value_idx(edges.root_idx,&stack_ptr);
      do {
         edge_s &edge = edges[e_idx];

         edges.ef_get_string(edge.value,value_str);

         if (value_str.size > 1) {
	    fprintf(a_file,"%s   /* %snode_%u_%snode_%u */ %snode_%u %s %snode_%u [ label = %s ]\n",a_indent,a_node_prefix,edge.src_vertex_idx,a_node_prefix,edge.trg_vertex_idx,a_node_prefix,edge.src_vertex_idx,c_graph_type_dot_codes[type],a_node_prefix,edge.trg_vertex_idx,value_str.data);
         }
         else {
	    fprintf(a_file,"%s   /* %snode_%u_%snode_%u */ %snode_%u %s %snode_%u [ label = %u ]\n",a_indent,a_node_prefix,edge.src_vertex_idx,a_node_prefix,edge.trg_vertex_idx,a_node_prefix,edge.src_vertex_idx,c_graph_type_dot_codes[type],a_node_prefix,edge.trg_vertex_idx,e_idx);
         }

         e_idx = edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
      } while(e_idx != c_idx_not_exist);
   }

   value_str.clear();

   fprintf(a_file,
"%s}\n"
   ,a_indent);

   vertices.value_fps = store_vertices_value_fps;
   edges.value_fps = store_edges_value_fps;

}/*}}}*/

void graph_s::DEBUG_print_dot_code(FILE *a_file)
{/*{{{*/

   switch (type) {
   case c_graph_type_graph:
      fprintf(a_file,
"graph G {\n"
      );
      break;
   case c_graph_type_digraph:
      fprintf(a_file,
"digraph G {\n"
      );
      break;
   default:
      cassert(0);
   }

   fprintf(a_file,
"   rankdir = LR\n"
"   node [ shape = ellipse ]\n"
"\n"
   );

   DEBUG_print_dot_code_params(a_file,"","","",vertices.value_fps,edges.value_fps);
}/*}}}*/

void graph_s::DEBUG_print_nauty_dreadnaut_code(FILE *a_file)
{/*{{{*/
   cassert(vertices.root_idx != c_idx_not_exist);

   fprintf(a_file,type == c_graph_type_graph?"-d\n":"d\n");
   fprintf(a_file,"n = %u\ng\n",vertex_cnt);

   // - mapping of vertex indexes to (0 ... n-1) indexes -
   unsigned vmap[vertices.used];

   // - set vertex map (vmap) -
   unsigned stack[vertices.get_descent_stack_size()];
   unsigned *stack_ptr = stack;

   unsigned vm_idx = 0;
   unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
   do {
      vmap[v_idx] = vm_idx;
      v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
   } while(++vm_idx,v_idx != c_idx_not_exist);

   // - print vertices and its edges -
   stack_ptr = stack;
   v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
   do {
      edge_idxs_s &edge_idxs = vertices[v_idx].edge_idxs;
      if (edge_idxs.root_idx != c_idx_not_exist) {
         unsigned ei_stack[edge_idxs.get_descent_stack_size()];
         unsigned *ei_stack_ptr = ei_stack;

         unsigned ei_idx = edge_idxs.get_stack_min_value_idx(edge_idxs.root_idx,&ei_stack_ptr);
         do {
            edge_s &edge = edges[edge_idxs[ei_idx]];

            // - print edge  -
            if (edge.src_vertex_idx == v_idx) {
               fprintf(a_file,"%u\n",vmap[edge.trg_vertex_idx]);
            }

            ei_idx = edge_idxs.get_stack_next_idx(ei_idx,&ei_stack_ptr,ei_stack);
         } while(ei_idx != c_idx_not_exist);
      }

      fprintf(a_file,";\n");

      v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
   } while(v_idx != c_idx_not_exist);
}/*}}}*/

void graph_s::DEBUG_print_dot_code_subgraph(FILE *a_file,graph_s &a_subgraph,ui_array_s &a_vertex_map,ui_array_s &a_edge_map)
{/*{{{*/
   cassert(type = a_subgraph.type && a_vertex_map.used >= a_subgraph.vertices.used && a_edge_map.used >= a_subgraph.edges.used);

   switch (type) {
   case c_graph_type_graph:
      fprintf(a_file,
"graph G {\n"
      );
      break;
   case c_graph_type_digraph:
      fprintf(a_file,
"digraph G {\n"
      );
      break;
   default:
      cassert(0);
   }

   fprintf(a_file,
"   rankdir = LR\n"
"   node [ shape = ellipse ]\n"
"\n"
   );

//#define print_dot_code_subgraph_LABEL_EVAL

#ifdef print_dot_code_subgraph_LABEL_EVAL
   string_s value_str;
   value_str.init();
#endif

   if (vertices.root_idx != c_idx_not_exist) {
      unsigned hsv_map[vertices.used];
      memset(hsv_map,c_idx_not_exist,vertices.used*sizeof(unsigned));

      // - get reference on subgraph vertices -
      vertex_rb_tree_s &s_vertices = a_subgraph.vertices;

      // - cycle throught subgraph vertices, and set hsv_map -
      if (s_vertices.root_idx != c_idx_not_exist) {
         unsigned stack[s_vertices.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned v_idx = s_vertices.get_stack_min_value_idx(s_vertices.root_idx,&stack_ptr);
         do {
            hsv_map[a_vertex_map[v_idx]] = v_idx;
            v_idx = s_vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
         } while(v_idx != c_idx_not_exist);
      }

      // - cycle throught host graph vertices and print their description -
      {
         unsigned stack[vertices.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
         do {
            
            if (hsv_map[v_idx] != c_idx_not_exist) {
               fprintf(a_file,"   node_%u [ color = red ",v_idx);
            }
            else {
               fprintf(a_file,"   node_%u [ ",v_idx);
            }

#ifdef print_dot_code_subgraph_LABEL_EVAL
            // - get host graph vertex value string -
            vertices.vf_get_string(vertices[v_idx].value,value_str);

            if (value_str.size > 1) {
               fprintf(a_file,"label = %s ]\n",value_str.data);
            }
            else {
               fprintf(a_file,"]\n");
            }
#else
            if (hsv_map[v_idx] != c_idx_not_exist) {
               fprintf(a_file,"label = \"%u > %u\" ]\n",hsv_map[v_idx],v_idx);
            }
            else {
               fprintf(a_file,"label = \"%u\" ]\n",v_idx);
            }
#endif

            v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
         } while(v_idx != c_idx_not_exist);
      }
   }

   if (edges.root_idx != c_idx_not_exist) {
      unsigned hse_map[edges.used];
      memset(hse_map,c_idx_not_exist,edges.used*sizeof(unsigned));

      // - get reference on subgraph edges -
      edge_rb_tree_s &s_edges = a_subgraph.edges;

      // - cycle throught subgraph edges, and set hse_map -
      if (s_edges.root_idx != c_idx_not_exist) {
         unsigned stack[s_edges.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned e_idx = s_edges.get_stack_min_value_idx(s_edges.root_idx,&stack_ptr);
         do {
            hse_map[a_edge_map[e_idx]] = e_idx;
            e_idx = s_edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
         } while(e_idx != c_idx_not_exist);
      }

      // - cycle throught host graph edges nad print their description -
      {
         unsigned stack[edges.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned e_idx = edges.get_stack_min_value_idx(edges.root_idx,&stack_ptr);
         do {
            edge_s &edge = edges[e_idx];

            if (hse_map[e_idx] != c_idx_not_exist) {
               fprintf(a_file,"   /* node_%u_node_%u */ node_%u %s node_%u [ color = red ",edge.src_vertex_idx,edge.trg_vertex_idx,edge.src_vertex_idx,c_graph_type_dot_codes[type],edge.trg_vertex_idx);
            }
            else {
               fprintf(a_file,"   /* node_%u_node_%u */ node_%u %s node_%u [ ",edge.src_vertex_idx,edge.trg_vertex_idx,edge.src_vertex_idx,c_graph_type_dot_codes[type],edge.trg_vertex_idx);
            }

#ifdef print_dot_code_subgraph_LABEL_EVAL
            // - get host graph edge value string -
            edges.ef_get_string(edges[e_idx].value,value_str);

            if (value_str.size > 1) {
               fprintf(a_file,"label = %s ]\n",value_str.data);
            }
            else {
               fprintf(a_file,"]\n");
            }
#else
            if (hse_map[e_idx] != c_idx_not_exist) {
               fprintf(a_file,"label = \"%u > %u\" ]\n",hse_map[e_idx],e_idx);
            }
            else {
               fprintf(a_file,"label = \"%u\" ]\n",e_idx);
            }
#endif

            e_idx = edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
         } while(e_idx != c_idx_not_exist);
      }
   }

#ifdef print_dot_code_subgraph_LABEL_EVAL
   value_str.clear();
#endif

   fprintf(a_file,
"}\n"
   );
}/*}}}*/

void graph_s::DEBUG_print_mapping_dot_code(FILE *a_file,graph_s &a_graph,ui_arrays_s &a_vertex_maps)
{/*{{{*/
   cassert(type == a_graph.type && a_vertex_maps.used >= 1);

   switch (type) {
   case c_graph_type_graph:
      fprintf(a_file,
"graph G {\n"
      );
      break;
   case c_graph_type_digraph:
      fprintf(a_file,
"digraph G {\n"
      );
      break;
   default:
      cassert(0);
   }

   fprintf(a_file,
"   rankdir = LR\n"
"   node [ shape = ellipse ]\n"
"\n"
   );

   string_s value_str;
   value_str.init();

   // - traverse throught vertices of a_graph and print their values to output -
   {
      vertex_rb_tree_s &a_vertices = a_graph.vertices;

      if (a_vertices.root_idx != c_idx_not_exist) {
         unsigned stack[a_vertices.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned v_idx = a_vertices.get_stack_min_value_idx(a_vertices.root_idx,&stack_ptr);
         do {
            a_vertices.vf_get_string(a_vertices[v_idx].value,value_str);

            if (value_str.size > 1) {
               fprintf(a_file,"   node_src_%u [ label = %s ]\n",v_idx,value_str.data);
            }
            else {
               fprintf(a_file,"   node_src_%u\n",v_idx);
            }

            v_idx = a_vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
         } while(v_idx != c_idx_not_exist);
      }
   }

   // - traverse throught edges and print their values to output -
   {
      edge_rb_tree_s &a_edges = a_graph.edges;

      if (a_edges.root_idx != c_idx_not_exist) {
         unsigned stack[a_edges.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned e_idx = a_edges.get_stack_min_value_idx(a_edges.root_idx,&stack_ptr);
         do {
            edge_s &edge = a_edges[e_idx];

            a_edges.ef_get_string(edge.value,value_str);

            if (value_str.size > 1) {
               fprintf(a_file,"   node_src_%u %s node_src_%u [ label = %s ]\n",edge.src_vertex_idx,c_graph_type_dot_codes[type],edge.trg_vertex_idx,value_str.data);
            }
            else {
               fprintf(a_file,"   node_src_%u %s node_src_%u\n",edge.src_vertex_idx,c_graph_type_dot_codes[type],edge.trg_vertex_idx);
            }

            e_idx = a_edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
         } while(e_idx != c_idx_not_exist);
      }
   }

   // - traverse throught vertices and print their values to output -
   if (vertices.root_idx != c_idx_not_exist) {
      unsigned stack[vertices.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
      do {
         vertices.vf_get_string(vertices[v_idx].value,value_str);

         if (value_str.size > 1) {
            fprintf(a_file,"   node_trg_%u [ label = %s ]\n",v_idx,value_str.data);
         }
         else {
            fprintf(a_file,"   node_trg_%u\n",v_idx);
         }

         v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
      } while(v_idx != c_idx_not_exist);
   }

   // - traverse throught edges and print their values to output -
   if (edges.root_idx != c_idx_not_exist) {
      unsigned stack[edges.get_descent_stack_size()];
      unsigned *stack_ptr = stack;

      unsigned e_idx = edges.get_stack_min_value_idx(edges.root_idx,&stack_ptr);
      do {
         edge_s &edge = edges[e_idx];

         edges.ef_get_string(edge.value,value_str);

         if (value_str.size > 1) {
	    fprintf(a_file,"   node_trg_%u %s node_trg_%u [ label = %s ]\n",edge.src_vertex_idx,c_graph_type_dot_codes[type],edge.trg_vertex_idx,value_str.data);
         }
         else {
	    fprintf(a_file,"   node_trg_%u %s node_trg_%u\n",edge.src_vertex_idx,c_graph_type_dot_codes[type],edge.trg_vertex_idx);
         }

         e_idx = edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
      } while(e_idx != c_idx_not_exist);
   }

   value_str.clear();

   // - traverse throught all mappings -
   unsigned m_idx = 0;
   do {
      ui_array_s &vertex_map = a_vertex_maps[m_idx];
      cassert(vertex_map.used >= a_graph.vertices.used);

      vertex_rb_tree_s &a_vertices = a_graph.vertices;

      // - create mapping between graph vertices -
      if (a_vertices.root_idx != c_idx_not_exist) {
         
         fprintf(a_file,"   edge [ color = red ]\n");

         unsigned stack[a_vertices.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned v_idx = a_vertices.get_stack_min_value_idx(a_vertices.root_idx,&stack_ptr);
         do {
            fprintf(a_file,"   node_src_%u %s node_trg_%u [ label = %u ]\n",v_idx,c_graph_type_dot_codes[type],vertex_map[v_idx],m_idx);

            v_idx = a_vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
         } while(v_idx != c_idx_not_exist);
      }
   } while(++m_idx < a_vertex_maps.used);

   fprintf(a_file,
"}\n"
   );
}/*}}}*/

