
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

#ifndef __GRAPH_H
#define __GRAPH_H

/*
 * header files
 */

@begin
   include "basic.h"
   include "mem_check.h"
   include "structures.h"
   include "string.h"
@end

/*
 * constants and definitions
 */

// - graph types count -
const unsigned c_graph_type_cnt = 2;

// - list of graph types -
enum {
   c_graph_type_graph = 0,
   c_graph_type_digraph 
};

// - graph function type count -
const unsigned c_graph_function_type_cnt = 7;

// - list of graph function types -
enum {
   c_graph_function_init = 0,
   c_graph_function_compare,
   c_graph_function_operator_equal,
   c_graph_function_get_string,
   c_graph_function_clear,

   c_graph_function_serialize,
   c_graph_function_deserialize,
};

// - declarations of graph functions for precreated types -
int gf_char_compare(void *a_first,void *a_second);
void gf_char_operator_equal(void **a_trg,void *a_src);
void gf_char_get_string(void *a_value,string_s &a_string);
void gf_char_serialize(void *a_value,string_s &a_string);
void gf_char_deserialize(string_s &a_string,void **a_value);

int gf_int_compare(void *a_first,void *a_second);
void gf_int_operator_equal(void **a_trg,void *a_src);
void gf_int_get_string(void *a_value,string_s &a_string);
void gf_int_serialize(void *a_value,string_s &a_string);
void gf_int_deserialize(string_s &a_string,void **a_value);

int gf_unsigned_compare(void *a_first,void *a_second);
void gf_unsigned_operator_equal(void **a_trg,void *a_src);
void gf_unsigned_get_string(void *a_value,string_s &a_string);
void gf_unsigned_serialize(void *a_value,string_s &a_string);
void gf_unsigned_deserialize(string_s &a_string,void **a_value);

int gf_float_compare(void *a_first,void *a_second);
void gf_float_operator_equal(void **a_trg,void *a_src);
void gf_float_get_string(void *a_value,string_s &a_string);
void gf_float_serialize(void *a_value,string_s &a_string);
void gf_float_deserialize(string_s &a_string,void **a_value);

void gf_string_init(void **a_value_ptr);
int gf_string_compare(void *a_first,void *a_second);
void gf_string_operator_equal(void **a_trg,void *a_src);
void gf_string_get_string(void *a_value,string_s &a_string);
void gf_string_clear(void *a_value);
void gf_string_serialize(void *a_value,string_s &a_string);
void gf_string_deserialize(string_s &a_string,void **a_value);

// - arrays of graph functions for precreated types -
extern const void *graph_blank_functions[];
extern const void *graph_char_functions[];
extern const void *graph_int_functions[];
extern const void *graph_unsigned_functions[];
extern const void *graph_float_functions[];
extern const void *graph_string_functions[];

/*
 * definition of generated structures
 */

// -- gg_vertex_map_s --
@begin
   struct
   <
   unsigned:sv_idx
   unsigned:mv_idx
   unsigned:prev_map_idx
   unsigned:reference_cnt
   unsigned:next_edge_idx
   unsigned:length
   >
   gg_vertex_map_s;
@end

// -- gg_vertex_map_set_s --
@begin
   set<gg_vertex_map_s>

   additions {
      unsigned get_idx_by_sub_vertex(unsigned a_idx,unsigned a_sv_idx);
      unsigned get_idx_by_mother_vertex(unsigned a_idx,unsigned a_mv_idx);
      unsigned get_map_with_unused_edges(unsigned sg_vertex_cnt,unsigned a_idx);
      void retrieve_mapping_sub_to_mother(unsigned a_idx,ui_array_s &a_mapping);
      void release_vmap_string(unsigned a_idx);

      void DEBUG_print_string(unsigned a_idx);
   }

   gg_vertex_map_set_s;
@end

// -- gg_vertex_map_sets_s --
@begin
   array<gg_vertex_map_set_s> gg_vertex_map_sets_s;
@end

// -- edge_s --
@begin
   struct
   <
   pointer:value
   unsigned:src_vertex_idx
   unsigned:trg_vertex_idx
   >
   edge_s;
@end

// -- edge_rb_tree_s --
@begin
   rb_tree
   <
   edge_s
   pointer_ptr:value_fps
   >

   additions {
      inline void ef_init(void **a_value_ptr);
      inline int ef_compare(void *a_first,void *a_second);
      inline void ef_operator_equal(void **a_trg,void *a_src);
      inline void ef_get_string(void *a_value,string_s &a_string);
      inline void ef_clear(void *a_value);
   }

   edge_rb_tree_s;
@end

// -- edge_idxs_s --
@begin
   rb_tree<unsigned
   pointer:graph_ptr
   unsigned:vertex_idx
   >

   additions {
      inline int compare_edges(edge_s &a_f_edge,edge_s &a_s_edge);
      void get_edge_idxs_by_edge(edge_s &a_edge,ui_array_s &a_edge_idxs,unsigned &a_edge_cnt);
   }

   edge_idxs_s;
@end

// -- vertex_s --
@begin
   struct
   <
   pointer:value
   unsigned:input_degree
   unsigned:output_degree
   edge_idxs_s:edge_idxs
   >
   
   additions {
      inline unsigned get_degree();
   }

   vertex_s;
@end

// -- vertex_rb_tree_s --
@begin
   rb_tree
   <
   vertex_s
   pointer_ptr:value_fps
   > 

   additions {
      inline void vf_init(void **a_value_ptr);
      inline int vf_compare(void *a_first,void *a_second);
      inline void vf_operator_equal(void **a_trg,void *a_src);
      inline void vf_get_string(void *a_value,string_s &a_string);
      inline void vf_clear(void *a_value);
   }

   vertex_rb_tree_s;
@end

// -- graph_s --
@begin
   struct
   <
   unsigned:type
   unsigned:vertex_cnt
   vertex_rb_tree_s:vertices
   unsigned:edge_cnt
   edge_rb_tree_s:edges
   >

   not_generate: init clear swap operator=
   options: strict_dynamic

   additions {

      inline void init();
      inline void init(unsigned a_type);
      void clear();

      void swap(graph_s &second);
      graph_s &operator=(graph_s &second);

      /*!
       * \brief insert new vertex to graph
       * \param a_vertex - vertex to be inserted to graph
       * \return index of vertex in graph vertex set
       */
      inline unsigned __insert_swap_vertex(vertex_s &a_vertex);

      /*!
       * \brief insert new vertex to graph, and move evaluation (invalidate parameter)
       * \param a_value - value which is moved to evaluation of vertex
       * \return index of vertex in graph vertex set
       */
      inline unsigned insert_vertex_move_value(void *a_value);

      /*!
       * \brief insert new vertex to graph, and set its evaluation to given value
       * \param a_value - value which is set as evaluation of vertex
       * \return index of vertex in graph vertex set
       */
      inline unsigned insert_vertex(void *a_value);

      /*!
       * \brief insert new edge identified by two vertex indexes
       * \param a_sv_idx - index of source vertex
       * \param a_tv_idx - index of target vertex
       * \param a_edge - edge to be inserted into graph
       * \return index of new edge in set of graph edges
       */
      inline unsigned __insert_swap_edge(unsigned a_sv_idx,unsigned a_tv_idx,edge_s &a_edge);

      /*!
       * \brief insert new edge identified by two vertex indexes, and move value
       * \param a_sv_idx - index of source vertex
       * \param a_tv_idx - index of target vertex
       * \param a_value - value which is moved to evaluation of edge
       * \return index of new edge in set of graph edges
       */
      inline unsigned insert_edge_move_value(unsigned a_sv_idx,unsigned a_tv_idx,void *a_value);

      /*!
       * \brief insert new edge identified by two vertex indexes
       * \param a_sv_idx - index of source vertex
       * \param a_tv_idx - index of target vertex
       * \param a_value - value to which is set evaluation of edge
       * \return index of new edge in set of graph edges
       */
      inline unsigned insert_edge(unsigned a_sv_idx,unsigned a_tv_idx,void *a_value);

      /*!
       * \brief remove vertex identified by its index from graph
       * \param a_idx - index of vertex
       */
      inline void remove_vertex(unsigned a_idx);

      /*!
       * \brief remove edge identified by its index from graph
       * \param a_idx - index of edge to remove
       */
      inline void remove_edge(unsigned a_idx);

      /*!
       * \brief deserialize graph string evaluations to target evaluation
       * \param a_vertices_value_fps - pointer to target vertices evaluation
       * \param a_edges_value_fps - pointer to target edges evaluation
       */
      void deserialize_evaluation(void **a_vertices_value_fps,void **a_edges_value_fps);

      /*!
       * \brief tests if graph is empty
       * \return bool answer
       */
      inline bool is_null_graph();

      /*!
       * \brief is graph trivial? (only one vertex)
       * \return bool answer
       */
      inline bool is_trivial_graph();

      /*!
       * \brief is graph boquet? (contains exactly one vertex and at least one edge)
       * \return bool answer
       */
      inline bool is_boquet_graph();

      /*!
       * \brief id graph dipole? (contains exactly two vertices, at least one edges, and there are no loop edges)
       * \return bool answer
       */
      bool is_dipole_graph();

      /*!
       * \brief is graph connected? (each two vertices are connected by any path)
       * \return bool answer
       */
      bool is_connected_graph();

      /*!
       * \brief is graph bipartite? (vertices can be separated to two sets, where all edges have one vertex in first set and one vertex in second set)
       * \return bool answer
       */
      bool is_bipartite_graph();

      /*!
       * \brief is graph regular? (all vertices have equal degree)
       * \return bool answer
       */
      bool is_regular_graph();

      /*!
       * \brief retrieve set of vertices adjacent to given vertex
       * \param a_vertex_idx - index of given vertex
       * \param a_vertex_idxs - array which will be set to adjacent vertex indexes
       */
      void get_adjacent_vertices(unsigned a_vertex_idx,ui_array_s &a_vertex_idxs);

      /*!
       * \brief retrieve set of edges adjacent to given vertex
       * \param a_vertex_idx - index of given vertex
       * \param a_edge_idxs - array which will be set to adjacent edge indexes
       */
      void get_adjacent_edges(unsigned a_vertex_idx,ui_array_s &a_edge_idxs);

      /*!
       * \brief return sequence of graph vertex degrees
       * \param degree_sequence - array which will be set do degrees
       */
      void get_degree_sequence(ui_array_s &a_degree_sequence);

      /*!
       * \brief return graph source node index
       */
      unsigned find_source_node();

      /*!
       * \brief return graph source node index (optimalized version)
       */
      unsigned find_source_node_optimalized();

      /*!
       * \brief find all isomorphic subdigraphs in this graph
       * \param a_subgraph - given subgraph
       * \param a_join_vertex_map - mapping of subgraph vertices to bool => if true vertex is join vertex (can have extra incident edges)
       * \param a_ret_vertex_maps - array of vertex mappings for each found subgraph
       * \param a_ret_edge_maps - array of edge mappings for each found subgraph
       */
      void find_isomorphic_subdigraphs(graph_s &a_subgraph,bb_array_s &a_join_vertex_map,ui_arrays_s &a_ret_vertex_maps,ui_arrays_s &a_ret_edge_maps,bool a_create_edge_mapping);

      /*!
       * \brief load graph from ARG database data structure
       * \param a_file_name - name of file containing ARG_database graph description
       */
      bool load_ARG_database_graph(const char *a_file_name);

      /*!
       * \brief save graph in ARG database data structure
       * \param a_file_name - name of file to save graph to
       */
      bool save_ARG_database_graph(const char *a_file_name);

      /*!
       * \brief generate random graph with given count of vertices and given count of edges
       * \param a_vertex_cnt - count of generated graph vertices
       * \param a_edge_cnt - count of generated graph edges
       * \param a_graph_type - type of generated graph
       */
      void generate_connected_graph(unsigned a_vertex_cnt,unsigned a_edge_cnt,unsigned a_graph_type);

      /*!
       * \brief create graph isomorphic to given graph
       * \param a_graph - source graph, to which is created new isomorphic graph
       */
       void create_isomorphic_graph(graph_s &a_graph);

      /*!
       * \brief printf graph dot code to a_file
       * \param a_header - graph code header
       * \param a_node_prefix - prefix of graph nodes
       * \param a_indent - indentation of lines
       */
      void DEBUG_print_dot_code_params(FILE *a_file,const char *a_header,const char *a_node_prefix,const char *a_indent,void **a_vertices_value_fps,void **a_edges_value_fps);

      /*!
       * \brief printf graph dot code to a_file (generate graph eps image: dot -Tps <generated_text_file> > graph.eps)
       */
      void DEBUG_print_dot_code(FILE *a_file);

      /*!
       * \brief print graph code in dreadnaut input format
       */
      void DEBUG_print_nauty_dreadnaut_code(FILE *a_file);

      /*!
       * \brief printf graph dot (with denoted subgraph) code to stdout
       */
      void DEBUG_print_dot_code_subgraph(FILE *a_file,graph_s &a_subgraph,ui_array_s &a_vertex_map,ui_array_s &a_edge_map);

      /*!
       * \brief printf graph dot code with mapping from a_graph to this graph
       */
      void DEBUG_print_mapping_dot_code(FILE *a_file,graph_s &a_graph,ui_arrays_s &a_vertex_maps);
   }

   graph_s;
@end

bool edge_src_trg_vertex_compare(edge_s &a_first,edge_s &a_second);

/*
 * inline methods of generated structures
 */

// -- gg_vertex_map_s --
@begin
   inlines gg_vertex_map_s
@end

// -- gg_vertex_map_set_s --
@begin
   inlines gg_vertex_map_set_s
@end

// -- gg_vertex_map_sets_s --
@begin
   inlines gg_vertex_map_sets_s
@end

// -- edge_s --
@begin
   inlines edge_s
@end

// -- edge_rb_tree_s --
@begin
   inlines edge_rb_tree_s
@end

inline int edge_rb_tree_s::__compare_value(edge_s &a_first,edge_s &a_second) 
{/*{{{*/
   return ef_compare(a_first.value,a_second.value);
}/*}}}*/

inline void edge_rb_tree_s::ef_init(void **a_value_ptr)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_init];

   if (caller != NULL) {
      ((int(*)(void **))caller)(a_value_ptr);
   }
   else {
      *a_value_ptr = NULL;
   }
}/*}}}*/

inline int edge_rb_tree_s::ef_compare(void *a_first,void *a_second)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_compare];

   if (caller == NULL) {
      return 0;
   }

   return ((int(*)(void *,void *))caller)(a_first,a_second);
}/*}}}*/

inline void edge_rb_tree_s::ef_operator_equal(void **a_trg,void *a_src)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_operator_equal];

   if (caller != NULL) {
      ((void(*)(void **,void *))caller)(a_trg,a_src);
   }
   else {
      *a_trg = NULL;
   }
}/*}}}*/

inline void edge_rb_tree_s::ef_get_string(void *a_value,string_s &a_string)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_get_string];

   if (caller != NULL) {
      ((void(*)(void *,string_s &))caller)(a_value,a_string);
   }
   else {
      a_string.clear();
   }
}/*}}}*/

inline void edge_rb_tree_s::ef_clear(void *a_value)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_clear];

   if (caller != NULL) {
      ((void(*)(void *))caller)(a_value);
   }
}/*}}}*/

// -- edge_idxs_s --
@begin
   inlines edge_idxs_s
@end

inline int edge_idxs_s::compare_edges(edge_s &a_f_edge,edge_s &a_s_edge)
{/*{{{*/
   edge_rb_tree_s &edges = ((graph_s *)graph_ptr)->edges;
   vertex_rb_tree_s &vertices = ((graph_s *)graph_ptr)->vertices;

   // - test edge evaluations -
   unsigned cmp_res = edges.ef_compare(a_f_edge.value,a_s_edge.value);
   if (cmp_res != 0) return cmp_res;

   unsigned f_tv_idx;
   unsigned s_tv_idx;

   bool f_orientation;
   bool s_orientation;

   // - find first edge target vertex and orientation -
   if (a_s_edge.src_vertex_idx == vertex_idx) {
      f_tv_idx = a_s_edge.trg_vertex_idx;
      f_orientation = true;
   }
   else {
      f_tv_idx = a_s_edge.src_vertex_idx;
      f_orientation = false;
   }

   // - find second edge target vertex and orientation -
   if (a_f_edge.src_vertex_idx == vertex_idx) {
      s_tv_idx = a_f_edge.trg_vertex_idx;
      s_orientation = true;
   }
   else {
      s_tv_idx = a_f_edge.src_vertex_idx;
      s_orientation = false;
   }

   cmp_res = vertices.vf_compare(vertices[f_tv_idx].value,vertices[s_tv_idx].value);
   if (cmp_res != 0) return cmp_res;

   if (f_orientation != s_orientation) {
      return f_orientation?-1:1;
   }

   return f_tv_idx < s_tv_idx?-1:f_tv_idx > s_tv_idx;
}/*}}}*/

inline int edge_idxs_s::__compare_value(unsigned &a_first,unsigned &a_second)
{/*{{{*/
   edge_rb_tree_s &edges = ((graph_s *)graph_ptr)->edges;

   int res = compare_edges(edges[a_first],edges[a_second]);
   if (res != 0) {
      return res;
   }

   return a_first < a_second?-1:a_second < a_first;
}/*}}}*/

// -- vertex_s --
@begin
   inlines vertex_s
@end

inline unsigned vertex_s::get_degree()
{/*{{{*/
   return input_degree + output_degree;
}/*}}}*/

// -- vertex_rb_tree_s --
@begin
   inlines vertex_rb_tree_s
@end

inline int vertex_rb_tree_s::__compare_value(vertex_s &a_first,vertex_s &a_second) 
{/*{{{*/
   return vf_compare(a_first.value,a_second.value);
}/*}}}*/

inline void vertex_rb_tree_s::vf_init(void **a_value_ptr)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_init];

   if (caller != NULL) {
      ((int(*)(void **))caller)(a_value_ptr);
   }
   else {
      *a_value_ptr = NULL;
   }
}/*}}}*/

inline int vertex_rb_tree_s::vf_compare(void *a_first,void *a_second)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_compare];

   if (caller == NULL) {
      return 0;
   }

   return ((int(*)(void *,void *))caller)(a_first,a_second);
}/*}}}*/

inline void vertex_rb_tree_s::vf_operator_equal(void **a_trg,void *a_src)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_operator_equal];

   if (caller != NULL) {
      ((void(*)(void **,void *))caller)(a_trg,a_src);
   }
   else {
      *a_trg = NULL;
   }
}/*}}}*/

inline void vertex_rb_tree_s::vf_get_string(void *a_value,string_s &a_string)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_get_string];

   if (caller != NULL) {
      ((void(*)(void *,string_s &))caller)(a_value,a_string);
   }
   else {
      a_string.clear();
   }
}/*}}}*/

inline void vertex_rb_tree_s::vf_clear(void *a_value)
{/*{{{*/
   register void *caller = value_fps[c_graph_function_clear];

   if (caller != NULL) {
      ((void(*)(void *))caller)(a_value);
   }
}/*}}}*/

// -- graph_s --
@begin
   inlines graph_s
@end

inline void graph_s::init()
{/*{{{*/
   init(c_graph_type_graph);
}/*}}}*/

inline void graph_s::init(unsigned a_type)
{/*{{{*/
   type = a_type;

   vertex_cnt = 0;
   vertices.init();
   vertices.value_fps = (void **)graph_blank_functions;

   edge_cnt = 0;
   edges.init();
   edges.value_fps = (void **)graph_blank_functions;
}/*}}}*/

inline unsigned graph_s::__insert_swap_vertex(vertex_s &a_vertex)
{/*{{{*/
   a_vertex.input_degree = 0;
   a_vertex.output_degree = 0;
   
   unsigned vertex_idx = vertices.swap_insert(a_vertex);

   // - set edge indexes structure pointers and vertex index -
   edge_idxs_s &edge_idxs = vertices[vertex_idx].edge_idxs;
   edge_idxs.graph_ptr = (pointer)this;
   edge_idxs.vertex_idx = vertex_idx;

   vertex_cnt++;

   return vertex_idx;
}/*}}}*/

inline unsigned graph_s::insert_vertex_move_value(void *a_value)
{/*{{{*/
   vertex_s new_vertex;
   new_vertex.init();
   new_vertex.value = a_value;
   
   unsigned vertex_idx = __insert_swap_vertex(new_vertex);
   new_vertex.clear();

   return vertex_idx;
}/*}}}*/

inline unsigned graph_s::insert_vertex(void *a_value)
{/*{{{*/
   vertex_s new_vertex;
   new_vertex.init();

   vertices.vf_init(&new_vertex.value);
   vertices.vf_operator_equal(&new_vertex.value,a_value);
   
   unsigned vertex_idx = __insert_swap_vertex(new_vertex);
   new_vertex.clear();

   return vertex_idx;
}/*}}}*/

inline unsigned graph_s::__insert_swap_edge(unsigned a_sv_idx,unsigned a_tv_idx,edge_s &a_edge)
{/*{{{*/
   vertex_s &src_vertex = vertices[a_sv_idx];
   vertex_s &trg_vertex = vertices[a_tv_idx];

   a_edge.src_vertex_idx = a_sv_idx;
   a_edge.trg_vertex_idx = a_tv_idx;

   unsigned edge_idx = edges.swap_insert(a_edge);

   src_vertex.output_degree++;
   src_vertex.edge_idxs.insert(edge_idx);

   trg_vertex.input_degree++;
   if (a_sv_idx != a_tv_idx) {
      trg_vertex.edge_idxs.insert(edge_idx);
   }

   edge_cnt++;

   return edge_idx;
}/*}}}*/

inline unsigned graph_s::insert_edge_move_value(unsigned a_sv_idx,unsigned a_tv_idx,void *a_value)
{/*{{{*/
   edge_s new_edge;
   new_edge.init();
   new_edge.value = a_value;

   unsigned edge_idx = __insert_swap_edge(a_sv_idx,a_tv_idx,new_edge);
   new_edge.clear();

   return edge_idx;
}/*}}}*/

inline unsigned graph_s::insert_edge(unsigned a_sv_idx,unsigned a_tv_idx,void *a_value)
{/*{{{*/
   edge_s new_edge;
   new_edge.init();

   edges.ef_init(&new_edge.value);
   edges.ef_operator_equal(&new_edge.value,a_value);

   unsigned edge_idx = __insert_swap_edge(a_sv_idx,a_tv_idx,new_edge);
   new_edge.clear();

   return edge_idx;
}/*}}}*/

inline void graph_s::remove_vertex(unsigned a_idx)
{/*{{{*/
   vertex_s &vertex = vertices[a_idx];
   cassert(vertex.edge_idxs.root_idx == c_idx_not_exist);
   
   vertices.vf_clear(vertex.value);
   vertices.remove(a_idx);

   vertex_cnt--;
}/*}}}*/

inline void graph_s::remove_edge(unsigned a_idx)
{/*{{{*/
   edge_s &edge = edges[a_idx];

   vertex_s &src_vertex = vertices[edge.src_vertex_idx];
   vertex_s &trg_vertex = vertices[edge.trg_vertex_idx];

   // - remove edge index from source vertex edge indexes -
   unsigned sv_edge_idx = src_vertex.edge_idxs.get_idx(a_idx);
   src_vertex.edge_idxs.remove(sv_edge_idx);
   src_vertex.output_degree--;

   // - if source and target vertex are different, then remove edge index from target vertex edge indexes -
   if (edge.src_vertex_idx != edge.trg_vertex_idx) {
      unsigned tv_edge_idx = trg_vertex.edge_idxs.get_idx(a_idx);
      trg_vertex.edge_idxs.remove(tv_edge_idx);
   }
   trg_vertex.input_degree--;

   edges.ef_clear(edge.value);
   edges.remove(a_idx);

   edge_cnt--;
}/*}}}*/

inline bool graph_s::is_null_graph()
{/*{{{*/
   return vertex_cnt == 0;
}/*}}}*/

inline bool graph_s::is_trivial_graph()
{/*{{{*/
   return vertex_cnt == 1 && edge_cnt == 0;
}/*}}}*/

inline bool graph_s::is_boquet_graph()
{/*{{{*/
   return vertex_cnt == 1 && edge_cnt >= 1;
}/*}}}*/

#endif

