
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

@begin
   include "graph_parser.h"
@end

/*
 * constants and definitions
 */

/*
 * global functions
 */

void *process_parser_states_thread_function(void *process_info)
{/*{{{*/
   pointer *proc_info = (pointer *)process_info;

   ((graph_parser_s *)proc_info[0])->process_parse_states(
      *((graph_array_s *)proc_info[1])
      ,*((graph_s *)proc_info[2])
      ,*((gp_parse_states_s *)proc_info[3])
      ,*((gg_vertex_map_set_s *)proc_info[4])
      ,*((ui_arrays_array_s *)proc_info[5])
      ,*((mutex_s *)proc_info[6]));

   return NULL;
}/*}}}*/

/*
 * methods of generated structures
 */

// -- graph_array_s --
@begin
   methods graph_array_s
@end

void graph_array_s::remove_by_idx(unsigned a_idx)
{/*{{{*/
   debug_assert(a_idx < used);

   data[a_idx].clear();

   if (a_idx < used - 1) {
      graph_s *ptr = data + a_idx;
      graph_s *ptr_end = data + (used - 1);
      do {
         ptr->swap(*(ptr + 1));
      } while(++ptr < ptr_end);
   }

   used -= 1;
}/*}}}*/

// -- gp_graph_state_info_s --
@begin
   methods gp_graph_state_info_s
@end

// -- gp_graph_state_info_array_s --
@begin
   methods gp_graph_state_info_array_s
@end

// -- gp_state_descriptor_s --
@begin
   methods gp_state_descriptor_s
@end

// -- gp_state_descriptors_s --
@begin
   methods gp_state_descriptors_s
@end

// -- gp_state_transition_s --
@begin
   methods gp_state_transition_s
@end

// -- gp_state_transitions_s --
@begin
   methods gp_state_transitions_s
@end

unsigned gp_state_transitions_s::get_idx_with_evaluations(graph_parser_s &a_gp,gp_state_transition_s &a_transition)
{/*{{{*/
   if (used == 0) return c_idx_not_exist;

   gp_state_transition_s *t_ptr = data;
   gp_state_transition_s *t_ptr_end = t_ptr + used;
   do {

      if (a_transition.ag_src_vertex_idx == t_ptr->ag_src_vertex_idx && 
         a_transition.ag_trg_vertex_idx == t_ptr->ag_trg_vertex_idx &&
         a_gp.ef_compare(a_transition.edge_evaluation,t_ptr->edge_evaluation) == 0 &&
         a_transition.edge_direction == t_ptr->edge_direction &&
         a_gp.vf_compare(a_transition.trg_vertex_evaluation,t_ptr->trg_vertex_evaluation) == 0 &&
         a_transition.trg_vertex_in_degree == t_ptr->trg_vertex_in_degree &&
         a_transition.trg_vertex_out_degree == t_ptr->trg_vertex_out_degree &&
         a_transition.trg_vertex_join == t_ptr->trg_vertex_join) {

         return t_ptr - data;
      }
   } while(++t_ptr < t_ptr_end);

   return c_idx_not_exist;
}/*}}}*/

// -- gp_state_s --
@begin
   methods gp_state_s
@end

// -- gp_states_s --
@begin
   methods gp_states_s
@end

// -- gp_next_move_s --
@begin
   methods gp_next_move_s
@end

// -- gp_found_graph_info_s --
@begin
   methods gp_found_graph_info_s
@end

// -- gp_found_graph_infos_s --
@begin
   methods gp_found_graph_infos_s
@end

// -- gp_parse_state_s --
@begin
   methods gp_parse_state_s
@end

// -- gp_parse_states_s --
@begin
   methods gp_parse_states_s
@end

// -- gp_parse_states_array_s --
@begin
   methods gp_parse_states_array_s
@end

// -- graph_parser_s --
@begin
   methods graph_parser_s
@end

bool graph_parser_s::find_next_move(graph_array_s &a_subgraphs,gp_graph_state_info_s &a_gp_si,gp_next_move_s &a_next_move)
{/*{{{*/
   graph_s &subgraph = a_subgraphs[a_gp_si.graph_idx];
   edge_rb_tree_s &edges = subgraph.edges;
   unsigned vmap_idx = a_gp_si.vmap_idx;

   do {
      gg_vertex_map_s *vmap_ptr = &create_vertex_map[vmap_idx];

      // - all vertices of subgraph were conected -
      if (vmap_ptr->length == subgraph.vertex_cnt) {
         a_next_move.prev_vmap_idx = vmap_idx;

         return true;
      }

      edge_idxs_s &vmap_sv_edge_idxs = subgraph.vertices[vmap_ptr->sv_idx].edge_idxs;

      // - find next edge and its target -
      if (vmap_ptr->next_edge_idx != c_idx_not_exist) {
         do {

            // - subgraph edge incident with vmap_ptr->sv_idx vertex -
            unsigned sv_edge_idx = vmap_sv_edge_idxs[vmap_ptr->next_edge_idx];
            edge_s &sv_edge = edges[sv_edge_idx];

            // - next possible processed subgraph vertex -
            unsigned sv_idx = (sv_edge.src_vertex_idx != vmap_ptr->sv_idx)?sv_edge.src_vertex_idx:sv_edge.trg_vertex_idx;

            // - if vertex is not used in mapping string -
            if (create_vertex_map.get_idx_by_sub_vertex(vmap_idx,sv_idx) == c_idx_not_exist) {
               
               // - set move and return -
               a_next_move.edge_idx = sv_edge_idx;
               a_next_move.trg_vertex_idx = sv_idx;
               a_next_move.prev_vmap_idx = vmap_idx;

               // - move to next diferend edge (ignore multi edges with same value, source vertex and target vertex) -
               unsigned old_edge_idx = vmap_sv_edge_idxs[vmap_ptr->next_edge_idx];
               do {
                  vmap_ptr->next_edge_idx = vmap_sv_edge_idxs.get_next_idx(vmap_ptr->next_edge_idx);
               } while(vmap_ptr->next_edge_idx != c_idx_not_exist && vmap_sv_edge_idxs.compare_edges(edges[old_edge_idx],edges[vmap_sv_edge_idxs[vmap_ptr->next_edge_idx]]) == 0);

               return false;
            }

            // - move to next diferend edge (ignore multi edges with same value, source vertex and target vertex) -
            unsigned old_edge_idx = vmap_sv_edge_idxs[vmap_ptr->next_edge_idx];
            do {
               vmap_ptr->next_edge_idx = vmap_sv_edge_idxs.get_next_idx(vmap_ptr->next_edge_idx);
            } while(vmap_ptr->next_edge_idx != c_idx_not_exist && vmap_sv_edge_idxs.compare_edges(edges[old_edge_idx],edges[vmap_sv_edge_idxs[vmap_ptr->next_edge_idx]]) == 0);
         } while(vmap_ptr->next_edge_idx != c_idx_not_exist);
      }

      // - search for mappings in map string with "unused" edges -
      unsigned m_idx = create_vertex_map.get_map_with_unused_edges(subgraph.vertices.used,vmap_idx);
      cassert(m_idx != c_idx_not_exist);

      // - insert new map by copy from m_idx with increased next_edge_idx -
      unsigned new_vmap_idx = create_vertex_map.insert_blank();

      // - refresh vmap_ptr -
      vmap_ptr = &create_vertex_map[vmap_idx];

      // - retrieve mapping to duplicate -
      gg_vertex_map_s &split_vmap = create_vertex_map[m_idx];

      // - set new create_vertex_map element -
      create_vertex_map[new_vmap_idx].set(split_vmap.sv_idx,split_vmap.mv_idx,vmap_idx,1,split_vmap.next_edge_idx,vmap_ptr->length);

      // - set vmap_idx to new_vmap_idx -
      vmap_idx = new_vmap_idx;
   } while(1);
}/*}}}*/

void graph_parser_s::create_parsing_automata(graph_array_s &a_subgraphs,ui_arrays_s &a_join_vertex_maps)
{/*{{{*/
   cassert(a_subgraphs.used != 0 && vertices_value_fps != NULL && edges_value_fps != NULL);

   // - test if all subgraphs are connected, and have same type, and vertex + edge value_fps -
   {

      // - reference graph for vertices and edges value functions -
      graph_s &ref_graph = a_subgraphs[0];

      pointer_ptr ref_vvf = ref_graph.vertices.value_fps;
      pointer_ptr ref_evf = ref_graph.edges.value_fps;

      graph_s *ptr = a_subgraphs.data;
      graph_s *ptr_end = ptr + a_subgraphs.used;
      do {
         cassert(ptr->vertices.root_idx != c_idx_not_exist && ptr->vertices.value_fps == ref_vvf &&
               ptr->edges.value_fps == ref_evf && ptr->is_connected_graph() && ptr->type == ref_graph.type);
      } while(++ptr < ptr_end);
   }

   // - clear state transitions info -
   states.used = 0;

   // - clear state descriptor array -
   state_descriptors.used = 0;

   // - clear subgraphs vertex mapping -
   create_vertex_map.clear();

   // - array of temporary vertex mappings (for subgraph isomorphism searching) -
   ui_arrays_s tmp_v_maps;
   tmp_v_maps.init();

   // - mapping of state transitions to target state descriptors -
   ui_array_s trans_to_des_map;
   trans_to_des_map.init();

   // - initialize first states (by value of first vertices) -
   //{
   //   debug_message_3(fprintf(stderr,"graph_parser: create_parsing_automata: initialize first states (by value of first vertices)\n"));

   //   // - insert first state -
   //   states.push_blank();
   //   gp_state_s &state = states.last();
   //   state.found_graph_info_idxs.used = 0;

   //   unsigned sg_idx = 0;
   //   do {
   //      graph_s &subgraph = a_subgraphs[sg_idx];
   //      //unsigned fv_idx = subgraph.find_source_node();
   //      unsigned fv_idx = subgraph.find_source_node_optimalized();
   //      vertex_s &vertex = subgraph.vertices[fv_idx];

   //      edge_idxs_s &edge_idxs = subgraph.vertices[fv_idx].edge_idxs;

   //      // - find index of first edge -
   //      unsigned first_edge_idx;
   //      if (edge_idxs.root_idx != c_idx_not_exist) {
   //         first_edge_idx = edge_idxs.get_min_value_idx(edge_idxs.root_idx);
   //      }
   //      else {
   //         first_edge_idx = c_idx_not_exist;
   //      }

   //      // - create new transition -
   //      gp_state_transition_s transition;

   //      // - transition has no source vertex and traverse edge -
   //      transition.ag_src_vertex_idx = c_idx_not_exist;
   //      transition.ag_trg_vertex_idx = 0;
   //      ef_init(&transition.edge_evaluation);
   //      
   //      transition.edge_direction = true;
   //      vf_init(&transition.trg_vertex_evaluation);
   //      vf_operator_equal(&transition.trg_vertex_evaluation,vertex.value);
   //      
   //      // - set transition target vertex info -
   //      vertex_s &svertex = subgraph.vertices[fv_idx];
   //      transition.trg_vertex_in_degree = svertex.input_degree;
   //      transition.trg_vertex_out_degree = svertex.output_degree;
   //      transition.trg_vertex_join = (a_join_vertex_maps[sg_idx][fv_idx] != c_idx_not_exist);

   //      transition.target_state = c_idx_not_exist;

   //      // - create new state descriptor -
   //      state_descriptors.push_blank();
   //      unsigned sd_idx = state_descriptors.used - 1;

   //      debug_message_4(fprintf(stderr,"graph_parser: create_parsing_automata: create new state descriptor: %u\n",sd_idx));

   //      state_descriptors[sd_idx].vertex_counter = 1;

   //      // - reference on state transitions -
   //      gp_state_transitions_s &transitions = state.transitions;

   //      // - insert transition to actual state -
   //      transitions.push_blank();
   //      transitions.last().swap(transition);

   //      unsigned transition_idx = transitions.used - 1;

   //      // - save mapping of transition to state descriptor -
   //      trans_to_des_map.push(sd_idx);

   //      // - get reference on state descriptor -
   //      gp_state_descriptor_s &state_des = state_descriptors[sd_idx];

   //      // - create new vertex map block -
   //      unsigned vmap_idx = create_vertex_map.insert_blank();
   //      create_vertex_map[vmap_idx].set(fv_idx,0,c_idx_not_exist,1,first_edge_idx,1);

   //      // - insert graph info -
   //      state_des.gp_si_array.push_blank();
   //      gp_graph_state_info_s &gs_info = state_des.gp_si_array.last();
   //      gs_info.graph_idx = sg_idx;
   //      gs_info.vmap_idx = vmap_idx;
   //      gs_info.src_state_idx = 0;

   //      // - insert transition idx to state description -
   //      gs_info.src_transition_idx = transition_idx;

   //   } while(++sg_idx < a_subgraphs.used);
   //}

   // - initialize first states (by value of first vertices) -
   {
      debug_message_3(fprintf(stderr,"graph_parser: create_parsing_automata: initialize first states (by value of first vertices)\n"));

      // - insert first state -
      states.push_blank();
      gp_state_s &state = states.last();
      state.found_graph_info_idxs.used = 0;

      // - graph processed array -
      bool sg_done[a_subgraphs.used];
      memset(sg_done,false,a_subgraphs.used*sizeof(bool));

      // - cycle throught all subgraphs -
      unsigned sg_idx = 0;
      do {
         if (!sg_done[sg_idx]) {

            vertex_s &vertex = a_subgraphs[sg_idx].vertices[a_subgraphs[sg_idx].vertices.root_idx];

            // - create new state based on value of vertex -
            unsigned g_idx = sg_idx;
            do {
               if (!sg_done[g_idx]) {
                  graph_s &subgraph = a_subgraphs[g_idx];
                  
                  unsigned fv_idx = subgraph.vertices.get_idx(vertex);
                  if (fv_idx != c_idx_not_exist) {

                     debug_message_4
                     (/*{{{*/
                        string_s str;
                        str.init();
                        vf_get_string(vertex.value,str);

                        fprintf(stderr,"create_parsing_automata: graph: %u: with vertex(eval.): %s\n",g_idx,str.data);

                        str.clear();
                     );/*}}}*/

                     edge_idxs_s &edge_idxs = subgraph.vertices[fv_idx].edge_idxs;

                     // - find index of first edge -
                     unsigned first_edge_idx;
                     if (edge_idxs.root_idx != c_idx_not_exist) {
                        first_edge_idx = edge_idxs.get_min_value_idx(edge_idxs.root_idx);
                     }
                     else {
                        first_edge_idx = c_idx_not_exist;
                     }

                     // - create new transition -
                     gp_state_transition_s transition;

                     // - transition has no source vertex and traverse edge -
                     transition.ag_src_vertex_idx = c_idx_not_exist;
                     transition.ag_trg_vertex_idx = 0;
                     ef_init(&transition.edge_evaluation);
                     
                     transition.edge_direction = true;
                     vf_init(&transition.trg_vertex_evaluation);
                     vf_operator_equal(&transition.trg_vertex_evaluation,vertex.value);
                     
                     // - set transition target vertex info -
                     vertex_s &svertex = subgraph.vertices[fv_idx];
                     transition.trg_vertex_in_degree = svertex.input_degree;
                     transition.trg_vertex_out_degree = svertex.output_degree;
                     transition.trg_vertex_join = (a_join_vertex_maps[g_idx][fv_idx] != c_idx_not_exist);

                     transition.target_state = c_idx_not_exist;

                     // - index of transition to next state -
                     unsigned transition_idx = state.transitions.get_idx_with_evaluations(*this,transition);

                     // - state already exist for this vertex evaluation -
                     unsigned sd_idx;
                     if (transition_idx == c_idx_not_exist) {
                        
                        // - create new state descriptor -
                        state_descriptors.push_blank();
                        sd_idx = state_descriptors.used - 1;

                        debug_message_4(fprintf(stderr,"graph_parser: create_parsing_automata: create new state descriptor: %u\n",sd_idx));

                        state_descriptors[sd_idx].vertex_counter = 1;

                        // - reference on state transitions -
                        gp_state_transitions_s &transitions = state.transitions;

                        // - insert transition to actual state -
                        transitions.push_blank();
                        transitions.last().swap(transition);

                        transition_idx = transitions.used - 1;

                        // - save mapping of transition to state descriptor -
                        trans_to_des_map.push(sd_idx);
                     }
                     else {
                        sd_idx = trans_to_des_map[transition_idx];

                        debug_message_4(fprintf(stderr,"graph_parser: create_parsing_automata: use existing state descriptor: %u\n",sd_idx));

                        transition.clear_evaluations(*this);
                        transition_idx = c_idx_not_exist;
                     }

                     // - get reference on state descriptor -
                     gp_state_descriptor_s &state_des = state_descriptors[sd_idx];

                     // - create new vertex map block -
                     unsigned vmap_idx = create_vertex_map.insert_blank();
                     create_vertex_map[vmap_idx].set(fv_idx,0,c_idx_not_exist,1,first_edge_idx,1);

                     // - insert graph info -
                     state_des.gp_si_array.push_blank();
                     gp_graph_state_info_s &gs_info = state_des.gp_si_array.last();
                     gs_info.graph_idx = g_idx;
                     gs_info.vmap_idx = vmap_idx;
                     gs_info.src_state_idx = 0;

                     // - insert transition idx to state description -
                     gs_info.src_transition_idx = transition_idx;

                     sg_done[g_idx] = true;
                  }
               }
            } while(++g_idx < a_subgraphs.used);
         }
      } while(++sg_idx < a_subgraphs.used);
   }

   debug_message_3(fprintf(stderr,"graph_parser: create_parsing_automata: process all state descriptors\n"));

   // - process all state descriptors -
   do {
      gp_state_descriptor_s state_des;
      state_des.init();
      state_des.vertex_counter = 0;

      trans_to_des_map.used = 0;

      // - retrieve state descriptor from stack -
      state_des.swap(state_descriptors.pop());

      debug_assert(state_des.gp_si_array.used != 0);

      // - insert new state -
      states.push_blank();
      gp_state_s &state = states.last();
      state.found_graph_info_idxs.used = 0;

      // - traverse throught graph state infos -
      unsigned gsi_idx = 0;
      do {
         gp_graph_state_info_s &gp_si = state_des.gp_si_array[gsi_idx];

         // - set source transition target state -
         if (gp_si.src_transition_idx != c_idx_not_exist) {
            states[gp_si.src_state_idx].transitions[gp_si.src_transition_idx].target_state = states.used - 1;
         }

         // - find next possible move -
         gp_next_move_s next_move;
         if (find_next_move(a_subgraphs,gp_si,next_move)) {
            
            // - process found vertex mapping -
            debug_message_4(fprintf(stderr,"graph_parser: create_parsing_automata: found final state of graph: %u\n",gp_si.graph_idx));

            // - create new found graph info and insert its index to state description -
            found_graph_infos.push_blank();
            state.found_graph_info_idxs.push(found_graph_infos.used - 1);

            // - reference on found graph info -
            gp_found_graph_info_s &found_graph_info = found_graph_infos.last();

            // - set found_graph_info -
            found_graph_info.graph_idx = gp_si.graph_idx;
            found_graph_info.a_graph_vertices_used = state_des.vertex_counter;

            ui_array_s &vmap = found_graph_info.subgraph_to_actual_vmap;
            graph_s &subgraph = a_subgraphs[gp_si.graph_idx];

            vmap.copy_resize(subgraph.vertices.used);
            vmap.used = subgraph.vertices.used;

            // - create vertex map -
            create_vertex_map.retrieve_mapping_sub_to_mother(next_move.prev_vmap_idx,vmap);

            debug_message_4
            (/*{{{*/
               fprintf(stderr,"graph_parser: create_parsing_automata: vertex_map: ");

               vertex_rb_tree_s &vertices = subgraph.vertices;
               unsigned stack[vertices.get_descent_stack_size()];
               unsigned *stack_ptr = stack;

               unsigned v_idx = vertices.get_stack_min_value_idx(vertices.root_idx,&stack_ptr);
               do {
                  fprintf(stderr,"%u -> %u | ",v_idx,vmap[v_idx]);
                  v_idx = vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
               } while(v_idx != c_idx_not_exist);

               fprintf(stderr,"\n");
            );/*}}}*/

            // - release graph vertex map -
            create_vertex_map.release_vmap_string(next_move.prev_vmap_idx);
         }
         else {

            // - insert new vertex and edge to "actual graph" -
            {
               graph_s &subgraph = a_subgraphs[gp_si.graph_idx];

               gg_vertex_map_s *prev_vmap_ptr = &create_vertex_map[next_move.prev_vmap_idx];
               unsigned ag_fv_idx = prev_vmap_ptr->mv_idx;

               edge_s &s_edge = subgraph.edges[next_move.edge_idx];
               vertex_s &s_trg_vertex = subgraph.vertices[next_move.trg_vertex_idx];

               debug_message_4
               (/*{{{*/
                  string_s e_str;
                  string_s v_str;

                  e_str.init();
                  v_str.init();

                  ef_get_string(s_edge.value,e_str);
                  vf_get_string(s_trg_vertex.value,v_str);

                  fprintf(stderr,"graph_parser: create_parsing_automata: graph: %u: throught edge(eval.): %s (direction %s) to vertex(eval.): %s\n",gp_si.graph_idx,e_str.data,s_edge.src_vertex_idx == next_move.trg_vertex_idx?"backward":"forward",v_str.data);

                  v_str.clear();
                  e_str.clear();
               );/*}}}*/

               // - transition vertex of actual graph -
               unsigned a_trg_vertex_idx = state_des.vertex_counter;

               // - transition edge of actual graph -
               bool edge_direction = s_edge.src_vertex_idx != next_move.trg_vertex_idx;

               // - create new transition -
               gp_state_transition_s transition;

               ef_init(&transition.edge_evaluation);
               ef_operator_equal(&transition.edge_evaluation,s_edge.value);
               
               transition.edge_direction = edge_direction;
               vf_init(&transition.trg_vertex_evaluation);
               vf_operator_equal(&transition.trg_vertex_evaluation,s_trg_vertex.value);

               // - set transition target vertex info -
               transition.trg_vertex_in_degree = s_trg_vertex.input_degree;
               transition.trg_vertex_out_degree = s_trg_vertex.output_degree;
               transition.trg_vertex_join = (a_join_vertex_maps[gp_si.graph_idx][next_move.trg_vertex_idx] != c_idx_not_exist);

               transition.target_state = c_idx_not_exist;

               // - set transition ag_src_vertex_idx and ag_trg_vertex_idx -
               transition.ag_src_vertex_idx = ag_fv_idx;
               transition.ag_trg_vertex_idx = a_trg_vertex_idx;

               // - index of transition to next state -
               unsigned transition_idx = state.transitions.get_idx_with_evaluations(*this,transition);

               // - state alredy exist for this transition -
               unsigned sd_idx;
               if (transition_idx == c_idx_not_exist) {

                  // - create new state descriptor -
                  state_descriptors.push_blank();
                  sd_idx = state_descriptors.used - 1;

                  debug_message_4(fprintf(stderr,"graph_parser: create_parsing_automata: create new state descriptor: %u\n",sd_idx));

                  state_descriptors[sd_idx].vertex_counter = state_des.vertex_counter + 1;

                  // - reference on transitions -
                  gp_state_transitions_s &transitions = state.transitions;

                  // - insert transition to actual state -
                  transitions.push_blank();
                  transitions.last().swap(transition);

                  transition_idx = transitions.used - 1;

                  // - save mapping of transition to state descriptor -
                  trans_to_des_map.push(sd_idx);
               }
               else {
                  sd_idx = trans_to_des_map[transition_idx];

                  debug_message_4(fprintf(stderr,"graph_parser: create_parsing_automata: use existing state descriptor: %u\n",sd_idx));

                  transition.clear_evaluations(*this);
                  transition_idx = c_idx_not_exist;
               }

               // - get reference on state descriptor -
               gp_state_descriptor_s &new_state_des = state_descriptors[sd_idx];

               // - retrieve subgraph target vertex first edge -
               unsigned stv_edge_idxs_first_idx = s_trg_vertex.edge_idxs.get_min_value_idx(s_trg_vertex.edge_idxs.root_idx);

               // - create new vertex map block -
               unsigned vmap_idx = create_vertex_map.insert_blank();

               // - reload prev_vmap_ptr -
               prev_vmap_ptr = &create_vertex_map[next_move.prev_vmap_idx];
               
               // - create new vertex map block -
               create_vertex_map[vmap_idx].set(next_move.trg_vertex_idx,a_trg_vertex_idx,next_move.prev_vmap_idx,1,stv_edge_idxs_first_idx,prev_vmap_ptr->length + 1);

               // - increase reference count of previous mapping -
               prev_vmap_ptr->reference_cnt++;

               // - insert graph info -
               new_state_des.gp_si_array.push_blank();
               gp_graph_state_info_s &gs_info = new_state_des.gp_si_array.last();
               gs_info.graph_idx = gp_si.graph_idx;
               gs_info.vmap_idx = vmap_idx;
               gs_info.src_state_idx = states.used - 1;

               // - insert transition idx to state description -
               gs_info.src_transition_idx = transition_idx;
            }

            create_vertex_map.release_vmap_string(next_move.prev_vmap_idx);
         }
      } while(++gsi_idx < state_des.gp_si_array.used);

      state_des.clear();
   } while(state_descriptors.used > 0);

   trans_to_des_map.clear();
   tmp_v_maps.clear();

   debug_assert(create_vertex_map.first_idx == c_idx_not_exist);
}/*}}}*/

void graph_parser_s::create_edge_arrays(graph_array_s &a_subgraphs)
{/*{{{*/
   cassert(a_subgraphs.used != 0);

   // - empty edge_arrays and reallocate its size -
   edge_arrays.used = 0;
   if (edge_arrays.size < a_subgraphs.used) {
      edge_arrays.copy_resize(a_subgraphs.used);
   }

   // - empty edge_cnts_array and reallocate its size -
   edge_cnts_array.used = 0;
   if (edge_cnts_array.size < a_subgraphs.used) {
      edge_cnts_array.copy_resize(a_subgraphs.used);
   }

   unsigned s_idx = 0;
   do {

      // - create reference on subgraph -
      graph_s &subgraph = a_subgraphs[s_idx];

      // - insert new edge array -
      edge_arrays.push_blank();
      ui_array_s &edge_array = edge_arrays.last();
      edge_array.used = 0;

      // - insert new edge counters -
      edge_cnts_array.push_blank();
      ui_array_s &edge_cnts = edge_cnts_array.last();
      edge_cnts.used = 0;

      {
         // - create reference on subgraph vertices and edges -
         vertex_rb_tree_s &s_vertices = subgraph.vertices;
         edge_rb_tree_s &s_edges = subgraph.edges;

         unsigned stack[s_edges.get_descent_stack_size()];
         unsigned *stack_ptr = stack;

         unsigned e_idx = s_edges.get_stack_min_value_idx(s_edges.root_idx,&stack_ptr);
         do {

            // - find all same edges -
            //subgraph.edges.__get_idxs(s_edges[e_idx],tmp_edge_idxs,edge_src_trg_vertex_compare);

            // - optimized find all matching edges -
            edge_cnts.push(0);
            s_vertices[s_edges[e_idx].src_vertex_idx].edge_idxs.get_edge_idxs_by_edge(s_edges[e_idx],edge_array,edge_cnts.last());

            // - find next differend edge -
            edge_s &old_edge = s_edges[e_idx];
            do {
               e_idx = s_edges.get_stack_next_idx(e_idx,&stack_ptr,stack);
            } while(e_idx != c_idx_not_exist && (s_edges.__compare_value(old_edge,s_edges[e_idx]) == 0 && edge_src_trg_vertex_compare(old_edge,s_edges[e_idx])));
         } while(e_idx != c_idx_not_exist);
      }
   } while(++s_idx < a_subgraphs.used);
}/*}}}*/

void graph_parser_s::process_parse_states(graph_array_s &a_subgraphs,graph_s &a_host_graph,gp_parse_states_s &parse_states,gg_vertex_map_set_s &vertex_map,ui_arrays_array_s &a_ret_vmap_arrays_array,mutex_s &sync_mutex)
{/*{{{*/
   ui_array_s actual_to_host_vmap;
   actual_to_host_vmap.init();

   if (parse_states.used != 0) {
      do {
         gp_parse_state_s parse_state = parse_states.pop();
         gp_state_s &state = states[parse_state.state_idx];

         // - process found graph infos -
         if (state.found_graph_info_idxs.used != 0) {

            // - lock synchronization mutex -
            sync_mutex.lock();

            {debug_message_4
            (/*{{{*/
               fprintf(stderr,"graph_parser: process_parse_states: found final state of graphs (idxs): ");

               unsigned *fgii_ptr = state.found_graph_info_idxs.data;
               unsigned *fgii_ptr_end = fgii_ptr + state.found_graph_info_idxs.used;
               do {
                  fprintf(stderr,"%u ",found_graph_infos[*fgii_ptr].graph_idx);
               } while(++fgii_ptr < fgii_ptr_end);
               fputc('\n',stderr);
            );/*}}}*/
            }

            unsigned *fgii_ptr = state.found_graph_info_idxs.data;
            unsigned *fgii_ptr_end = fgii_ptr + state.found_graph_info_idxs.used;
            do {
               gp_found_graph_info_s &fg_info = found_graph_infos[*fgii_ptr];

               // - resize mapping array -
               if (actual_to_host_vmap.used < fg_info.a_graph_vertices_used) {
                  actual_to_host_vmap.used = 0;
                  actual_to_host_vmap.copy_resize(fg_info.a_graph_vertices_used);
                  actual_to_host_vmap.used = fg_info.a_graph_vertices_used;
               }

               vertex_map.retrieve_mapping_sub_to_mother(parse_state.vmap_idx,actual_to_host_vmap);

               // - reference to subgrapg vertex maps, and subgraph vertices -
               ui_arrays_s &vertex_maps = a_ret_vmap_arrays_array[fg_info.graph_idx];
               vertex_rb_tree_s &svertices = a_subgraphs[fg_info.graph_idx].vertices;

               // - insert new vertex map -
               vertex_maps.push_blank();
               ui_array_s &vmap = vertex_maps.last();

               // - resize vertex_map to subgraph.vertices.used -
               vmap.used = 0;
               vmap.copy_resize(svertices.used);
               vmap.used = svertices.used;

               unsigned stack[svertices.get_descent_stack_size()];
               unsigned *stack_ptr = stack;

               // - traverse throught subgraph vertices -
               unsigned v_idx = svertices.get_stack_min_value_idx(svertices.root_idx,&stack_ptr);
               do {
                  vmap[v_idx] = actual_to_host_vmap[fg_info.subgraph_to_actual_vmap[v_idx]];
                  v_idx = svertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
               } while(v_idx != c_idx_not_exist);
            } while(++fgii_ptr < fgii_ptr_end);

            // - unlock synchronization mutex -
            sync_mutex.unlock();
         }

         // - get references -
         gg_vertex_map_s *vmap_ptr = &vertex_map[parse_state.vmap_idx];

         // - process state transitions -
         if (state.transitions.used != 0) {
            gp_state_transition_s *st_ptr = state.transitions.data;
            gp_state_transition_s *st_ptr_end = st_ptr + state.transitions.used;
            do {
               unsigned host_vertex_idx;
               if (st_ptr->ag_src_vertex_idx == vmap_ptr->sv_idx) {
                  host_vertex_idx = vmap_ptr->mv_idx;
               }
               else {
                  unsigned tmp_vmap_idx = vertex_map.get_idx_by_sub_vertex(parse_state.vmap_idx,st_ptr->ag_src_vertex_idx);
                  cassert(tmp_vmap_idx != c_idx_not_exist);

                  host_vertex_idx = vertex_map[tmp_vmap_idx].mv_idx;
               }

               // - reference on host vertex edge indexes -
               edge_idxs_s &hv_edge_idxs = a_host_graph.vertices[host_vertex_idx].edge_idxs;
               edge_rb_tree_s &h_edges = a_host_graph.edges;

               if (hv_edge_idxs.root_idx != c_idx_not_exist) {
                  unsigned stack[hv_edge_idxs.get_descent_stack_size()];
                  unsigned *stack_ptr = stack;

                  unsigned ei_idx = hv_edge_idxs.get_stack_min_value_idx(hv_edge_idxs.root_idx,&stack_ptr);
                  do {
                     edge_s &edge = h_edges[hv_edge_idxs[ei_idx]];

                     do {
                        unsigned trg_vertex_idx;
                        if (st_ptr->edge_direction) {
                           if (edge.src_vertex_idx != host_vertex_idx) continue;
                           trg_vertex_idx = edge.trg_vertex_idx;
                        }
                        else {
                           if (edge.trg_vertex_idx != host_vertex_idx) continue;
                           trg_vertex_idx = edge.src_vertex_idx;
                        }

                        // - reference on target host vertex -
                        vertex_s &h_trg_vertex = a_host_graph.vertices[trg_vertex_idx];

                        bool degrees_ok = st_ptr->trg_vertex_join?
                                          h_trg_vertex.input_degree >= st_ptr->trg_vertex_in_degree && h_trg_vertex.output_degree >= st_ptr->trg_vertex_out_degree
                                          :h_trg_vertex.input_degree == st_ptr->trg_vertex_in_degree && h_trg_vertex.output_degree == st_ptr->trg_vertex_out_degree;

                        if (degrees_ok && ef_compare(st_ptr->edge_evaluation,edge.value) == 0 && vf_compare(st_ptr->trg_vertex_evaluation,h_trg_vertex.value) == 0) {

                           if (vertex_map.get_idx_by_mother_vertex(parse_state.vmap_idx,trg_vertex_idx) == c_idx_not_exist) {

                              // - create new vertex mapping -
                              unsigned vmap_idx = vertex_map.insert_blank();

                              // - reload vmap_ptr -
                              vmap_ptr = &vertex_map[parse_state.vmap_idx];

                              // - set new vertex map -
                              vertex_map[vmap_idx].set(st_ptr->ag_trg_vertex_idx,trg_vertex_idx,parse_state.vmap_idx,1,c_idx_not_exist,vmap_ptr->length + 1);

                              // - increase refernce count of previous mapping -
                              vmap_ptr->reference_cnt++;

                              // - create new parse state -
                              parse_states.push_blank();
                              parse_states.last().set(st_ptr->target_state,vmap_idx);

                              
                              {debug_message_4
                              (/*{{{*/
                                 string_s e_str;
                                 string_s v_str;

                                 e_str.init();
                                 v_str.init();

                                 ef_get_string(st_ptr->edge_evaluation,e_str);
                                 vf_get_string(st_ptr->trg_vertex_evaluation,v_str);

                                 fprintf(stderr,"graph_parser: process_parse_states: move (%u -> %u) throught edge (eval.): %s to vertex (eval.): %s\n",parse_state.state_idx,st_ptr->target_state,e_str.data,v_str.data);

                                 v_str.clear();
                                 e_str.clear();
                              );/*}}}*/
                              }
                           }
                        }
                     } while(0);

                     // - find next diferend edge -
                     unsigned old_edge_idx = hv_edge_idxs[ei_idx];
                     do {
                        ei_idx = hv_edge_idxs.get_stack_next_idx(ei_idx,&stack_ptr,stack);
                     } while(ei_idx != c_idx_not_exist && hv_edge_idxs.compare_edges(h_edges[old_edge_idx],h_edges[hv_edge_idxs[ei_idx]]) == 0);
                  } while(ei_idx != c_idx_not_exist);
               }
            } while(++st_ptr < st_ptr_end);
         }

         vertex_map.release_vmap_string(parse_state.vmap_idx);
      } while(parse_states.used > 0);
   }

   actual_to_host_vmap.clear();
}/*}}}*/

void graph_parser_s::find_isomorphic_subdigraphs(graph_array_s &a_subgraphs,graph_s &a_host_graph,ui_arrays_array_s &a_ret_vmap_arrays_array)
{/*{{{*/
   cassert(found_graph_infos.used != 0 && states.used != 0 && a_subgraphs.used != 0 &&
   a_subgraphs[0].vertices.value_fps == a_host_graph.vertices.value_fps && a_subgraphs[0].edges.value_fps == a_host_graph.edges.value_fps);

   // - initialize a_ret_vmap_arrays_array -
   if (a_ret_vmap_arrays_array.used != a_subgraphs.used) {
      a_ret_vmap_arrays_array.used = 0;
      a_ret_vmap_arrays_array.copy_resize(a_subgraphs.used);
      a_ret_vmap_arrays_array.used = a_subgraphs.used;
   }

   // - clear all vmap arrays -
   ui_arrays_s *a_ptr = a_ret_vmap_arrays_array.data;
   ui_arrays_s *a_ptr_end = a_ptr + a_ret_vmap_arrays_array.used;
   do {
      a_ptr->used = 0;
   } while(++a_ptr < a_ptr_end);

   // - create parse_states array -
   gp_parse_states_array_s parse_states_array;
   parse_states_array.init_size(c_cpu_core_cnt);
   parse_states_array.used = parse_states_array.size;

   // - create vertex_map_array -
   gg_vertex_map_sets_s vertex_map_array;
   vertex_map_array.init_size(c_cpu_core_cnt);
   vertex_map_array.used = vertex_map_array.size;

   // - initialize parse state and vertex mapping arrays -
   {
      unsigned c_idx = 0;
      do {
         parse_states_array[c_idx].init();
         vertex_map_array[c_idx].init();
      } while(++c_idx < c_cpu_core_cnt);
   }

   debug_message_3(fprintf(stderr,"graph_parser: find_isomorphic_subdigraphs: create initial parse states\n"));

   // - create initial parse states -
   {
      gp_state_s &first_state = states[0];
      debug_assert(first_state.found_graph_info_idxs.used == 0 && first_state.transitions.used != 0);

      vertex_s vertex;
      vertex.init();

      ui_array_s found_v_idxs;
      found_v_idxs.init();

      // - parse state and vertex map index -
      unsigned ps_vm_idx = 0;

      // - traverse throught first state transitions -
      gp_state_transition_s *st_ptr = first_state.transitions.data;
      gp_state_transition_s *st_ptr_end = st_ptr + first_state.transitions.used;
      do {
         vertex.value = st_ptr->trg_vertex_evaluation;
         a_host_graph.vertices.get_idxs(vertex,found_v_idxs);

         // - process found vertices -
         if (found_v_idxs.used != 0) {

            {debug_message_4
            (/*{{{*/
               string_s v_str;
               v_str.init();

               vf_get_string(vertex.value,v_str);
               fprintf(stderr,"graph_parser: find_isomorphic_subdigraphs: for vertex (eval.) in brackets target state: %s, found: ",v_str.data);

               unsigned *v_ptr = found_v_idxs.data;
               unsigned *v_ptr_end = v_ptr + found_v_idxs.used;
               do {
                  fprintf(stderr,"%u(%u) ",*v_ptr,st_ptr->target_state);
               } while(++v_ptr < v_ptr_end);
               fputc('\n',stderr);

               v_str.clear();
            );/*}}}*/
            }

            unsigned *v_ptr = found_v_idxs.data;
            unsigned *v_ptr_end = v_ptr + found_v_idxs.used;
            do {
               vertex_s &h_trg_vertex = a_host_graph.vertices[*v_ptr];
               
               bool degrees_ok = st_ptr->trg_vertex_join?
                                 h_trg_vertex.input_degree >= st_ptr->trg_vertex_in_degree && h_trg_vertex.output_degree >= st_ptr->trg_vertex_out_degree
                                 :h_trg_vertex.input_degree == st_ptr->trg_vertex_in_degree && h_trg_vertex.output_degree == st_ptr->trg_vertex_out_degree;

               if (degrees_ok) {

                  // - create new vertex mapping -
                  unsigned vmap_idx = vertex_map_array[ps_vm_idx].insert_blank();
                  vertex_map_array[ps_vm_idx][vmap_idx].set(st_ptr->ag_trg_vertex_idx,*v_ptr,c_idx_not_exist,1,c_idx_not_exist,1);

                  // - create new parse state -
                  parse_states_array[ps_vm_idx].push_blank();
                  parse_states_array[ps_vm_idx].last().set(st_ptr->target_state,vmap_idx);

                  // - change parse state and vertex map index -
                  ++ps_vm_idx >= c_cpu_core_cnt?ps_vm_idx = 0:0;
               }
            } while(++v_ptr < v_ptr_end);
         }
      } while(++st_ptr < st_ptr_end);

      found_v_idxs.clear();
      vertex.clear();
   }

   ui_array_s actual_to_host_vmap;
   actual_to_host_vmap.init();

   debug_message_3(fprintf(stderr,"graph_parser: find_isomorphic_subdigraphs: process existing parse states (start of stack state processing)\n"));

   // - parallel process existing parse states -
   {
      unsigned cpu_ccm1 = c_cpu_core_cnt - 1;

      mutex_s sync_mutex;
      sync_mutex.init();
      
      pointer process_infos[cpu_ccm1][7];
      thread_s threads[cpu_ccm1];

      if (c_cpu_core_cnt > 1) {
         unsigned c_idx = 0;
         do {
            pointer *process_info = process_infos[c_idx];

            process_info[0] = this;
            process_info[1] = &a_subgraphs;
            process_info[2] = &a_host_graph;
            process_info[3] = &parse_states_array[c_idx];
            process_info[4] = &vertex_map_array[c_idx];
            process_info[5] = &a_ret_vmap_arrays_array;
            process_info[6] = &sync_mutex;
            
            thread_s &thread = threads[c_idx];
            thread.init();

            // - run parallel thread -
            int ret = thread.create((void *)process_parser_states_thread_function,(void *)process_info);
            cassert(ret == c_error_OK);

         } while(++c_idx < cpu_ccm1);
      }

      process_parse_states(a_subgraphs,a_host_graph,parse_states_array[cpu_ccm1],vertex_map_array[cpu_ccm1],a_ret_vmap_arrays_array,sync_mutex);

      if (cpu_ccm1) {
         unsigned c_idx = 0;
         do {
            thread_s &thread = threads[c_idx];

            thread.join(NULL);
            thread.clear();
         } while(++c_idx < cpu_ccm1);
      }

      sync_mutex.clear();
   }

   actual_to_host_vmap.clear();

#if DEBUG_ASSERT == ENABLE
   {
      gg_vertex_map_set_s *vm_ptr = vertex_map_array.data;
      gg_vertex_map_set_s *vm_ptr_end = vm_ptr + vertex_map_array.used;
      do {
         cassert(vm_ptr->first_idx == c_idx_not_exist);
      } while(++vm_ptr < vm_ptr_end);
   }
#endif

   vertex_map_array.clear();
   parse_states_array.clear();

}/*}}}*/

void graph_parser_s::find_edge_mapping(unsigned a_s_idx,graph_s &a_subgraph,graph_s &a_host_graph,ui_arrays_s &a_vertex_maps_array,ui_array_s &a_vertex_map_idxs,ui_arrays_s &a_edge_maps_array,bool a_verify_vertex_maps,ui_array_s &a_tmp_edge_idxs,ui_arrays_array_s &a_graph_edge_mapping_arrays,ui_arrays_s &a_graph_edge_arrays)
{/*{{{*/
   if (a_subgraph.edge_cnt != 0 && a_vertex_maps_array.used != 0) {

      // - resize a_graph_edge_mapping_arrays -
      a_graph_edge_mapping_arrays.used = 0;
      if (a_graph_edge_mapping_arrays.size < a_vertex_maps_array.used) {
         a_graph_edge_mapping_arrays.copy_resize(a_vertex_maps_array.used);
      }

      // - test vertex mapping, and create final edge mapping -
      unsigned vm_idx = 0;
      do {
         
         // - insert next graph edge mapping array -
         a_graph_edge_mapping_arrays.push_blank();
         ui_arrays_s &graph_edge_mapping_array = a_graph_edge_mapping_arrays.last();
         graph_edge_mapping_array.used = 0;

         ui_array_s &vertex_map = a_vertex_maps_array[vm_idx];
         
         unsigned *ea_data_ptr = edge_arrays[a_s_idx].data;
         unsigned *ea_size_ptr = edge_cnts_array[a_s_idx].data;
         unsigned *ea_size_ptr_end = ea_size_ptr + edge_cnts_array[a_s_idx].used;
         do {

            // - clear a_graph_edge_arrays -
            a_graph_edge_arrays.used = 0;

            // - get reference on first edge of set -
            edge_s &s_edge = a_subgraph.edges[*ea_data_ptr];

            edge_s m_edge;
            m_edge.set(s_edge.value,vertex_map[s_edge.src_vertex_idx],vertex_map[s_edge.trg_vertex_idx]);

            // - find all matching edges -
            //a_host_graph.edges.__get_idxs(m_edge,a_tmp_edge_idxs,edge_src_trg_vertex_compare);

            // - optimized find all matching edges -
            {
               a_tmp_edge_idxs.used = 0;
               unsigned edge_cnt = 0;

               a_host_graph.vertices[m_edge.src_vertex_idx].edge_idxs.get_edge_idxs_by_edge(m_edge,a_tmp_edge_idxs,edge_cnt);
            }

            if (a_verify_vertex_maps) {
               if (a_tmp_edge_idxs.used < *ea_size_ptr) {
                  vertex_map.used = 0;

                  // - BREAK -
                  break;
               }
            }
            else {
               if (a_tmp_edge_idxs.used >= *ea_size_ptr) {

                  debug_message_5
                  (/*{{{*/
                     fprintf(stderr,"graph_parser_s: find_edge_mappings: edge_mapping: (%4u",a_tmp_edge_idxs[0]);

                     if (a_tmp_edge_idxs.used > 1) {
                        unsigned *ptr = a_tmp_edge_idxs.data + 1;
                        unsigned *ptr_end = a_tmp_edge_idxs.data + a_tmp_edge_idxs.used;
                        do {
                           fprintf(stderr,",%4u",*ptr);
                        } while(++ptr < ptr_end);
                     }

                     fprintf(stderr,") -> (%u",*ea_data_ptr);

                     if (*ea_size_ptr > 1) {
                        unsigned *ptr = ea_data_ptr + 1;
                        unsigned *ptr_end = ea_data_ptr + *ea_size_ptr;
                        do {
                           fprintf(stderr,",%u",*ptr);
                        } while(++ptr < ptr_end);
                     }

                     fprintf(stderr,")\n");
                  );/*}}}*/

                  unsigned p_map[*ea_size_ptr];
                  unsigned p_iter[*ea_size_ptr];
                  unsigned perm[a_tmp_edge_idxs.used];

                  memset(p_map,c_idx_not_exist,*ea_size_ptr*sizeof(unsigned));
                  memset(p_iter,0,*ea_size_ptr*sizeof(unsigned));
                  memset(perm,c_idx_not_exist,a_tmp_edge_idxs.used*sizeof(unsigned));

                  // - find all posible permutations of matching edges mapping -
                  unsigned e_idx = 0;
                  do {
                     
                     // - clear mapping from actual permutaion -
                     if (p_map[e_idx] != c_idx_not_exist) {
                        perm[p_map[e_idx]] = c_idx_not_exist;
                        p_map[e_idx] = c_idx_not_exist;
                     }

                     // - find next possible position -
                     while(p_iter[e_idx] < a_tmp_edge_idxs.used && perm[p_iter[e_idx]] != c_idx_not_exist) {
                        p_iter[e_idx]++;
                     }

                     // - if next position is OK -
                     if (p_iter[e_idx] >= a_tmp_edge_idxs.used) {
                        if (e_idx == 0) {
                           break;
                        }
                        else {
                           p_iter[e_idx] = 0;
                           e_idx--;
                           continue;
                        }
                     }

                     p_map[e_idx] = p_iter[e_idx];
                     perm[p_iter[e_idx]] = e_idx;

                     p_iter[e_idx]++;

                     if (++e_idx >= *ea_size_ptr) {
                        debug_message_5
                        (/*{{{*/
                           fprintf(stderr,"graph_parser_s: find_edge_mappings: permutation:  |");
                           
                           unsigned *ptr = perm;
                           unsigned *ptr_end = ptr + a_tmp_edge_idxs.used;
                           do {
                              if (*ptr == c_idx_not_exist) {
                                 fprintf(stderr,"   X|");
                              }
                              else {
                                 fprintf(stderr,"%4u|",ea_data_ptr[*ptr]);
                              }
                           } while(++ptr < ptr_end);

                           fputc('\n',stderr);
                        );/*}}}*/

                        // - insert edge array to graph edge arrays -
                        a_graph_edge_arrays.push_blank();
                        a_graph_edge_arrays.last().set(*ea_size_ptr,p_map);

                        p_iter[e_idx] = 0;
                        e_idx--;
                     }
                  } while(1);
               }
               else {
                  debug_message_5(fprintf(stderr,"graph_parser_s: find_edge_mappings: cannot find subgraph edges mapping -> cancel vertex mapping\n"););

                  graph_edge_mapping_array.used = 0;

                  // - BREAK -
                  break;
               }

               // - create (multiply mappings of edges), result: final mapping of edges on given vertex mapping -
               if (graph_edge_mapping_array.used == 0) {
                  graph_edge_mapping_array.copy_resize(a_graph_edge_arrays.used);
                  graph_edge_mapping_array.used = a_graph_edge_arrays.used;

                  ui_array_s *m_ptr = a_graph_edge_arrays.data;
                  ui_array_s *m_ptr_end = m_ptr + a_graph_edge_arrays.used;
                  ui_array_s *gem_ptr = graph_edge_mapping_array.data;
                  do {
                     gem_ptr->copy_resize(a_subgraph.edges.used);
                     gem_ptr->used = a_subgraph.edges.used;

                     unsigned idx = 0;
                     do {
                        gem_ptr->data[ea_data_ptr[idx]] = a_tmp_edge_idxs[m_ptr->data[idx]];
                     } while(++idx < *ea_size_ptr);
                  } while(++gem_ptr,++m_ptr < m_ptr_end);
               }
               else {
                  unsigned old_graph_edge_mapping_array_used = graph_edge_mapping_array.used;

                  if (a_graph_edge_arrays.used > 1) {
                     graph_edge_mapping_array.copy_resize(a_graph_edge_arrays.used*graph_edge_mapping_array.used);
                     graph_edge_mapping_array.used = a_graph_edge_arrays.used*graph_edge_mapping_array.used;

                     unsigned idx = old_graph_edge_mapping_array_used;
                     do {
                        graph_edge_mapping_array[idx] = graph_edge_mapping_array[idx%old_graph_edge_mapping_array_used];
                     } while(++idx < graph_edge_mapping_array.used);
                  }

                  unsigned idx = 0;
                  do {
                     ui_array_s &edge_map = graph_edge_mapping_array[idx];
                     ui_array_s &graph_edge_array = a_graph_edge_arrays[idx/old_graph_edge_mapping_array_used];

                     unsigned idx = 0;
                     do {
                        edge_map[ea_data_ptr[idx]] = a_tmp_edge_idxs[graph_edge_array[idx]];
                     } while(++idx < *ea_size_ptr);
                  } while(++idx < graph_edge_mapping_array.used);
               }
            }
         } while(ea_data_ptr += *ea_size_ptr,++ea_size_ptr < ea_size_ptr_end);
      } while(++vm_idx < a_vertex_maps_array.used);

      if (a_verify_vertex_maps) {
         unsigned tvm_idx = 0;
         unsigned svm_idx = 0;
         do {
            ui_array_s &vertex_map = a_vertex_maps_array[svm_idx];

            if (vertex_map.used != 0) {
               if (svm_idx != tvm_idx) {
                  vertex_map.swap(a_vertex_maps_array[tvm_idx]);
               }

               tvm_idx++;
            }
         } while(++svm_idx < a_vertex_maps_array.used);

         a_vertex_maps_array.used = tvm_idx;
      }
      else {

         // - create final vertex and edge mapping -
         {
            unsigned vm_idx = 0;
            do {
               ui_arrays_s &graph_edge_mapping_array = a_graph_edge_mapping_arrays[vm_idx];

               if (graph_edge_mapping_array.used != 0) {
                  ui_array_s *a_ptr = graph_edge_mapping_array.data;
                  ui_array_s *a_ptr_end = a_ptr + graph_edge_mapping_array.used;
                  do {
                     
                     // - store vertex map index -
                     a_vertex_map_idxs.push(vm_idx);

                     // - swap edge map -
                     a_edge_maps_array.push_blank();
                     a_edge_maps_array.last().swap(*a_ptr);
                  } while(++a_ptr < a_ptr_end);
               }
            } while(++vm_idx < a_vertex_maps_array.used);
         }
      }
   }
}/*}}}*/

void graph_parser_s::find_edge_mappings(graph_array_s &a_subgraphs,graph_s &a_host_graph,ui_arrays_array_s &a_vertex_maps_arrays,ui_arrays_s &a_vertex_map_idxs_array,ui_arrays_array_s &a_edge_maps_arrays,bool a_verify_vertex_maps)
{/*{{{*/
   cassert(a_subgraphs.used != 0 && a_subgraphs.used == a_vertex_maps_arrays.used);

   // - initialize a_vertex_map_idxs_array -
   if (a_vertex_map_idxs_array.used != a_subgraphs.used) {
      a_vertex_map_idxs_array.used = 0;
      a_vertex_map_idxs_array.copy_resize(a_subgraphs.used);
      a_vertex_map_idxs_array.used = a_subgraphs.used;
   }

   // - initialize a_edge_maps_arrays -
   if (a_edge_maps_arrays.used != a_subgraphs.used) {
      a_edge_maps_arrays.used = 0;
      a_edge_maps_arrays.copy_resize(a_subgraphs.used);
      a_edge_maps_arrays.used = a_subgraphs.used;
   }

   // - clear all a_vertex_map_idxs_array and a_edge_maps_arrays -
   ui_array_s *vmi_ptr = a_vertex_map_idxs_array.data;
   ui_array_s *vmi_ptr_end = vmi_ptr + a_vertex_map_idxs_array.used;
   ui_arrays_s *ema_ptr = a_edge_maps_arrays.data;
   do {
      vmi_ptr->used = 0;
      ema_ptr->used = 0;
   } while(++ema_ptr,++vmi_ptr < vmi_ptr_end);

   ui_array_s tmp_edge_idxs;
   tmp_edge_idxs.init();

   // - create graph edge mapping arrays -
   ui_arrays_array_s graph_edge_mapping_arrays;
   graph_edge_mapping_arrays.init();

   ui_arrays_s graph_edge_arrays;
   graph_edge_arrays.init();

   // - test edges for each of subgraphs -
   unsigned s_idx = 0;
   do {
      find_edge_mapping(s_idx,a_subgraphs[s_idx],a_host_graph,a_vertex_maps_arrays[s_idx],a_vertex_map_idxs_array[s_idx],a_edge_maps_arrays[s_idx],a_verify_vertex_maps,tmp_edge_idxs,graph_edge_mapping_arrays,graph_edge_arrays);
   } while(++s_idx < a_subgraphs.used);

   graph_edge_arrays.clear();
   graph_edge_mapping_arrays.clear();
   tmp_edge_idxs.clear();
}/*}}}*/

void graph_parser_s::DEBUG_print_states(FILE *a_file)
{/*{{{*/
   cassert(states.used != 0);

   string_s e_str;
   string_s v_str;

   e_str.init();
   v_str.init();

   unsigned s_idx = 0;
   do {
      gp_state_s &state = states[s_idx];

      fprintf(a_file," -- STATE: %u --\n",s_idx);

      if (state.found_graph_info_idxs.used != 0) {
         unsigned *ptr = state.found_graph_info_idxs.data;
         unsigned *ptr_end = ptr + state.found_graph_info_idxs.used;
         do {
            gp_found_graph_info_s &fgi = found_graph_infos[*ptr];
            fprintf(a_file,"   - GRAPH FINAL: %u - \n",fgi.graph_idx);
         } while(++ptr < ptr_end);
      }

      if (state.transitions.used != 0) {
         gp_state_transition_s *st_ptr = state.transitions.data;
         gp_state_transition_s *st_ptr_end = st_ptr + state.transitions.used;
         do {
            
            ef_get_string(st_ptr->edge_evaluation,e_str);
            vf_get_string(st_ptr->trg_vertex_evaluation,v_str);

            fprintf(a_file,
"   - TRANSITION: %u -\n"
"      ag_src_vertex_idx: %u\n"
"      ag_trg_vertex_idx: %u\n"
"      edge_evaluation: %s\n"
"      edge_direction: %s\n"
"      trg_vertex_evaluation: %s\n"
"      trg_vertex_in_degree: %u\n"
"      trg_vertex_out_degree: %u\n"
"      trg_vertex_join: %s\n"
"      target_state: %u\n"
            ,(unsigned)(st_ptr - state.transitions.data),
            st_ptr->ag_src_vertex_idx,
            st_ptr->ag_trg_vertex_idx,
            e_str.data,
            st_ptr->edge_direction?"forward":"backward",
            v_str.data,
            st_ptr->trg_vertex_in_degree,
            st_ptr->trg_vertex_out_degree,
            st_ptr->trg_vertex_join?"true":"false",
            st_ptr->target_state);

         } while(++st_ptr < st_ptr_end);
      }

      fputc('\n',a_file);
   } while(++s_idx < states.used);

   e_str.clear();
   v_str.clear();
}/*}}}*/

void graph_parser_s::DEBUG_print_states_latex_table(FILE *a_file,graph_array_s &a_subgraphs)
{/*{{{*/
   cassert(states.used != 0 && a_subgraphs.used != 0);

   string_s str;
   str.init();

   unsigned s_vmap_max_size = 0;

   // - print states and their transitions -
   fprintf(a_file,
"\\begin{table}[ht]\n"
"\\centering\n"
"\\begin{tabular}{|c|c|c|c|c|c|c|c|}\n"
"\\hline\n"
"state & transition & ag-src & ag-trg & in-degree & out-degree & direction & target state\\\\\n"
   );

   // - for every state -
   unsigned s_idx = 0;
   do {
      gp_state_s &state = states[s_idx];

      // - find length og longest subgraph vertex map -
      if (state.found_graph_info_idxs.used != 0) {
         unsigned *ptr = state.found_graph_info_idxs.data;
         unsigned *ptr_end = ptr + state.found_graph_info_idxs.used;
         do {
            gp_found_graph_info_s &fgi = found_graph_infos[*ptr];

            if (s_vmap_max_size < fgi.a_graph_vertices_used) {
               s_vmap_max_size = fgi.a_graph_vertices_used;
            }
         } while(++ptr < ptr_end);
      }

      // - if state contain any transitions -
      if (state.transitions.used != 0) {
         fprintf(a_file,
"\\hline\n"
"\\multirow{%u}{*}{"
         ,state.transitions.used);

         // - name first state as "S" -
         if (s_idx == 0) {
            fprintf(a_file,"S} ");
         }
         else {
            fprintf(a_file,"%u} ",s_idx);
         }

         // - traverse throught transitions -
         gp_state_transition_s *st_ptr = state.transitions.data;
         gp_state_transition_s *st_ptr_end = st_ptr + state.transitions.used;
         do {
            
            if (st_ptr->ag_src_vertex_idx == c_idx_not_exist) {
               str.setf("none");
            }
            else {
               str.setf("%u",st_ptr->ag_src_vertex_idx);
            }

            fprintf(a_file,
"& %u & %s & %u & %u & %u & %s & %u\\\\\n"
            ,(unsigned)(st_ptr - state.transitions.data)
            ,str.data
            ,st_ptr->ag_trg_vertex_idx
            ,st_ptr->trg_vertex_in_degree
            ,st_ptr->trg_vertex_out_degree
            ,st_ptr->edge_direction?"forward":"backward"
            ,st_ptr->target_state);
         } while(++st_ptr < st_ptr_end);
      }
   } while(++s_idx < states.used);

   fprintf(a_file,
"\\hline\n"
"\\end{tabular}\n"
"\\caption{States of graph parser generated for example graphs}\n"
"\\label{tab_states_of_graph_parser_generated_for_example_graphs}\n"
"\\end{table}\n"
"\n"
   );

   // - print final states and final graph vertex maps -
   fprintf(a_file,
"\\begin{table}[ht]\n"
"\\centering\n"
"\\begin{tabular}{|c|c|"
   );

   unsigned idx = 0;
   do {
      fprintf(a_file,"c|");
   } while(++idx < s_vmap_max_size);
   fprintf(a_file,"}\n");

   fprintf(a_file,
"\\hline\n"
"state & graph & \\multicolumn{%u}{c|}{vertex mappings} \\\\\n"
   ,s_vmap_max_size);

   s_idx = 0;
   do {
      gp_state_s &state = states[s_idx];

      if (state.found_graph_info_idxs.used != 0) {
         fprintf(a_file,
"\\hline\n"
"\\multirow{%u}{*}{%u} "
         ,state.found_graph_info_idxs.used,s_idx);

         unsigned *ptr = state.found_graph_info_idxs.data;
         unsigned *ptr_end = ptr + state.found_graph_info_idxs.used;
         do {
            gp_found_graph_info_s &fgi = found_graph_infos[*ptr];
            graph_s &subgraph = a_subgraphs[fgi.graph_idx];
            ui_array_s &sa_vmap = fgi.subgraph_to_actual_vmap;

            // - print final graph index -
            fprintf(a_file,"& %u ",fgi.graph_idx);

            // - print subgraph to vmap vertex mapping -
            unsigned stack[subgraph.vertices.get_descent_stack_size()];
            unsigned *stack_ptr = stack;

            unsigned v_idx = subgraph.vertices.get_stack_min_value_idx(subgraph.vertices.root_idx,&stack_ptr);
            do {
               fprintf(a_file,"& %u $\\rightarrow$ %u ",v_idx,sa_vmap[v_idx]);

               v_idx = subgraph.vertices.get_stack_next_idx(v_idx,&stack_ptr,stack);
            } while(v_idx != c_idx_not_exist);

            if (fgi.a_graph_vertices_used < s_vmap_max_size) {
               unsigned idx = fgi.a_graph_vertices_used;
               do {
                  fprintf(a_file,"& $-$ ");
               } while(++idx < s_vmap_max_size);
            }

            fprintf(a_file,"\\\\\n");
         } while(++ptr < ptr_end);
      }
   } while(++s_idx < states.used);

   fprintf(a_file,
"\\hline\n"
"\\end{tabular}\n"
"\\caption{Final states and vertex mapping of subgraphs}\n"
"\\label{tab_final_states_and_vertex_mapping_of_subgraphs}\n"
"\\end{table}\n"
"\n"
   );

   str.clear();
}/*}}}*/

void graph_parser_s::DEBUG_print_state_graph(FILE *a_file)
{/*{{{*/
   cassert(states.used != 0);

   fprintf(a_file,
"digraph G {\n"
"   rankdir = LR\n"
"   node [ shape = circle margin = 0.0 fixedsize = true width = 0.55 height = 0.55]\n"
"\n"
   );

   // - print first state node -
   fprintf(a_file,"   state_0 [ label = \"S\" color = \"0.5,0.5,0.6\" penwidth = 2.0 ]\n");

   // - print state transitions -
   unsigned s_idx = 0;
   do {

      // - reference on state transitions -
      gp_state_transitions_s &transitions = states[s_idx].transitions;

      // - if exist some transition from this state -
      if (transitions.used != 0) {
         gp_state_transition_s *st_ptr = transitions.data;
         gp_state_transition_s *st_ptr_end = st_ptr + transitions.used;
         do {

            // - print target node info -
            fprintf(a_file,"   state_%u [ label = %u ",st_ptr->target_state,st_ptr->target_state);

            if (states[st_ptr->target_state].found_graph_info_idxs.used != 0) {
               fprintf(a_file,"color = \"0.5,0.5,0.6\" penwidth = 2.0 ");
            }

            fprintf(a_file,"]\n");

            // - print transition edge info -
            fprintf(a_file,"   state_%u -> state_%u [ label = %u ]\n",s_idx,st_ptr->target_state,st_ptr - transitions.data);

         } while(++st_ptr < st_ptr_end);
      }
   } while(++s_idx < states.used);

   fprintf(a_file,
"}\n"
   );
}/*}}}*/

