
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

#ifndef __GRAPH_PARSER_H
#define __GRAPH_PARSER_H

/*
 * header files
 */

@begin
   include "basic.h"
   include "thread.h"
   include "mem_check.h"
   include "structures.h"
   include "string.h"
   include "graph.h"
@end

/*
 * constants and definitions
 */

typedef struct graph_parser_s graph_parser_s;

/*
 * global functions
 */

void *process_parser_states_thread_function(void *process_info);

/*
 * definition of generated structures
 */

// -- graph_array_s --
@begin
   array<graph_s> 
   
   additions {
      void remove_by_idx(unsigned a_idx);
   }

   graph_array_s;
@end

// -- gp_graph_state_info_s --
@begin
   struct
   <
   unsigned:graph_idx
   unsigned:vmap_idx
   unsigned:src_state_idx
   unsigned:src_transition_idx
   >
   gp_graph_state_info_s;
@end

// -- gp_graph_state_info_array_s --
@begin
   array<gp_graph_state_info_s> gp_graph_state_info_array_s;
@end

// -- gp_state_descriptor_s --
@begin
   struct
   <
   unsigned:vertex_counter
   gp_graph_state_info_array_s:gp_si_array
   >
   gp_state_descriptor_s;
@end

// -- gp_state_descriptors_s --
@begin
   array<gp_state_descriptor_s> gp_state_descriptors_s;
@end

// -- gp_state_transition_s --
@begin
   struct
   <
   unsigned:ag_src_vertex_idx
   unsigned:ag_trg_vertex_idx
   pointer:edge_evaluation
   bool:edge_direction
   pointer:trg_vertex_evaluation
   
   unsigned:trg_vertex_in_degree
   unsigned:trg_vertex_out_degree
   bool:trg_vertex_join

   unsigned:target_state
   >

   additions {
      inline void clear_evaluations(graph_parser_s &gp);
   }

   gp_state_transition_s;
@end

// -- gp_state_transitions_s --
@begin
   array<gp_state_transition_s> 
  
   additions {
      unsigned get_idx_with_evaluations(graph_parser_s &a_gp,gp_state_transition_s &a_transition);
   }

   gp_state_transitions_s;
@end

// -- gp_state_s --
@begin
   struct
   <
   gp_state_transitions_s:transitions
   ui_array_s:found_graph_info_idxs
   >
   gp_state_s;
@end

// -- gp_states_s --
@begin
   array<gp_state_s> gp_states_s;
@end

// -- gp_next_move_s --
@begin
   struct
   <
   unsigned:edge_idx
   unsigned:trg_vertex_idx
   unsigned:prev_vmap_idx
   >
   gp_next_move_s;
@end

// -- gp_found_graph_info_s --
@begin
   struct
   <
   unsigned:graph_idx
   unsigned:a_graph_vertices_used
   ui_array_s:subgraph_to_actual_vmap
   >
   gp_found_graph_info_s;
@end

// -- gp_found_graph_infos_s --
@begin
   array<gp_found_graph_info_s> gp_found_graph_infos_s;
@end

// -- gp_parse_state_s --
@begin
   struct
   <
   unsigned:state_idx
   unsigned:vmap_idx
   >
   gp_parse_state_s;
@end

// -- gp_parse_states_s --
@begin
   array<gp_parse_state_s> gp_parse_states_s;
@end

// -- gp_parse_states_array_s --
@begin
   array<gp_parse_states_s> gp_parse_states_array_s;
@end

// -- graph_parser_s --
@begin
   struct
   <
   pointer_ptr:vertices_value_fps
   pointer_ptr:edges_value_fps
   gg_vertex_map_set_s:create_vertex_map
   gp_found_graph_infos_s:found_graph_infos

   ui_arrays_s:edge_arrays
   ui_arrays_s:edge_cnts_array

   gp_states_s:states
   gp_state_descriptors_s:state_descriptors
   >

   not_generate: init clear

   additions {
      inline void init();
      inline void clear();

      inline void vf_init(void **a_value_ptr);
      inline int vf_compare(void *a_first,void *a_second);
      inline void vf_operator_equal(void **a_trg,void *a_src);
      inline void vf_get_string(void *a_value,string_s &a_string);
      inline void vf_clear(void *a_value);

      inline void ef_init(void **a_value_ptr);
      inline int ef_compare(void *a_first,void *a_second);
      inline void ef_operator_equal(void **a_trg,void *a_src);
      inline void ef_get_string(void *a_value,string_s &a_string);
      inline void ef_clear(void *a_value);

      bool find_next_move(graph_array_s &a_subgraphs,gp_graph_state_info_s &a_gp_si,gp_next_move_s &a_next_move);
      void create_parsing_automata(graph_array_s &a_subgraphs,ui_arrays_s &a_join_vertex_maps);
      void create_edge_arrays(graph_array_s &a_subgraphs);

      void process_parse_states(graph_array_s &a_subgraphs,graph_s &a_host_graph,gp_parse_states_s &parse_states,gg_vertex_map_set_s &vertex_map,ui_arrays_array_s &a_ret_vmap_arrays_array,mutex_s &sync_mutex);
      void find_isomorphic_subdigraphs(graph_array_s &a_subgraphs,graph_s &a_host_graph,ui_arrays_array_s &a_ret_vmap_arrays_array);

      void find_edge_mapping(unsigned a_s_idx,graph_s &a_subgraph,graph_s &a_host_graph,ui_arrays_s &a_vertex_maps_array,ui_array_s &a_vertex_map_idxs,ui_arrays_s &a_edge_maps_array,bool a_verify_vertex_maps,ui_array_s &a_tmp_edge_idxs,ui_arrays_array_s &a_graph_edge_mapping_arrays,ui_arrays_s &a_graph_edge_arrays);
      void find_edge_mappings(graph_array_s &a_subgraphs,graph_s &a_host_graph,ui_arrays_array_s &a_vertex_maps_arrays,ui_arrays_s &a_vertex_map_idxs_array,ui_arrays_array_s &a_edge_maps_arrays,bool a_verify_vertex_maps);

      void DEBUG_print_states(FILE *a_file);
      void DEBUG_print_states_latex_table(FILE *a_file,graph_array_s &a_subgraphs);
      void DEBUG_print_state_graph(FILE *a_file);
   }

   graph_parser_s;
@end

/*
 * inline methods of generated structures
 */

// -- graph_array_s --
@begin
   inlines graph_array_s
@end

// -- gp_graph_state_info_s --
@begin
   inlines gp_graph_state_info_s
@end

// -- gp_graph_state_info_array_s --
@begin
   inlines gp_graph_state_info_array_s
@end

// -- gp_state_descriptor_s --
@begin
   inlines gp_state_descriptor_s
@end

// -- gp_state_descriptors_s --
@begin
   inlines gp_state_descriptors_s
@end

// -- gp_state_transition_s --
@begin
   inlines gp_state_transition_s
@end

inline void gp_state_transition_s::clear_evaluations(graph_parser_s &gp)
{/*{{{*/
   gp.ef_clear(edge_evaluation);
   gp.vf_clear(trg_vertex_evaluation);
}/*}}}*/

// -- gp_state_transitions_s --
@begin
   inlines gp_state_transitions_s
@end

// -- gp_state_s --
@begin
   inlines gp_state_s
@end

// -- gp_states_s --
@begin
   inlines gp_states_s
@end

// -- gp_next_move_s --
@begin
   inlines gp_next_move_s
@end

// -- gp_found_graph_info_s --
@begin
   inlines gp_found_graph_info_s
@end

// -- gp_found_graph_infos_s --
@begin
   inlines gp_found_graph_infos_s
@end

// -- gp_parse_state_s --
@begin
   inlines gp_parse_state_s
@end

// -- gp_parse_states_s --
@begin
   inlines gp_parse_states_s
@end

// -- gp_parse_states_array_s --
@begin
   inlines gp_parse_states_array_s
@end

// -- graph_parser_s --
@begin
   inlines graph_parser_s
@end

inline void graph_parser_s::init()
{
   vertices_value_fps = NULL;
   edges_value_fps = NULL;

   create_vertex_map.init();
   found_graph_infos.init();
   edge_arrays.init();
   edge_cnts_array.init();
   states.init();
   state_descriptors.init();
}

inline void graph_parser_s::clear()
{/*{{{*/
   if (states.used != 0) {
      gp_state_s *s_ptr = states.data;
      gp_state_s *s_ptr_end = s_ptr + states.used;
      do {
         gp_state_transitions_s &transitions = s_ptr->transitions;

         if (transitions.used != 0) {
            gp_state_transition_s *t_ptr = transitions.data;
            gp_state_transition_s *t_ptr_end = t_ptr + transitions.used;
            do {
               t_ptr->clear_evaluations(*this);
            } while(++t_ptr < t_ptr_end);
         }
      } while(++s_ptr < s_ptr_end);
   }

   create_vertex_map.clear();
   found_graph_infos.clear();
   edge_arrays.clear();
   edge_cnts_array.clear();
   states.clear();
   state_descriptors.clear();

   init();
}/*}}}*/

inline void graph_parser_s::vf_init(void **a_value_ptr)
{/*{{{*/
   register void *caller = vertices_value_fps[c_graph_function_init];

   if (caller != NULL) {
      ((int(*)(void **))caller)(a_value_ptr);
   }
   else {
      *a_value_ptr = NULL;
   }
}/*}}}*/

inline int graph_parser_s::vf_compare(void *a_first,void *a_second)
{/*{{{*/
   register void *caller = vertices_value_fps[c_graph_function_compare];

   if (caller == NULL) {
      return 0;
   }

   return ((int(*)(void *,void *))caller)(a_first,a_second);
}/*}}}*/

inline void graph_parser_s::vf_operator_equal(void **a_trg,void *a_src)
{/*{{{*/
   register void *caller = vertices_value_fps[c_graph_function_operator_equal];

   if (caller != NULL) {
      ((void(*)(void **,void *))caller)(a_trg,a_src);
   }
   else {
      *a_trg = NULL;
   }
}/*}}}*/

inline void graph_parser_s::vf_get_string(void *a_value,string_s &a_string)
{/*{{{*/
   register void *caller = vertices_value_fps[c_graph_function_get_string];

   if (caller != NULL) {
      ((void(*)(void *,string_s &))caller)(a_value,a_string);
   }
   else {
      a_string.clear();
   }
}/*}}}*/

inline void graph_parser_s::vf_clear(void *a_value)
{/*{{{*/
   register void *caller = vertices_value_fps[c_graph_function_clear];

   if (caller != NULL) {
      ((void(*)(void *))caller)(a_value);
   }
}/*}}}*/

inline void graph_parser_s::ef_init(void **a_value_ptr)
{/*{{{*/
   register void *caller = edges_value_fps[c_graph_function_init];

   if (caller != NULL) {
      ((int(*)(void **))caller)(a_value_ptr);
   }
   else {
      *a_value_ptr = NULL;
   }
}/*}}}*/

inline int graph_parser_s::ef_compare(void *a_first,void *a_second)
{/*{{{*/
   register void *caller = edges_value_fps[c_graph_function_compare];

   if (caller == NULL) {
      return 0;
   }

   return ((int(*)(void *,void *))caller)(a_first,a_second);
}/*}}}*/

inline void graph_parser_s::ef_operator_equal(void **a_trg,void *a_src)
{/*{{{*/
   register void *caller = edges_value_fps[c_graph_function_operator_equal];

   if (caller != NULL) {
      ((void(*)(void **,void *))caller)(a_trg,a_src);
   }
   else {
      *a_trg = NULL;
   }
}/*}}}*/

inline void graph_parser_s::ef_get_string(void *a_value,string_s &a_string)
{/*{{{*/
   register void *caller = edges_value_fps[c_graph_function_get_string];

   if (caller != NULL) {
      ((void(*)(void *,string_s &))caller)(a_value,a_string);
   }
   else {
      a_string.clear();
   }
}/*}}}*/

inline void graph_parser_s::ef_clear(void *a_value)
{/*{{{*/
   register void *caller = edges_value_fps[c_graph_function_clear];

   if (caller != NULL) {
      ((void(*)(void *))caller)(a_value);
   }
}/*}}}*/

#endif

