
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

#ifndef __ISO_PROCESSOR
#define __ISO_PROCESSOR

/*
 * header files
 */

@begin
   include "basic.h"
   include "thread.h"
   include "mem_check.h"
   include "structures.h"
   include "string.h"
   include "graph.h"
   include "graph_parser.h"
   include "dot_loader.h"
@end

/*
 * constant and definitions
 */

// - parse rules info -
const unsigned c_ip_rule_cnt = 42;
const unsigned ip_rule_head_idxs[c_ip_rule_cnt] = {27, 28, 28, 29, 29, 29, 29, 30, 34, 34, 34, 34, 34, 34, 34, 30, 35, 35, 30, 31, 31, 31, 32, 32, 33, 30, 36, 36, 30, 37, 37, 37, 37, 37, 37, 38, 38, 38, 38, 39, 39, 39, };
const unsigned ip_rule_body_lengths[c_ip_rule_cnt] = {1, 2, 1, 3, 2, 2, 1, 2, 1, 2, 2, 1, 1, 2, 2, 2, 2, 2, 3, 1, 1, 3, 3, 1, 1, 2, 1, 1, 2, 2, 2, 1, 1, 2, 2, 1, 2, 1, 2, 1, 1, 1, };

// - parse table info -
#define ip_blank c_idx_not_exist
#define IP_SHIFT(VALUE) VALUE
#define IP_REDUCE(VALUE) c_ip_lalr_table_reduce_base + VALUE
#define IP_GOTO(VALUE) VALUE

const unsigned c_ip_lalr_table_reduce_base = 0x80000000;
const unsigned c_ip_terminal_plus_nonterminal_cnt = 40;
const unsigned ip_lalr_state_cnt = 62;

const unsigned ip_lalr_table[ip_lalr_state_cnt*c_ip_terminal_plus_nonterminal_cnt] = 
{/*{{{*/
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_SHIFT(8),       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_SHIFT(6),    IP_SHIFT(7),    IP_SHIFT(9),   IP_SHIFT(10),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_SHIFT(5),       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_SHIFT(3),       ip_blank,     IP_GOTO(1),     IP_GOTO(2),     IP_GOTO(4),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_SHIFT(8),       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_SHIFT(6),    IP_SHIFT(7),    IP_SHIFT(9),   IP_SHIFT(10),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(13),       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(11),       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(12),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(14),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(6),       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(6),   IP_REDUCE(6),   IP_REDUCE(6),   IP_REDUCE(6),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(6),       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(6),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(18),   IP_SHIFT(17),   IP_SHIFT(19),   IP_SHIFT(20),       ip_blank,       ip_blank,   IP_SHIFT(16),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(15),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(23),   IP_SHIFT(22),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(21),       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(24),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(26),   IP_SHIFT(27),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(25),       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(30),   IP_SHIFT(29),   IP_SHIFT(31),   IP_SHIFT(32),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(28),       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(33),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(4),       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(4),   IP_REDUCE(4),   IP_REDUCE(4),   IP_REDUCE(4),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(4),       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(4),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(5),       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(5),   IP_REDUCE(5),   IP_REDUCE(5),   IP_REDUCE(5),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(5),       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(5),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(7),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(8),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
   IP_SHIFT(36),   IP_SHIFT(37),   IP_SHIFT(38),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(34),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(35),
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(11),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(12),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
   IP_SHIFT(36),   IP_SHIFT(37),   IP_SHIFT(38),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(39),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(40),
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(15),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(41),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(42),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(44),   IP_SHIFT(45),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(46),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(43),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(25),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(26),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(27),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(28),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
   IP_SHIFT(36),   IP_SHIFT(37),   IP_SHIFT(38),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(47),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(48),
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(31),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(49),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(32),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
   IP_SHIFT(36),   IP_SHIFT(37),   IP_SHIFT(38),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(51),       ip_blank,   IP_SHIFT(52),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(50),    IP_GOTO(53),
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(3),       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(3),   IP_REDUCE(3),   IP_REDUCE(3),   IP_REDUCE(3),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(3),       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(3),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_REDUCE(9),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(10),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(39),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(40),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(41),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(13),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(14),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(16),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(17),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(18),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(19),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(20),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(56),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(54),    IP_GOTO(55),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(29),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(30),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(33),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(34),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(35),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
   IP_SHIFT(36),   IP_SHIFT(37),   IP_SHIFT(38),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(57),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(58),
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(37),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(60),   IP_SHIFT(59),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(23),  IP_REDUCE(23),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(24),  IP_REDUCE(24),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(36),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(38),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,   IP_SHIFT(56),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,    IP_GOTO(61),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(21),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,  IP_REDUCE(22),  IP_REDUCE(22),       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,       ip_blank,
};/*}}}*/

/*
 * definition of generated structures
 */

// -- iso_processor_s --
@begin
   struct
   <
   bool:interactive

   unsigned:old_input_idx 
   unsigned:input_idx

   string_s:source_string
   lalr_stack_s:lalr_stack
   ui_array_s:uint_stack

   graph_array_s:searched_graphs
   ui_arrays_s:join_vertex_maps

   graph_parser_s:graph_parser
   graph_s:host_graph

   dot_loader_s:searched_dot_loader
   dot_loader_s:host_dot_loader

   $// - found isomorphism mappings -
   ui_arrays_array_s:li_vertex_maps_arrays
   ui_arrays_array_s:li_edge_maps_arrays
   ui_arrays_s:li_vertex_map_idxs_array
   >

   additions {
      unsigned recognize_terminal(unsigned &input_idx);

      void get_interactive_source_string();
      bool parse_source_string();

      void print_isomorphism_informations(unsigned a_sg_idx);
      void print_isomorphism_dot_informations(unsigned a_sg_idx);

      static void pa_null(iso_processor_s &a_ip);
      static void pa_clean_all(iso_processor_s &a_ip);
      static void pa_clean_searched_all(iso_processor_s &a_ip);
      static void pa_clean_searched_int(iso_processor_s &a_ip);
      static void pa_clean_host(iso_processor_s &a_ip);
      static void pa_clean_automata(iso_processor_s &a_ip);
      static void pa_clean_isomorphisms_all(iso_processor_s &a_ip);
      static void pa_clean_isomorphisms_int(iso_processor_s &a_ip);
      static void pa_load_searched(iso_processor_s &a_ip);
      static void pa_load_host(iso_processor_s &a_ip);
      static void pa_join_vertices_all(iso_processor_s &a_ip);
      static void pa_join_vertices_none(iso_processor_s &a_ip);
      static void pa_join_vertex(iso_processor_s &a_ip);
      static void pa_create_automata(iso_processor_s &a_ip);
      static void pa_create_isomorphisms(iso_processor_s &a_ip);

      static void print_searched_all(iso_processor_s &a_ip);
      static void print_searched_int(iso_processor_s &a_ip);
      static void pa_print_host(iso_processor_s &a_ip);
      static void pa_print_automata(iso_processor_s &a_ip);
      static void pa_print_automata_dot(iso_processor_s &a_ip);
      static void pa_print_isomorphisms_all(iso_processor_s &a_ip);
      static void pa_print_isomorphisms_all_dot(iso_processor_s &a_ip);
      static void pa_print_isomorphisms_int(iso_processor_s &a_ip);
      static void pa_print_isomorphisms_int_dot(iso_processor_s &a_ip);

      static void pa_oct_int_constant(iso_processor_s &a_ip);
      static void pa_dec_int_constant(iso_processor_s &a_ip);
      static void pa_hex_int_constant(iso_processor_s &a_ip);
   }

   iso_processor_s;
@end

extern void(*ip_parse_callbacks[])(iso_processor_s &a_ip);

/*
 * inline methods of generated structures
 */

// -- iso_processor_s --
@begin
   inlines iso_processor_s
@end

#endif

