
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

#define SET_INIT() \
{\
printf(\
"inline void %s::init()\n"\
"{/*{{{*/\n"\
"   size = 0;\n"\
"   used = 0;\n"\
"   data = NULL;\n"\
"   free_idx = c_idx_not_exist;\n"\
"   first_idx = c_idx_not_exist;\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME);\
}

#define SET_INIT_SIZE() \
{\
printf(\
"inline void %s::init_size(unsigned a_size)\n"\
"{/*{{{*/\n"\
"   init();\n"\
"   copy_resize(a_size);\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME);\
}

#define SET_CLEAR() \
{\
   if (!(TYPE_NUMBER & c_type_dynamic)) {\
printf(\
"inline void %s::clear()\n"\
,IM_STRUCT_NAME);\
   }\
   else {\
printf(\
"void %s::clear()\n"\
,IM_STRUCT_NAME);\
   }\
printf(\
"{/*{{{*/\n"\
"   if (data != NULL) {\n"\
);\
   if (TYPE_NUMBER & c_type_dynamic) {\
printf(\
"      %s_element *ptr = data;\n"\
"      %s_element *ptr_end = ptr + size;\n"\
"\n"\
"      do {\n"\
"	 ptr->object.clear();\n"\
"      } while(++ptr < ptr_end);\n"\
"\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME);\
   }\
printf(\
"      cfree(data);\n"\
"   }\n"\
"\n"\
"   init();\n"\
"}/*}}}*/\n"\
"\n"\
);\
}

#define SET_FLUSH() \
{\
printf(\
"inline void %s::flush()\n"\
"{/*{{{*/\n"\
"   copy_resize(used);\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME);\
}

#define SET_FLUSH_ALL() \
{\
   if (!(TYPE_NUMBER & c_type_flushable)) {\
printf(\
"inline void %s::flush_all()\n"\
,IM_STRUCT_NAME);\
   }\
   else {\
printf(\
"void %s::flush_all()\n"\
,IM_STRUCT_NAME);\
   }\
printf(\
"{/*{{{*/\n"\
);\
   if (TYPE_NUMBER & c_type_flushable) {\
printf(\
"   %s_element *ptr = data;\n"\
"   %s_element *ptr_end = ptr + used;\n"\
"\n"\
"   do {\n"\
"      ptr->object.flush_all();\n"\
"   } while(++ptr < ptr_end);\n"\
"\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME);\
   }\
printf(\
"   copy_resize(used);\n"\
"}/*}}}*/\n"\
"\n"\
);\
}

#define SET_SWAP() \
{\
printf(\
"inline void %s::swap(%s &second)\n"\
"{/*{{{*/\n"\
"   unsigned tmp_unsigned = size;\n"\
"   size = second.size;\n"\
"   second.size = tmp_unsigned;\n"\
"\n"\
"   tmp_unsigned = used;\n"\
"   used = second.used;\n"\
"   second.used = tmp_unsigned;\n"\
"\n"\
"   %s_element *tmp_data = data;\n"\
"   data = second.data;\n"\
"   second.data = tmp_data;\n"\
"\n"\
"   tmp_unsigned = free_idx;\n"\
"   free_idx = second.free_idx;\n"\
"   second.free_idx = tmp_unsigned;\n"\
"\n"\
"   tmp_unsigned = first_idx;\n"\
"   first_idx = second.first_idx;\n"\
"   second.first_idx = tmp_unsigned;\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME,IM_STRUCT_NAME);\
}

#define SET_OPERATOR_LE_BR_RE_BR() \
{\
printf(\
"inline %s &%s::operator[](unsigned a_idx)\n"\
"{/*{{{*/\n"\
"   debug_assert(a_idx < used);\n"\
"   return data[a_idx].object;\n"\
"}/*}}}*/\n"\
"\n"\
,TYPE_NAME,IM_STRUCT_NAME);\
}

#define SET_INSERT() \
{\
   if (TYPE_NUMBER & c_type_basic) {\
printf(\
"inline unsigned %s::insert(%s a_value)\n"\
,IM_STRUCT_NAME,TYPE_NAME);\
   }\
   else {\
printf(\
"inline unsigned %s::insert(%s &a_value)\n"\
,IM_STRUCT_NAME,TYPE_NAME);\
   }\
printf(\
"{/*{{{*/\n"\
"   unsigned new_idx;\n"\
"\n"\
"   if (free_idx != c_idx_not_exist) {\n"\
"      new_idx = free_idx;\n"\
"      free_idx = data[new_idx].next_idx;\n"\
"   }\n"\
"   else {\n"\
"      if (used >= size) {\n"\
"	 copy_resize((size << 1) + c_array_add);\n"\
"      }\n"\
"\n"\
"      new_idx = used++;\n"\
"   }\n"\
"\n"\
"   %s_element &new_element = data[new_idx];\n"\
"\n"\
"   new_element.next_idx = first_idx;\n"\
"   new_element.prev_idx = c_idx_not_exist;\n"\
"\n"\
"   if (first_idx != c_idx_not_exist) {\n"\
"      data[first_idx].prev_idx = new_idx;\n"\
"   }\n"\
"\n"\
"   first_idx = new_idx;\n"\
"\n"\
"   new_element.object = a_value;\n"\
"\n"\
"   return new_idx;\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME);\
}

#define SET_INSERT_BLANK() \
{\
printf(\
"inline unsigned %s::insert_blank()\n"\
"{/*{{{*/\n"\
"   unsigned new_idx;\n"\
"\n"\
"   if (free_idx != c_idx_not_exist) {\n"\
"      new_idx = free_idx;\n"\
"      free_idx = data[new_idx].next_idx;\n"\
"   }\n"\
"   else {\n"\
"      if (used >= size) {\n"\
"	 copy_resize((size << 1) + c_array_add);\n"\
"      }\n"\
"\n"\
"      new_idx = used++;\n"\
"   }\n"\
"\n"\
"   %s_element &new_element = data[new_idx];\n"\
"\n"\
"   new_element.next_idx = first_idx;\n"\
"   new_element.prev_idx = c_idx_not_exist;\n"\
"\n"\
"   if (first_idx != c_idx_not_exist) {\n"\
"      data[first_idx].prev_idx = new_idx;\n"\
"   }\n"\
"\n"\
"   first_idx = new_idx;\n"\
"\n"\
"   return new_idx;\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME);\
}

#define SET_REMOVE() \
{\
printf(\
"inline void %s::remove(unsigned a_idx)\n"\
"{/*{{{*/\n"\
"   %s_element &rm_element = data[a_idx];\n"\
"\n"\
"   if (rm_element.next_idx != c_idx_not_exist) {\n"\
"      data[rm_element.next_idx].prev_idx = rm_element.prev_idx;\n"\
"   }\n"\
"\n"\
"   if (rm_element.prev_idx != c_idx_not_exist) {\n"\
"      data[rm_element.prev_idx].next_idx = rm_element.next_idx;\n"\
"   }\n"\
"   else {\n"\
"      first_idx = rm_element.next_idx;\n"\
"   }\n"\
"\n"\
"   rm_element.next_idx = free_idx;\n"\
"   free_idx = a_idx;\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME);\
}

#define SET_NEXT_IDX() \
{\
printf(\
"inline unsigned %s::next_idx(unsigned a_idx)\n"\
"{/*{{{*/\n"\
"   return data[a_idx].next_idx;\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME);\
}

#define SET_PREV_IDX() \
{\
printf(\
"inline unsigned %s::prev_idx(unsigned a_idx)\n"\
"{/*{{{*/\n"\
"   return data[a_idx].prev_idx;\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME);\
}

#define SET_COPY_RESIZE() \
{\
printf(\
"void %s::copy_resize(unsigned a_size)\n"\
"{/*{{{*/\n"\
"   debug_assert(a_size >= used);\n"\
"\n"\
"   %s_element *n_data;\n"\
"\n"\
"   if (a_size == 0) {\n"\
"      n_data = NULL;\n"\
"   }\n"\
"   else {\n"\
"      n_data = (%s_element *)cmalloc(a_size*sizeof(%s_element));\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME,IM_STRUCT_NAME,IM_STRUCT_NAME);\
   if (TYPE_NUMBER & c_type_dynamic) {\
printf(\
"\n"\
"      if (a_size > used) {\n"\
"         %s_element *ptr = n_data + used;\n"\
"         %s_element *ptr_end = n_data + a_size;\n"\
"\n"\
"         do {\n"\
"            ptr->object.init();\n"\
"         } while(++ptr < ptr_end);\n"\
"      }\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME);\
   }\
printf(\
"   }\n"\
"\n"\
"   if (used != 0) {\n"\
"      memcpy(n_data,data,used*sizeof(%s_element));\n"\
"   }\n"\
,IM_STRUCT_NAME);\
   if (TYPE_NUMBER & c_type_dynamic) {\
printf(\
"\n"\
"   if (size > used) {\n"\
"      %s_element *ptr = data + used;\n"\
"      %s_element *ptr_end = data + size;\n"\
"\n"\
"      do {\n"\
"	 ptr->object.clear();\n"\
"      } while(++ptr < ptr_end);\n"\
"   }\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME);\
   }\
printf(\
"\n"\
"   if (size != 0) {\n"\
"      cfree(data);\n"\
"   }\n"\
"\n"\
"   data = n_data;\n"\
"   size = a_size;\n"\
"}/*}}}*/\n"\
"\n"\
);\
}

#define SET_GET_IDX() \
{\
   if (TYPE_NUMBER & c_type_basic) {\
printf(\
"unsigned %s::get_idx(%s a_value)\n"\
,IM_STRUCT_NAME,TYPE_NAME);\
   }\
   else {\
printf(\
"unsigned %s::get_idx(%s &a_value)\n"\
,IM_STRUCT_NAME,TYPE_NAME);\
   }\
printf(\
"{/*{{{*/\n"\
"   if (first_idx == c_idx_not_exist) return c_idx_not_exist;\n"\
"\n"\
"   unsigned idx = first_idx;\n"\
"   do {\n"\
"      %s_element &element = data[idx];\n"\
"\n"\
"      if (element.object == a_value) {\n"\
"	 return idx;\n"\
"      }\n"\
"\n"\
"      idx = element.next_idx;\n"\
"   } while(idx != c_idx_not_exist);\n"\
"\n"\
"   return c_idx_not_exist;\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME);\
}

#define SET_OPERATOR_EQUAL() \
{\
   if (TYPE_NUMBER & c_type_dynamic) {\
printf(\
"%s &%s::operator=(%s &src)\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME,IM_STRUCT_NAME);\
   }\
   else {\
printf(\
"inline %s &%s::operator=(%s &src)\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME,IM_STRUCT_NAME);\
   }\
printf(\
"{/*{{{*/\n"\
"   clear();\n"\
"\n"\
"   if (src.used == 0) return *this;\n"\
"\n"\
"   copy_resize(src.used);\n"\
);\
   if (!(TYPE_NUMBER & c_type_dynamic)) {\
printf(\
"   memcpy(data,src.data,src.used*sizeof(%s_element));\n"\
,IM_STRUCT_NAME);\
   }\
   else {\
printf(\
"\n"\
"   %s_element *ptr = data;\n"\
"   %s_element *s_ptr = src.data;\n"\
"   %s_element *s_ptr_end = s_ptr + src.used;\n"\
"\n"\
"   do {\n"\
"      ptr->object = s_ptr->object;\n"\
"      ptr->next_idx = s_ptr->next_idx;\n"\
"      ptr->prev_idx = s_ptr->prev_idx;\n"\
"   } while(++ptr,++s_ptr < s_ptr_end);\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME,IM_STRUCT_NAME);\
   }\
printf(\
"\n"\
"   used = src.used;\n"\
"   free_idx = src.free_idx;\n"\
"   first_idx = src.first_idx;\n"\
"\n"\
"   return *this;\n"\
"}/*}}}*/\n"\
"\n"\
);\
}

#define SET_OPERATOR_DOUBLE_EQUAL() \
{\
printf(\
"bool %s::operator==(%s &second)\n"\
"{/*{{{*/\n"\
"   if (first_idx == c_idx_not_exist) {\n"\
"      return second.first_idx == c_idx_not_exist;\n"\
"   }\n"\
"\n"\
"   if (second.first_idx == c_idx_not_exist) {\n"\
"      return false;\n"\
"   }\n"\
"\n"\
"   unsigned idx = first_idx;\n"\
"   unsigned s_idx = second.first_idx;\n"\
"\n"\
"   do {\n"\
"      %s_element &element = data[idx];\n"\
"      %s_element &s_element = second.data[s_idx];\n"\
"\n"\
"      if (!(element.object == s_element.object)) {\n"\
"	 return false;\n"\
"      }\n"\
"\n"\
"      idx = element.next_idx;\n"\
"      s_idx = s_element.next_idx;\n"\
"\n"\
"   } while(idx != c_idx_not_exist || s_idx != c_idx_not_exist);\n"\
"\n"\
"   return idx == s_idx;\n"\
"}/*}}}*/\n"\
"\n"\
,IM_STRUCT_NAME,IM_STRUCT_NAME,IM_STRUCT_NAME,IM_STRUCT_NAME);\
}

void processor_s::generate_set_type(string_arrays_s &params,unsigned type_settings)
{
   string_array_s &type_names = params[c_param_types];
   string_array_s &fun_defs = params[c_param_functions];
   string_array_s &abbs = params[c_param_names];

   if (type_names.used != 1) {
      fprintf(stderr,"set: not given name (or too many names) of contained type\n");
      assert(0);
   }

   if (abbs.used < 1) {
      fprintf(stderr,"set: not given name of generated structure\n");
      assert(0);
   }

   unsigned type_abb_idx = abbreviations.get_idx_by_string(type_names[0].size - 1,type_names[0].data);
   if (type_abb_idx == c_idx_not_exist) {
      fprintf(stderr,"set: contained type \"%s\" does not exist\n",type_names[0].data);
      assert(0);
   }

   unsigned type_idx = abbreviations[type_abb_idx].number;
   data_type_s &type = data_types[type_idx];

   // - test type options -
   if (type.number & c_type_setting_strict_dynamic) {
      fprintf(stderr,"set: container have not implemented processing of types with option strict_dynamic\n");
      assert(0);
   }

   string_s real_name;
   real_name.init();
   real_name.conc_set(type.string.size - 1,type.string.data,strlen(c_cont_postfixes[c_cont_set]),(char *)c_cont_postfixes[c_cont_set]);

   string_s &data_type_name = abbs[0];

   unsigned data_type_idx;

   if ((data_type_idx = abbreviations.get_idx_by_string(data_type_name.size - 1,data_type_name.data)) == c_idx_not_exist) {
      //abbreviations.push_blank();
      //abbreviations.last().set(data_type_name,data_types.used);

      unsigned d_idx = data_types.get_idx_by_real_name(real_name.size - 1,real_name.data);
      if (d_idx != c_idx_not_exist) {
         fprintf(stderr,"DUPLICATE STRUCTURES: %s <-> %s\n",data_type_name.data,data_types[d_idx].string.data);
      }

      data_types.push_blank();
      data_type_s &data_type = data_types.last();

      data_type.cont_idx = c_cont_set;
      data_type.string.set(data_type_name.size - 1,data_type_name.data);
      data_type.real_name.swap(real_name);

      data_type.number = c_type_dynamic | c_type_flushable  | (type_settings & c_type_setting_mask);
      data_type.types.push(abbreviations[type_abb_idx].string);

      data_type_idx = data_types.used - 1;
   }
   else {
      //data_type_idx = abbreviations[data_type_idx].number;
      fprintf(stderr,"set: name of generated structure \"%s\" is already used\n",data_type_name.data);
      assert(0);
   }

   real_name.clear();

   // zkontrolovani a zapsani zkratek
   if (abbs.used != 0) {
      unsigned idx = 0;
      do {
	 string_s &abb = abbs[idx];

         if (abbreviations.get_idx_by_string(abb.size - 1,abb.data) != c_idx_not_exist) {
            fprintf(stderr,"set: abreviated name \"%s\" for generated structure is already used\n",abb.data);
            assert(0);
         }

	 abbreviations.push_blank();
	 abbreviations.last().set(abb,data_type_idx);
      } while(++idx < abbs.used);
   }

   data_type_s &data_type = data_types[data_type_idx];

   // - definice struktury set -

printf(
"// struct %s definition\n"
"\n"
,STRUCT_NAME);

   if (abbs.used > 1) {
      unsigned idx = 1;
      do {
printf(
"typedef struct %s %s;\n"
,abbs[0].data,abbs[idx].data);
      } while(++idx < abbs.used);
printf(
"\n"
);
   }

printf(
"/*!\n"
" * \\brief __GEN element of set of type %s\n"
" */\n"
"\n"
"struct %s_element\n"
"{\n"
"   %s object;\n"
"   unsigned next_idx;\n"
"   unsigned prev_idx;\n"
"};\n"
"\n"
"/*!\n"
" * \\brief __GEN set container of type %s\n"
" */\n"
"struct %s\n"
"{\n"
"   unsigned size; //!< actual size of allocated space (element count)\n"
"   unsigned used; //!< count of stored elements\n"
"   %s_element *data; //!< pointer to set elements\n"
"   unsigned free_idx; //!< index of first free element\n"
"   unsigned first_idx; //!< index of first element\n"
"\n"
,TYPE_NAME,STRUCT_NAME,TYPE_NAME,TYPE_NAME,STRUCT_NAME,STRUCT_NAME);
   if (!(data_type.number & c_type_setting_not_generate_init)) {
printf(
"   /*!\n"
"    * \\brief __GEN initialization of set dynamic variables\n"
"    */\n"
"   inline void init();\n"
"\n"
);
   }
printf(
"   /*!\n"
"    * \\brief __GEN initialization of set of desired size\n"
"    * \\param a_size - desired size of set\n"
"    */\n"
"   inline void init_size(unsigned a_size);\n"
"\n"
);
   if (!(TYPE_NUMBER & c_type_dynamic)) {
      if (!(data_type.number & c_type_setting_not_generate_clear)) {
printf(
"   /*!\n"
"    * \\brief __GEN release of memory used by set\n"
"    */\n"
"   inline void clear();\n"
"\n"
);
      }
   }
   else {
      if (!(data_type.number & c_type_setting_not_generate_clear)) {
printf(
"   /*!\n"
"    * \\brief __GEN release of memory used by set\n"
"    */\n"
"   void clear();\n"
"\n"
);
      }
   }
printf(
"   /*!\n"
"    * \\brief __GEN flush set to minimal necessary memory usage\n"
"    */\n"
"   inline void flush();\n"
"\n"
);
   if (!(TYPE_NUMBER & c_type_flushable)) {
printf(
"   /*!\n"
"    * \\brief __GEN flush set and its elements, to minimal necessary memory usage\n"
"    */\n"
"   inline void flush_all();\n"
"\n"
);
   }
   else {
printf(
"   /*!\n"
"    * \\brief __GEN flush set and its elements, to minimal necessary memory usage\n"
"    */\n"
"   void flush_all();\n"
"\n"
);
   }
   if (!(data_type.number & c_type_setting_not_generate_swap)) {
printf(
"   /*!\n"
"    * \\brief __GEN swap members of set to another set\n"
"    * \\param second - reference to another set\n"
"    */\n"
"   inline void swap(%s &second);\n"
"\n"
,STRUCT_NAME);
   }
printf(
"   /*!\n"
"    * \\brief __GEN selection of element set, by index\n"
"    * \\param a_idx - index of element in set\n"
"    * \\return reference to element of set\n"
"    */\n"
"   inline %s &operator[](unsigned a_idx);\n"
"\n"
,TYPE_NAME);
   if (TYPE_NUMBER & c_type_basic) {
printf(
"   /*!\n"
"    * \\brief __GEN insert element to set\n"
"    * \\param a_value - value inserted to set\n"
"    * \\return - position of element in set\n"
"    */\n"
"   inline unsigned insert(%s a_value);\n"
"\n"
,TYPE_NAME);
   }
   else {
printf(
"   /*!\n"
"    * \\brief __GEN insert element to set\n"
"    * \\param a_value - reference to value inserted to set\n"
"    * \\return - position of element in set\n"
"    */\n"
"   inline unsigned insert(%s &a_value);\n"
"\n"
,TYPE_NAME);
   }
printf(
"   /*!\n"
"    * \\brief __GEN insert blank element to set\n"
"    * \\return - position of element in set\n"
"    */\n"
"   inline unsigned insert_blank();\n"
"\n"
"   /*!\n"
"    * \\brief __GEN remove element at given position from set\n"
"    * \\param a_idx - index of removed element\n"
"    */\n"
"   inline void remove(unsigned a_idx);\n"
"\n"
"   /*!\n"
"    * \\brief __GEN return index of next element in set\n"
"    * \\param a_idx - index of element\n"
"    */\n"
"   inline unsigned next_idx(unsigned a_idx);\n"
"\n"
"   /*!\n"
"    * \\brief __GEN return index of previous element in set\n"
"    * \\param a_idx - index of element\n"
"    */\n"
"   inline unsigned prev_idx(unsigned a_idx);\n"
"\n"
"   /*!\n"
"    * \\brief __GEN resize set capacity\n"
"    * \\param a_size - desired set capacity\n"
"    */\n"
"   void copy_resize(unsigned a_size);\n"
"\n"
);
   if (TYPE_NUMBER & c_type_basic) {
printf(
"   /*!\n"
"    * \\brief __GEN search for index of element\n"
"    * \\param a_value - value which index is searched\n"
"    */\n"
"   unsigned get_idx(%s a_value);\n"
,TYPE_NAME);
   }
   else {
printf(
"   /*!\n"
"    * \\brief __GEN search for index of element\n"
"    * \\param a_value - value which index is searched\n"
"    */\n"
"   unsigned get_idx(%s &a_value);\n"
,TYPE_NAME);
   }
   if (!(data_type.number & c_type_setting_not_generate_operator_equal)) {
      if (!(TYPE_NUMBER & c_type_dynamic)) {
printf(
"   /*!\n"
"    * \\brief __GEN copy set from another set\n"
"    * \\param src - reference to another set\n"
"    * \\return reference to this set\n"
"    */\n"
"   inline %s &operator=(%s &src);\n"
"\n"
,STRUCT_NAME,STRUCT_NAME);
      }
      else {
printf(
"   /*!\n"
"    * \\brief __GEN copy set from another set\n"
"    * \\param src - reference to another set\n"
"    * \\return reference to this set\n"
"    */\n"
"   %s &operator=(%s &src);\n"
"\n"
,STRUCT_NAME,STRUCT_NAME);
      }
   }
printf(
"   /*!\n"
"    * \\brief __GEN compare set with another set\n"
"    * \\param second - reference to another set\n"
"    * \\return result of comparision\n"
"    */\n"
"   bool operator==(%s &second);\n"
"\n"
,STRUCT_NAME);
   if (fun_defs.used != 0) {
      unsigned f_idx = 0;
      do {
printf(
"   %s\n"
,fun_defs[f_idx].data);
      } while(++f_idx < fun_defs.used);
   }
printf(
"};\n"
"\n"
);
}

void processor_s::generate_set_inlines(unsigned abb_idx,unsigned a_dt_idx)
{
   data_type_s &data_type = data_types[a_dt_idx];
   
   string_s &type_abb_string = data_type.types[0];
   unsigned type_abb_idx = abbreviations.get_idx_by_string(type_abb_string.size - 1,type_abb_string.data);

   if (type_abb_idx == c_idx_not_exist) {
      fprintf(stderr,"set: inlines: abbreviated type name \"%s\" does not exist\n",type_abb_string.data);
      assert(0);
   }

   unsigned type_idx = abbreviations[type_abb_idx].number;
   data_type_s &type = data_types[type_idx];

   // --- definice inline funkci ---

printf(
"// --- struct %s inline method definition ---\n"
"\n"
,IM_STRUCT_NAME);

   // - set init method -
   if (!(data_type.number & c_type_setting_not_generate_init)) {
SET_INIT();
   }

   // - set init_size method -
SET_INIT_SIZE();

   // - set clear method -
   if (!(TYPE_NUMBER & c_type_dynamic)) {
      if (!(data_type.number & c_type_setting_not_generate_clear)) {
SET_CLEAR();
      }
   }

   // - set flush method -
SET_FLUSH();

   // - set flush_all method -
   if (!(TYPE_NUMBER & c_type_flushable)) {
SET_FLUSH_ALL();
   }

   // - set swap method -
   if (!(data_type.number & c_type_setting_not_generate_swap)) {
SET_SWAP();
   }

   // - set operator[] method -
SET_OPERATOR_LE_BR_RE_BR();

   // - set insert method -
SET_INSERT();

   // - set insert_blank method -
SET_INSERT_BLANK();

   // - set remove method -
SET_REMOVE();

   // - set next_idx method -
SET_NEXT_IDX();

   // - set prev_idx method -
SET_PREV_IDX();

   // - set copy_resize method -

   // - set get_idx method -

   // - set operator= method -
   if (!(TYPE_NUMBER & c_type_dynamic)) {
      if (!(data_type.number & c_type_setting_not_generate_operator_equal)) {
SET_OPERATOR_EQUAL();
      }
   }

   // - set operator== method -

}

void processor_s::generate_set_methods(unsigned abb_idx,unsigned a_dt_idx)
{
   data_type_s &data_type = data_types[a_dt_idx];
   
   string_s &type_abb_string = data_type.types[0];
   unsigned type_abb_idx = abbreviations.get_idx_by_string(type_abb_string.size - 1,type_abb_string.data);

   if (type_abb_idx == c_idx_not_exist) {
      fprintf(stderr,"set: methods: abreviated type name \"%s\" does not exist\n",type_abb_string.data);
      assert(0);
   }

   unsigned type_idx = abbreviations[type_abb_idx].number;
   data_type_s &type = data_types[type_idx];

   // --- definice funkci ---

printf(
"// --- struct %s method definition ---\n"
"\n"
,IM_STRUCT_NAME);

   // - set init method -

   // - set init_size method -

   // - set clear method -
   if (TYPE_NUMBER & c_type_dynamic) {
      if (!(data_type.number & c_type_setting_not_generate_clear)) {
SET_CLEAR();
      }
   }

   // - set flush method -

   // - set flush_all method -
   if (TYPE_NUMBER & c_type_flushable) {
SET_FLUSH_ALL();
   }

   // - set swap method - 

   // - set operator[] method -

   // - set insert method -

   // - set insert_blank method -

   // - set remove method -

   // - set next_idx method -

   // - set prev_idx method -

   // - set copy_resize method -
SET_COPY_RESIZE();

   // - set get_idx method -
SET_GET_IDX();

   // - set operator= method -
   if (TYPE_NUMBER & c_type_dynamic) {
      if (!(data_type.number & c_type_setting_not_generate_operator_equal)) {
SET_OPERATOR_EQUAL();
      }
   }

   // - saet operator== method -
SET_OPERATOR_DOUBLE_EQUAL();

}


