#!/usr/bin/perl

#
#  author Jiří Zuzaňák
#  institution BUT FIT UPGM
#

$code_line = 0;

$t_basic = 1;
$t_no_dynamic = 2;
$t_dynamic = 3;

$t_m_type = 3;
@basic_types = ("bool","char","short","unsigned short","long","unsigned","int","float","double","pointer");
@basic_type_names = ("bb","bc","si","us","li","ui","bi","bf","df","ptr");
@no_dynamic_types = ();
@dynamic_types = ("string_s","buffer_s");

$t_m_flushable = 4;
@flushable_types = ();

@type_abbreviations = ();
@abbreviated_types = ();

sub edit_type {
   $_type = $_[0];

   if ($_type eq "unsigned int") {
      return "unsigned";
   }

   if ($_type eq "long int") {
      return "long";
   }

   if ($_type eq "short int") {
      return "short";
   }

   if ($_type eq "double float") {
      return "double";
   }

   if ($_type eq "unsigned short int" or $_type eq "short unsigned int") {
      return "unsigned short";
   }

   $_idx = 0;
   foreach $_abbreviation (@type_abbreviations) {
      if ($_type eq $_abbreviation) {
	 return $abbreviated_types[$_idx];
      }
      $_idx++;
   }

   return $_type;
}

# --- nalezeni typy typu ---

sub is_basic_type {
   $_type = $_[0];

   foreach $_basic (@basic_types) {
      if ($_type eq $_basic) {
	 return 1;
      }
   }

   return 0;
}

sub is_no_dynamic_type {
   $_type = $_[0];

   foreach $_no_dynamic (@no_dynamic_types) {
      if ($_type eq $_no_dynamic) {
	 return 1;
      }
   }

   return 0;
}

sub is_dynamic_type {
   $_type = $_[0];

   foreach $_dynamic_type (@dynamic_types) {
      if ($_type eq $_dynamic_type) {
	 return 1;
      }
   }

   return 0;
}

sub is_flushable_type {
   $_type = $_[0];

   foreach $_flushable_type (@flushable_types) {
      if ($_type eq $_flushable_type) {
	 return 1;
      }
   }

   return 0;
}

sub get_type_type {
   $_type = $_[0];
   $_ret_type_type = 0;

   if (is_basic_type($_type)) {
      $_ret_type_type = $t_basic;
   }

   if (is_no_dynamic_type($_type)) {
      $_ret_type_type = $t_no_dynamic;
   }

   if (is_dynamic_type($_type)) {
      $_ret_type_type =  $t_dynamic;
   }
   
   if ($_ret_type_type eq 0) {
      die "Err. unknown type $_type.";
   }

   if (is_flushable_type($_type)) {
      $_ret_type_type |= $t_m_flushable;
   }

   return $_ret_type_type;
}

# --- nalezeni jmena typu ---

sub get_type_name {
   $_type = $_[0];

   if ((get_type_type($_type) & $t_m_type) eq $t_basic) {
      $_idx = 0;
      foreach $_basic (@basic_types) {
	 if ($_type eq $_basic) {
	    return $basic_type_names[$_idx];
	 }
	 $_idx++;
      }

      die "Err. Unknown basic type $_type.";
   }
   else {
      return $_type;
   }
}

# --- vytisknuti typu ---
# TODO odstranit mezery pri vypisu?

sub print_type {
$_space = $_[0];
$_type = $_[1];
$_var_name = $_[2];
$_type_type = get_type_type($_type);

if (($_type_type & $t_m_type) eq $t_basic) {
if ($_type eq "bool") {
print
"${_space}if (*$_var_name) {
${_space}   printf(\"true\");
${_space}}
${_space}else {
${_space}   printf(\"false\");
${_space}}
";
}
elsif ($_type eq "char") {
print
"${_space}putchar(*$_var_name);
${_space}putchar(' ');
";
}
elsif ($_type eq "int" or $_type eq "unsigned" or $_type eq "unsigned short") {
print
"${_space}printf(\"%d \",*$_var_name);
";
}
elsif ($_type eq "pointer") {
"${_space}printf(\"#pointer# \");
";
}
else {
die "Err. unknown type $_type print request."
}
}
else {
print
"$_space$_var_name->print();
";
}
}

# *** def function definition ***

$fd_save_file = "save_file";
$fd_load_file = "load_file";

# * def function save_file *

sub fd_definition_save_file {
print
"   inline bool save_file(const char *file_name);
";
}

sub fd_body_save_file {
print
"inline bool $struct_name\:\:save_file(const char *file_name)
{
   if (file_name == NULL) {
      return false;
   }

   unsigned file_size = save_size();

   char buffer[file_size];
   char *b_ptr = buffer;

   save_to(&b_ptr);
   assert((unsigned)(b_ptr - buffer) == file_size);

   FILE *f = fopen(file_name,\"wb\");
   if (f == NULL) {
      return false;
   }

   fwrite(buffer,file_size,1,f);
   fclose(f);

   return true;
}

";
}

# * def function load_file *

sub fd_definition_load_file {
print
"   inline bool load_file(const char *file_name);
";
}

sub fd_body_load_file {
print
"inline bool $struct_name\:\:load_file(const char *file_name)
{
   if (file_name == NULL) {
      return false;
   }

   FILE *f = fopen(file_name,\"rb\");
   if (f == NULL) {
      return false;
   }

   fseek(f,0,SEEK_END);
   unsigned file_size = ftell(f);
   fseek(f,0,SEEK_SET);

   char buffer[file_size];
   fread(buffer,file_size,1,f);
   fclose(f);

   char *b_ptr = buffer;

   clear();
   load_from(&b_ptr);

   assert((unsigned)(b_ptr - buffer) == file_size);

   return true;
}

";
}

# *** struct map definition ***

$map_struct_postfix = "_m";
@map_struct_defined = ();

sub make_map_struct_definition {

@type = (edit_type($m_types[0]),edit_type($m_types[1]));

@type_type = (get_type_type($type[0]),get_type_type($type[1]));
$type_name = get_type_name($type[0]) . "_" . get_type_name($type[1]);

foreach $_defined (@map_struct_defined) {
   if ($_defined eq $type_name) {
      die "Err. already defined map structure of type $type_name.";
   }
}

push @map_struct_defined,$type_name;

$struct_name = $type_name . $map_struct_postfix;

if ((($type_type[0] & $t_m_type) eq $t_dynamic) || (($type_type[1] & $t_m_type) eq $t_dynamic)) {
   push @dynamic_types,$struct_name;
}
else {
   push @no_dynamic_types,$struct_name;
}

if (($type_type[0] & $t_m_flushable) || ($type_type[1] & $t_m_flushable)) {
   push @flushable_types,$struct_name;
}

print
"// --- struct $struct_name definition ---

";
if (@m_names ne 0) {
foreach $_m_name (@m_names) {
push @type_abbreviations,$_m_name;
push @abbreviated_types,$struct_name;
print 
"typedef struct $struct_name $_m_name;
";
}
print 
"
";
}

print
"struct $struct_name
{
   $type[0] idx;
   $type[1] value;

   inline void init();
   inline void clear();
";
print
"   inline void set(";
if (($type_type[0] & $t_m_type) eq $t_basic) {
print
"$type[0] a_idx,";
}
else {
print
"$type[0] &a_idx,";
}
if (($type_type[1] & $t_m_type) eq $t_basic) {
print
"$type[1] a_value";
}
else {
print
"$type[1] &a_value";
}
print
");
";
print
"   inline void flush_all();
   inline void swap($struct_name &second);
   inline $struct_name &operator=($struct_name &src);
   inline bool operator==($struct_name &second);
   inline unsigned save_size();
   inline void save_to(char **b_ptr);
   inline void load_from(char **b_ptr);
   inline void print();
";
if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_definition_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_definition_load_file();
}
}
}
if (@m_functions ne 0) {
foreach $_m_function (@m_functions) {
print
"   $_m_function
";
}
}
print
"};

";

print
"// --- struct $struct_name method definition ---

";

# *** map init method ***

print
"inline void $struct_name\:\:init()
{
";
if (($type_type[0] & $t_m_type) eq $t_dynamic) {
print 
"   idx.init();
";
}
if (($type_type[1] & $t_m_type) eq $t_dynamic) {
print
"   value.init();
";
}
print
"}

";

# *** map clear method ***

print
"inline void $struct_name\:\:clear()
{
";
if (($type_type[0] & $t_m_type) eq $t_dynamic) {
print
"   idx.clear();
";
}
if (($type_type[1] & $t_m_type) eq $t_dynamic) {
print
"   value.clear();
";
}
print
"}

";

# *** map set method ****

print
"inline void $struct_name\:\:set(";
if (($type_type[0] & $t_m_type) eq $t_basic) {
print
"$type[0] a_idx,";
}
else {
print
"$type[0] &a_idx,";
}
if (($type_type[1] & $t_m_type) eq $t_basic) {
print
"$type[1] a_value";
}
else {
print
"$type[1] &a_value";
}
print
")
";
print
"{
   idx = a_idx;
   value = a_value;
}

";

# *** map flush_all method ***

print
"inline void $struct_name\:\:flush_all()
{
";
if ($type_type[0] & $t_m_flushable) {
print
"   idx.flush_all();
";
}
if ($type_type[1] & $t_m_flushable) {
print
"   value.flush_all();
";
}
print
"}

";

# *** map swap method ***

print
"inline void $struct_name\:\:swap($struct_name &second)
{
";
if (($type_type[0] & $t_m_type) eq $t_basic) {
print
"   $type[0] tmp_idx = idx;
   idx = second.idx;
   second.idx = tmp_idx;

";
}
else {
print
"   idx.swap(second.idx);

";
}
if (($type_type[1] & $t_m_type) eq $t_basic) {
print
"   $type[1] tmp_value = value;
   value = second.value;
   second.value = tmp_value;
";
}
else {
print
"   value.swap(second.value);
";
}
print
"}

";

# *** map operator= method ***

print
"inline $struct_name &$struct_name\:\:operator=($struct_name &src)
{
   idx = src.idx;
   value = src.value;

   return *this;
}

";

# *** map operator== method ***

print
"inline bool $struct_name\:\:operator==($struct_name &second)
{
   return (idx == second.idx);
}

";

# *** map save_size method ***

print
"inline unsigned $struct_name\:\:save_size()
{
";
if (($type_type[0] & $t_m_type) ne $t_dynamic) {
print
"   return sizeof($type[0]) + ";
}
else {
print
"   return idx.save_size() + ";
}
if (($type_type[1] & $t_m_type) ne $t_dynamic) {
print
"sizeof($type[1]);
";
}
else {
print
"value.save_size();
";
}
print
"}

";

# *** map save_to method ***

print
"inline void $struct_name\:\:save_to(char **b_ptr)
{
";
if (($type_type[0] & $t_m_type) eq $t_basic) {
print
"   *($type[0] *)*b_ptr = idx; *b_ptr += sizeof($type[0]);
";
}
else {
print
"   idx.save_to(b_ptr);
";
}
if (($type_type[1] & $t_m_type) eq $t_basic) {
print
"   *($type[1] *)*b_ptr = value; *b_ptr += sizeof($type[1]);
";
}
else {
print
"   value.save_to(b_ptr);
";
}
print
"}

";

# *** map load_from method ***

print
"inline void $struct_name\:\:load_from(char **b_ptr)
{
";
if (($type_type[0] & $t_m_type) eq $t_basic) {
print
"   idx = *($type[0] *)*b_ptr; *b_ptr += sizeof($type[0]);
";
}
else {
print
"   idx.load_from(b_ptr);
";
}
if (($type_type[1] & $t_m_type) eq $t_basic) {
print
"   value = *($type[1] *)*b_ptr; *b_ptr += sizeof($type[1]);
";
}
else {
print
"   value.load_from(b_ptr);
";
}
print
"}

";

# *** map print method ***

print
"inline void $struct_name\:\:print()
{
";
print_type("   ",$type[0],"(&idx)");
print
"   printf(\" -> \");
";
print_type("   ",$type[1],"(&value)");
print
"}

";

# *** map def functions ***

if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_body_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_body_load_file();
}
}
}
}

# *** struct set definition ***

$set_struct_postfix = "_s";
@set_struct_defined = ();

sub make_set_struct_definition {

@type = ();
@type_type = ();
$type_name = "";

$t_idx = 0;
do {
   push @type,edit_type($m_types[$t_idx]);
   push @type_type,get_type_type($type[$t_idx]);
   if ($type_name eq "") {
      $type_name = get_type_name($type[$t_idx]);
   }
   else {
      $type_name .= "_" . get_type_name($type[$t_idx]);
   }
} while(++$t_idx < @m_types);

foreach $_defined (@set_struct_defined) {
   if ($_defined eq $type_name) {
      die "Err. already defined set structure of type $type_name.";
   }
}

push @set_struct_defined,$type_name;

$struct_name = $type_name . $set_struct_postfix;

$t_idx = 0;
$_bool_dynamic = 0;
do {
   if ((($type_type[$t_idx]) & $t_m_type) eq $t_dynamic) {
      $_bool_dynamic = 1;
   }
} while(++$t_idx < @type);

if ($_bool_dynamic) {
   push @dynamic_types,$struct_name;
}
else {
   push @no_dynamic_types,$struct_name;
}

$t_idx = 0;
$_bool_flushable = 0;
do {
   if ($type_type[$t_idx] & $t_m_flushable) {
      $_bool_flushable = 1;
   }
} while(++$t_idx < @type);

if ($_bool_flushable) {
   push @flushable_types,$struct_name;
}


print
"// --- struct $struct_name definition ---

";
if (@m_names ne 0) {
foreach $_m_name (@m_names) {
push @type_abbreviations,$_m_name;
push @abbreviated_types,$struct_name;
print
"typedef struct $struct_name $_m_name;
";
}
print
"
";
}

print
"struct $struct_name
{
";
$t_idx = 0;
do {
print
"   $type[$t_idx] $m_variables[$t_idx];
";
} while(++$t_idx < @type);
print
"
   inline void init();
   inline void clear();
   inline void set(";
$t_idx = 0;
do {
if ($t_idx ne 0) {
print ",";
}
if (($type_type[$t_idx] & $t_m_type) eq $t_basic) {
print
"$type[$t_idx] a_$m_variables[$t_idx]";
}
else {
print
"$type[$t_idx] &a_$m_variables[$t_idx]";
}
} while(++$t_idx < @type);
print
");
   inline void flush_all();
   inline void swap($struct_name &second);
   inline $struct_name &operator=($struct_name &src);
   inline bool operator==($struct_name &second);
   inline unsigned save_size();
   inline void save_to(char **b_ptr);
   inline void load_from(char **b_ptr);
   inline void print();
";
if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_definition_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_definition_load_file();
}
}
}
if (@m_functions ne 0) {
foreach $_m_function (@m_functions) {
print
"   $_m_function
";
}
}
print
"};

";

print
"// -- struct $struct_name method definition ---

";

# *** set init method ***

print
"inline void $struct_name\:\:init()
{
";
$t_idx = 0;
do {
if (($type_type[$t_idx] & $t_m_type) eq $t_dynamic) {
print
"   $m_variables[$t_idx].init();
";
}
} while(++$t_idx < @type);
print
"}

";

# *** set clear method ***

print
"inline void $struct_name\:\:clear()
{
";
$t_idx = 0;
do {
if (($type_type[$t_idx] & $t_m_type) eq $t_dynamic) {
print
"   $m_variables[$t_idx].clear();
";
}
} while(++$t_idx < @type);
print
"}

";

# *** set set method ***

print
"inline void $struct_name\:\:set(";
$t_idx = 0;
do {
if ($t_idx ne 0) {
print ",";
}
if (($type_type[$t_idx] & $t_m_type) eq $t_basic) {
print
"$type[$t_idx] a_$m_variables[$t_idx]";
}
else {
print
"$type[$t_idx] &a_$m_variables[$t_idx]";
}
} while(++$t_idx < @type);
print
")
{
";
$t_idx = 0;
do {
print
"   $m_variables[$t_idx] = a_$m_variables[$t_idx];
";
} while(++$t_idx < @type);
print
"}

";

# *** set flush_all method ***

print
"inline void $struct_name\:\:flush_all()
{
";
$t_idx = 0;
do {
if ($type_type[$t_idx] & $t_m_flushable) {
print 
"   $m_variables[$t_idx].flush_all();
";
}
} while(++$t_idx < @type);
print
"}

";

# *** set swap method ***

print
"inline void $struct_name\:\:swap($struct_name &second)
{
";
$t_idx = 0;
do {
if (($type_type[$t_idx] & $t_m_type) eq $t_basic) {
print
"   $type[$t_idx] tmp_$m_variables[$t_idx] = $m_variables[$t_idx];
   $m_variables[$t_idx] = second.$m_variables[$t_idx];
   second.$m_variables[$t_idx] = tmp_$m_variables[$t_idx];
";
}
else {
print
"   $m_variables[$t_idx].swap(second.$m_variables[$t_idx]);
";
}
if ($t_idx < @type - 1) {
print
"
";
}
} while(++$t_idx < @type);
print
"}

";

# *** set operator= method ***

print
"inline $struct_name &$struct_name\:\:operator=($struct_name &src)
{
";
$t_idx = 0;
do {
print
"   $m_variables[$t_idx] = src.$m_variables[$t_idx];
";
} while(++$t_idx < @type);
print
"
   return *this;
}

";

# *** set operator== method ***

print
"inline bool $struct_name\:\:operator==($struct_name &second)
{
   return (";
$t_idx = 0;
do {
if ($t_idx ne 0) {
print 
" && ";
}
print 
"$m_variables[$t_idx] == second.$m_variables[$t_idx]";
} while(++$t_idx < @type);
print
");
}

";

# *** set save_size method ***

print
"inline unsigned $struct_name\:\:save_size()
{
   return (";
$t_idx = 0;
do {
if ($t_idx ne 0) {
print
" + ";
}
if (($type_type[$t_idx] & $t_m_type) eq $t_basic) {
print 
"sizeof($type[$t_idx])";
}
else {
print
"$m_variables[$t_idx].save_size()";
}
} while(++$t_idx < @type);
print
");
}

";

# *** set save_to method ***

print
"inline void $struct_name\:\:save_to(char **b_ptr)
{
";
$t_idx = 0;
do {
if (($type_type[$t_idx] & $t_m_type) eq  $t_basic) {
print
"   *($type[$t_idx] *)*b_ptr = $m_variables[$t_idx]; *b_ptr += sizeof($type[$t_idx]);
";
}
else {
print
"   $m_variables[$t_idx].save_to(b_ptr);
";
}
} while(++$t_idx < @type);
print
"}

";

# *** set load_from method ***

print
"inline void $struct_name\:\:load_from(char **b_ptr)
{
";
$t_idx = 0;
do {
if (($type_type[$t_idx] & $t_m_type) eq $t_basic) {
print
"   $m_variables[$t_idx] = *($type[$t_idx] *)*b_ptr; *b_ptr += sizeof($type[$t_idx]);
";
}
else {
print
"   $m_variables[$t_idx].load_from(b_ptr);
";
}
} while(++$t_idx < @type);
print
"}

";

# *** set print method ***

print
"inline void $struct_name\:\:print()
{
";
$t_idx = 0;
do {
print_type("   ",$type[$t_idx],"(&$m_variables[$t_idx])");
} while(++$t_idx < @type);
print
"}

";

# *** set def functions ***

if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_body_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_body_load_file();
}
}
}
}

# *** container double_array definition ***

$double_array_struct_postfix = "_da";
@double_array_struct_defined = ();

sub make_double_array_struct_definition {

$type = edit_type($m_types[0]);

$type_type = get_type_type($type);
$type_name = get_type_name($type);

foreach $_defined (@double_array_struct_defined) {
   if ($_defined eq $type_name) {
      die "Err. already defined double_array structure of type $type_name.";
   }
}

push @double_array_struct_defined,$type_name;

$struct_name = $type_name . $double_array_struct_postfix;
push @dynamic_types,$struct_name;

if ($type_type & $t_m_flushable) {
   push @flushable_types,$struct_name;
}

print
"// --- struct $struct_name definition ---

";
if (@m_names ne 0) {
foreach $_m_name (@m_names) {
push @type_abbreviations,$_m_name;
push @abbreviated_types,$struct_name;
print 
"typedef struct $struct_name $_m_name;
";
}
print 
"
";
}

print
"struct $struct_name
{
   unsigned x_size;
   unsigned y_size;
   $type *data;

   inline void init();
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   inline void clear();
";
}
else {
print
"   void clear();
";
}
if (!($type_type & $t_m_flushable)) {
print
"   inline void flush_all();
";
}
else {
print
"   void flush_all();
";
}
print
"   inline void swap($struct_name &second);
   inline $type &value(unsigned a_x_idx,unsigned a_y_idx);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   inline void resize(unsigned a_x_size,unsigned a_y_size);
";
}
else {
print
"   void resize(unsigned a_x_size,unsigned a_y_size);
";
}
if ($type eq "char") {
print
"   inline void fill($type a_value);
";
}
else {
if (($type_type & $t_m_type) eq $t_basic) {
print
"   void fill($type a_value);
";
}
else {
print
"   void fill($type &a_value);
";
}
}
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   inline $struct_name &operator=($struct_name &src);
   inline bool operator==($struct_name &second);
   inline unsigned save_size();
   inline void save_to(char **b_ptr);
   inline void load_from(char **b_ptr);
";
}
else {
print
"   $struct_name &operator=($struct_name &src);
   bool operator==($struct_name &second);
   unsigned save_size();
   void save_to(char **b_ptr);
   void load_from(char **b_ptr);
";
}
print
"   void print();
";
if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_definition_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_definition_load_file();
}
}
}
if (@m_functions ne 0) {
foreach $_m_function (@m_functions) {
print
"   $_m_function
";
}
}
print
"};

";

print
"// --- struct $struct_name method definition ---

";

# *** double_array init method ***

print
"inline void $struct_name\:\:init()
{
   x_size = 0;
   y_size = 0;
   data = NULL;
}

";

# *** double_array clear method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline void $struct_name\:\:clear()
";
}
else {
print
"void $struct_name\:\:clear()
";
}
print
"{
   if (data != NULL) {
";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"      $type *ptr = data;
      $type *ptr_end = ptr + x_size*y_size;

      do {
	 ptr->clear();
      } while(++ptr < ptr_end);

";
}
print
"      cfree(data);
   }

   init();
}

";

# *** double_array flush_all method ***

if (!($type_type & $t_m_flushable)) {
print
"inline void $struct_name\:\:flush_all()
";
}
else {
print
"void $struct_name\:\:flush_all()
";
}
print
"{
";
if ($type_type & $t_m_flushable) {
print
"   if (x_size == 0 || y_size == 0) {
      return;
   }
   
   $type *ptr = data;
   $type *ptr_end = ptr + x_size*y_size;

   do {
      ptr->flush_all();
   } while(++ptr < ptr_end);
";
}
print
"}

";

# *** double_array swap method ***

print
"inline void $struct_name\:\:swap($struct_name &second)
{
   unsigned tmp_x_size = x_size;
   x_size = second.x_size;
   second.x_size = tmp_x_size;

   unsigned tmp_y_size = y_size;
   y_size = second.y_size;
   second.y_size = tmp_y_size;

   $type *tmp_data = data;
   data = second.data;
   second.data = tmp_data;
}

";

# *** double_array value method ***

print
"inline $type &$struct_name\:\:value(unsigned a_x_idx,unsigned a_y_idx)
{
   debug_assert(a_x_idx < x_size && a_y_idx < y_size);
   return data[a_y_idx*x_size + a_x_idx];
}

";

# *** double_array resize method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline void $struct_name\:\:resize(unsigned a_x_size,unsigned a_y_size)
";
}
else {
print
"void $struct_name\:\:resize(unsigned a_x_size,unsigned a_y_size)
";
}
print
"{
   if (data != NULL) {
";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"      $type *ptr = data;
      $type *ptr_end = ptr + x_size*y_size;

      do {
	 ptr->clear();
      } while(++ptr < ptr_end);

";
}
print
"      cfree(data);
   }

   if (a_x_size == 0 || a_y_size == 0) {
      data = NULL;

      x_size = 0;
      y_size = 0;
   }
   else {
      data = ($type *)cmalloc(a_x_size*a_y_size*sizeof($type));

";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"      $type *ptr = data;
      $type *ptr_end = ptr + a_y_size*a_x_size;

      do {
	 ptr->init();
      } while(++ptr < ptr_end);

";
}
print
"      x_size = a_x_size;
      y_size = a_y_size;
   }
}

";

# *** double_array fill method ***

if ($type eq "char") {
print
"inline $struct_name\:\:void fill($type a_value)
";
}
else {
if (($type_type & $t_m_type) eq $t_basic) {
print
"void $struct_name\:\:fill($type a_value)
";
}
else {
print
"void $struct_name\:\:fill($type &a_value)
";
}
}
print
"{
   if (x_size == 0 || y_size == 0) {
      return;
   }

";
if ($type eq "char") {
print
"   memset(data,a_value,x_size*y_size*sizeof($type));
";
}
else {
print
"   $type *ptr = data;
   $type *ptr_end = ptr + x_size*y_size;

   do {
      *ptr = a_value;
   } while(++ptr < ptr_end);
";
}
print
"}

";

# *** double_array operator= method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline $struct_name &$struct_name\:\:operator=($struct_name &src)
";
}
else {
print
"$struct_name &$struct_name\:\:operator=($struct_name &src)
";
}
print
"{
   resize(src.x_size,src.y_size);

   if (src.x_size == 0 || src.y_size == 0) {
      return *this;
   }

";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   memcpy(data,src.data,x_size*y_size*sizeof($type));
";
}
else {
print
"   $type *ptr = data;
   $type *ptr_end = ptr + x_size*y_size;
   $type *s_ptr = src.data;

   do {
      *ptr = *s_ptr;
   } while(++s_ptr,++ptr < ptr_end);

";
}
print
"   return *this;
}

";

# *** double_array operator== method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline bool $struct_name\:\:operator==($struct_name &second)
";
}
else {
print
"bool $struct_name\:\:operator==($struct_name &second)
";
}
print
"{
   if (x_size != second.x_size || y_size != second.y_size) {
      return false;
   }
   if (x_size == 0 || y_size == 0) {
      return true;
   }

";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   return (memcmp(data,second.data,x_size*y_size*sizeof($type)) == 0);
";
}
else {
print
"   $type *ptr = data;
   $type *ptr_end = ptr + x_size*y_size;
   $type *s_ptr = second.data;

   do {
      if (!(*ptr == *s_ptr)) {
	 return false;
      }
   } while(++s_ptr,++ptr < ptr_end);

   return true;
";
}
print
"}

";

# *** double_array save_size method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline unsigned $struct_name\:\:save_size()
";
}
else {
print
"unsigned $struct_name\:\:save_size()
";
}
print
"{
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   return (sizeof(unsigned) + sizeof(unsigned) + x_size*y_size*sizeof($type));
";
}
else {
print
"   if (x_size == 0 || y_size == 0) return (sizeof(unsigned) + sizeof(unsigned));

   unsigned ret_size = sizeof(unsigned) + sizeof(unsigned);

   $type *ptr = data;
   $type *ptr_end = ptr + x_size*y_size;

   do {
      ret_size += ptr->save_size();
   } while(++ptr < ptr_end);

   return ret_size;
";
}
print
"}

";

# *** double_array save_to method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline void $struct_name\:\:save_to(char **b_ptr)
";
}
else {
print
"void $struct_name\:\:save_to(char **b_ptr)
";
}
print
"{
   *(unsigned *)*b_ptr = x_size; *b_ptr += sizeof(unsigned);
   *(unsigned *)*b_ptr = y_size; *b_ptr += sizeof(unsigned);

   if (x_size == 0 || y_size == 0) return;

";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   unsigned data_size = x_size*y_size*sizeof($type);
   memcpy(*b_ptr,data,data_size);
   *b_ptr += data_size;
";
}
else {
print
"   $type *ptr = data;
   $type *ptr_end = ptr + x_size*y_size;

   do {
      ptr->save_to(b_ptr);
   } while(++ptr < ptr_end);
";
}
print
"}

";

# *** double_array load_from method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline void $struct_name\:\:load_from(char **b_ptr)
";
}
else {
print
"void $struct_name\:\:load_from(char **b_ptr)
";
}
print
"{
   clear();

   unsigned tmp_x_size = *(unsigned *)*b_ptr; *b_ptr += sizeof(unsigned);
   unsigned tmp_y_size = *(unsigned *)*b_ptr; *b_ptr += sizeof(unsigned);

   if (tmp_x_size == 0 || tmp_y_size == 0) return;

   resize(tmp_x_size,tmp_y_size);

";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   unsigned data_size = tmp_x_size*tmp_y_size*sizeof($type);
   memcpy(data,*b_ptr,data_size);

   *b_ptr += data_size;
";
}
else {
print
"   $type *ptr = data;
   $type *ptr_end = ptr + tmp_x_size*tmp_y_size;

   do {
      ptr->load_from(b_ptr);
   } while(++ptr < ptr_end);
";
}
print
"}

";

# *** double_array print methos ***

print
"void $struct_name\:\:print()
{
   if (x_size == 0 || y_size == 0) {
      printf(\"empty\");
      return;
   }

   $type *ptr = data;
   $type *ptr_end = ptr + x_size*y_size;

   do {
      $type *ptr_w_end = ptr + x_size;
      do {
";
print_type("         ",$type,ptr);
print
"      } while(++ptr < ptr_w_end);
      
      putchar(\'\\n\');
   } while(ptr < ptr_end);
}

";

# *** array def functions ***

if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_body_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_body_load_file();
}
}
}
}

# *** container hash_array definition ***

$hash_array_struct_postfix = "_ha";
@hash_array_struct_defined = ();

sub make_hash_array_struct_definition {

$type = edit_type($m_types[0]);

$type_type = get_type_type($type);
$type_name = get_type_name($type);

foreach $_define (@hash_array_struct_defined) {
   if ($_defined eq $type_name) {
      die "Err. already defined hash_array structure of type $type_name.";
   }
}

push @hash_array_struct_defined,$type_name;

$struct_name = $type_name . $hash_array_struct_postfix;
push @dynamic_types,$struct_name;
push @flushable_types,$struct_name;

print 
"// --- struct $struct_name definition ---

";
if (@m_names ne 0) {
foreach $_m_name (@m_names) {
push @type_abbreviations,$_m_name;
push @abbreviated_types,$struct_name;
print 
"typedef struct $struct_name $_m_name;
";
}
print 
"
";
}

$data_element = ${struct_name}. "_data_element";
print
"struct $data_element
{
   $type data;
   unsigned next_idx;
   unsigned prev_idx;
};

";

print
"struct $struct_name
{
   unsigned size;
   unsigned used;
   unsigned data_start;
   unsigned empty_start;
   $data_element *data;

   inline void init();
   inline void init_size(unsigned a_size);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print 
"   inline void clear();
";
}
else {
print
"   void clear();
";   
}
print 
"   void flush();
";
if (!($type_type & $t_m_flushable)) {
print
"   inline void flush_all();
";
}
else {
print
"   void flush_all();
";
}
print
"   inline void swap($struct_name &second);
   inline $type &operator[](unsigned a_idx);
";
if (($type_type & $t_m_type) eq $t_basic) {
print
"   inline unsigned insert($type a_value);
";
}
else {
print
"   inline unsigned insert($type &a_value);
";
}
print
"   inline unsigned insert_blank();
   inline unsigned insert_clear();
   inline $type &remove(unsigned a_idx);
   void copy_resize(unsigned a_size);
";
if (($type_type & $t_m_type) eq $t_basic) {
print
"   void fill($type a_value);
   unsigned get_idx($type a_value);
";
}
else {
print
"   void fill($type &a_value);
   unsigned get_idx($type &a_value);
";
}
print
"   $struct_name &operator=($struct_name &src);
   bool operator==($struct_name &second);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   inline unsigned save_size();
";
}
else {
print
"   unsigned save_size();
";
}
print
"   void save_to(char **b_ptr);
   void load_from(char **b_ptr);
";
print
"   void print();
";
if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_definition_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_definition_load_file();
}
}
}
if (@m_functions ne 0) {
foreach $_m_function (@m_functions) {
print
"   $_m_function
";
}
}
print
"};

";

print
"// --- struct $struct_name method definition ---

";

# *** hash_array init method ***

print
"inline void $struct_name\:\:init()
{
   size = 0;
   used = 0;
   data_start = c_idx_not_exist;
   empty_start = c_idx_not_exist;
   data = NULL;
}

";

# *** hash_array init_size method ***

print
"inline void $struct_name\:\:init_size(unsigned a_size)
{
   init();
   copy_resize(a_size);
}

";

# *** hash_array clear method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print 
"inline void $struct_name\:\:clear()
";
}
else {
print
"void $struct_name\:\:clear()
";
}
print
"{
   if (data != NULL) {
";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"      $data_element *ptr = data;
      $data_element *ptr_end = ptr + size;

      do {
	 ptr->data.clear();
      } while(++ptr < ptr_end);

";
}
print
"      cfree(data);
   }

   init();
}

";

# *** hash_array flush method ***

print
"void $struct_name\:\:flush()
{
   if (data_start == c_idx_not_exist) {
      clear();
      return;
   }

   if (used == size) {
      return;
   }

   $data_element *n_data = ($data_element *)cmalloc(used*sizeof($data_element));

   unsigned idx = data_start;
   unsigned n_idx = 0;
   do {
      $data_element &element = data[idx];
      $data_element &n_element = n_data[n_idx];
      
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"      n_element.data = element.data;
";
}
else {
print
"      n_element.data.swap(element.data);
";
}
print
"      n_element.prev_idx = n_idx - 1;
      n_element.next_idx = n_idx + 1;

      idx = element.next_idx;
   } while(++n_idx,idx != c_idx_not_exist);

   n_data[0].prev_idx = c_idx_not_exist;
   n_data[used - 1].next_idx = c_idx_not_exist;

";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"   idx = empty_start;
   do {
      $data_element &element = data[idx];
      
      element.data.clear();
      idx = element.next_idx;
   } while(idx != c_idx_not_exist);
";
}
print
"   cfree(data);

   data = n_data;
   size = used;
   data_start = 0;
   empty_start = c_idx_not_exist;
}

";

# *** hash_array flush_all function ***

if (!($type_type & $t_m_flushable)) {
print
"inline void $struct_name\:\:flush_all()
";
}
else {
print
"void $struct_name\:\:flush_all()
";
}
print
"{
";
if ($type_type & $t_m_flushable) {
print
"   if (data_start != c_idx_not_exist) {

      unsigned idx = data_start;
      do {
	 $data_element &element = data[idx];

	 element.data.flush_all();
	 idx = element.next_idx;
      } while(idx != c_idx_not_exist);
   }

";
}
print
"   flush();
}

";

# *** hash_array swap function ***

print
"inline void $struct_name\:\:swap($struct_name &second)
{
   unsigned tmp_unsigned = size;
   size = second.size;
   second.size = tmp_unsigned;

   tmp_unsigned = used;
   used = second.used;
   second.used = tmp_unsigned;

   tmp_unsigned = data_start;
   data_start = second.data_start;
   second.data_start = tmp_unsigned;

   tmp_unsigned = empty_start;
   empty_start = second.empty_start;
   second.empty_start = tmp_unsigned;

   $data_element *tmp_data = data;
   data = second.data;
   second.data = tmp_data;
}

";

# *** hash_array operator[] function ***

print
"inline $type &$struct_name\:\:operator[](unsigned a_idx)
{
   assert(a_idx < size);
   return data[a_idx].data;
}

";

# *** hash_array insert function ***

if (($type_type & $t_m_type) eq $t_basic) {
print
"inline unsigned $struct_name\:\:insert($type a_value)
";
}
else {
print
"inline unsigned $struct_name\:\:insert($type &a_value)
";
}
print
"{
   if (empty_start == c_idx_not_exist) {
      copy_resize(size + c_array_add);
   }

   unsigned idx = empty_start;
   $data_element &element = data[idx];

   empty_start = element.next_idx;

   if (element.next_idx != c_idx_not_exist) {
      data[element.next_idx].prev_idx = c_idx_not_exist;
   }

   element.prev_idx = c_idx_not_exist;
   element.next_idx = data_start;

   if (data_start != c_idx_not_exist) {
      data[data_start].prev_idx = idx;
   }

   data_start = idx;

   element.data = a_value;

   used++;
   return idx;
}

";

# *** hash_array insert_blank function ***

print
"inline unsigned $struct_name\:\:insert_blank()
{
   if (empty_start == c_idx_not_exist) {
      copy_resize(size + c_array_add);
   }

   unsigned idx = empty_start;
   $data_element &element = data[idx];

   empty_start = element.next_idx;

   if (element.next_idx != c_idx_not_exist) {
      data[element.next_idx].prev_idx = c_idx_not_exist;
   }

   element.prev_idx = c_idx_not_exist;
   element.next_idx = data_start;

   if (data_start != c_idx_not_exist) {
      data[data_start].prev_idx = idx;
   }

   data_start = idx;

   used++;
   return idx;
}

";

# *** hash_array insert_clear function ***

print
"inline unsigned $struct_name\:\:insert_clear()
{
   if (empty_start == c_idx_not_exist) {
      copy_resize(size + c_array_add);
   }

   unsigned idx = empty_start;
   $data_element &element = data[idx];

   empty_start = element.next_idx;

   if (element.next_idx != c_idx_not_exist) {
      data[element.next_idx].prev_idx = c_idx_not_exist;
   }

   element.prev_idx = c_idx_not_exist;
   element.next_idx = data_start;

   if (data_start != c_idx_not_exist) {
      data[data_start].prev_idx = idx;
   }

   data_start = idx;

";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"   element.data.clear();

";
}
print
"   used++;
   return idx;
}

";

# *** hash_array remove function ***

print
"inline $type &$struct_name\:\:remove(unsigned a_idx)
{
   assert(a_idx < size);

   $data_element &element = data[a_idx];

   if (element.next_idx != c_idx_not_exist) {
      data[element.next_idx].prev_idx = element.prev_idx;
   }

   if (element.prev_idx != c_idx_not_exist) {
      data[element.prev_idx].next_idx = element.next_idx;
   }

   element.prev_idx = c_idx_not_exist;
   element.next_idx = empty_start;

   if (empty_start != c_idx_not_exist) {
      data[empty_start].prev_idx = a_idx;
   }

   empty_start = a_idx;

   used--;
   return element.data;
}

";

# *** hash_array copy_resize function ***

print
"void $struct_name\:\:copy_resize(unsigned a_size)
{
   assert(a_size >= size);

   if (a_size == size) return;

   $data_element *n_data = ($data_element *)cmalloc(a_size*sizeof($data_element));

   if (size != 0) {
      memcpy(n_data,data,size*sizeof($data_element));
      cfree(data);
   }

   data = n_data;

   unsigned idx = size;
   do {
      $data_element &element = data[idx];

";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"      element.data.init();
";
}
print
"      element.prev_idx = idx - 1;
      element.next_idx = idx + 1;
   } while(++idx < a_size);

   data[size].prev_idx = c_idx_not_exist;
   data[a_size - 1].next_idx = empty_start;

   if (empty_start != c_idx_not_exist) {
      data[empty_start].prev_idx = a_size - 1;
   }

   empty_start = size;

   size = a_size;
}

";

# *** hash_array fill function ***

if (($type_type & $t_m_type) eq $t_basic) {
print
"void $struct_name\:\:fill($type a_value)
";
}
else {
print
"void $struct_name\:\:fill($type &a_value)
";
}
print
"{
   if (size == 0) {
      return;
   }

   if (data_start != c_idx_not_exist) {

      unsigned idx = data_start;
      do {
	 $data_element &element = data[idx];

	 element.data = a_value;

	 if (element.next_idx == c_idx_not_exist) {
	    break;
	 }
	 
	 idx = element.next_idx;
      } while(1);
      
      data[idx].next_idx = empty_start;

      if (empty_start != c_idx_not_exist) {
	 data[empty_start].prev_idx = idx;
      }
   }
   else {
      data_start = empty_start;
   }

   if (empty_start != c_idx_not_exist) {

      unsigned idx = empty_start;
      do {
	 $data_element &element = data[idx];

	 element.data = a_value;
	 idx = element.next_idx;
      } while(idx != c_idx_not_exist);
   }

   empty_start = c_idx_not_exist;
   used = size;
}

";

# *** hash_array get_idx function ***

if (($type_type & $t_m_type) eq $t_basic) {
print
"unsigned $struct_name\:\:get_idx($type a_value)
";
}
else {
print
"unsigned $struct_name\:\:get_idx($type &a_value)
";
}
print
"{
   if (data_start == c_idx_not_exist) {
      return c_idx_not_exist;
   }

   unsigned idx = data_start;
   do {
      $data_element &element = data[idx];

      if (element.data == a_value) {
	 return idx;
      }

      idx = element.next_idx;
   } while(idx != c_idx_not_exist);

   return c_idx_not_exist;
}

";

# *** hash_array operator= function ***

print
"$struct_name &$struct_name\:\:operator=($struct_name &src)
{
   clear();

   if (src.data_start == c_idx_not_exist) {
      return *this;
   }

   copy_resize(src.used);

   $data_element *ptr = data;
   unsigned s_idx = src.data_start;
   do {
      $data_element &s_element = src.data[s_idx];
      
      ptr->data = s_element.data;
      s_idx = s_element.next_idx;
   } while(++ptr,s_idx != c_idx_not_exist);

   used = size;
   data_start = 0;
   empty_start = c_idx_not_exist;

   return *this;
}

";

# *** hash_array operator== function ***

print
"bool $struct_name\:\:operator==($struct_name &second)
{
   if (used != second.used) return false;
   if (data_start == c_idx_not_exist) return true;

   unsigned idx = data_start;
   unsigned s_idx = second.data_start;

   do {
      $data_element &element = data[idx];
      $data_element &s_element = second.data[s_idx];

      if (!(element.data == s_element.data)) {
	 return false;
      }

      idx = element.next_idx;
      s_idx = s_element.next_idx;
   } while(idx != c_idx_not_exist);

   return true;
}

";

# *** hash_array save_size function ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline unsigned $struct_name\:\:save_size()
";
}
else {
print
"unsigned $struct_name\:\:save_size()
";
}
print
"{
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   return sizeof(unsigned) + used*sizeof($type);
";
}
else {
print
"   if (used == 0) {
      return sizeof(unsigned);
   }
   
   unsigned ret_size = sizeof(unsigned);

   unsigned idx = data_start;
   do {
      $data_element &element = data[idx];

      ret_size += element.data.save_size();
      idx = element.next_idx;
   } while(idx != c_idx_not_exist);

   return ret_size;
";
}
print
"}

";

# *** hash_array save_to function ***

print
"void $struct_name\:\:save_to(char **b_ptr)
{
   *(unsigned *)*b_ptr = used; *b_ptr += sizeof(unsigned);

   if (data_start == c_idx_not_exist) return;

   unsigned idx = data_start;
   do {
      $data_element &element = data[idx];

";
if (($type_type & $t_m_type) eq $t_basic) {
print
"      *($type *)*b_ptr = element.data; *b_ptr += sizeof($type);
";
}
else {
print
"      element.data.save_to(b_ptr);
";
}
print
"      idx = element.next_idx;
   } while(idx != c_idx_not_exist);
}

";

# *** hash_array load_from function ***

print
"void $struct_name\:\:load_from(char **b_ptr)
{
   clear();

   unsigned tmp_used = *(unsigned *)*b_ptr; *b_ptr += sizeof(unsigned);
   if (tmp_used == 0) return;

   copy_resize(tmp_used);

   $data_element *ptr = data;
   $data_element *ptr_end = ptr + tmp_used;

   do {
";
if (($type_type & $t_m_type) eq $t_basic) {
print
"      ptr->data = *($type *)*b_ptr; *b_ptr += sizeof($type);
";
}
else {
print
"      ptr->data.load_from(b_ptr);
";
}
print
"   } while(++ptr < ptr_end);

   used = size;
   data_start = 0;
   empty_start = c_idx_not_exist;
}

";

# *** hash_array print function ***

print
"void $struct_name\:\:print()
{
   if (data_start == c_idx_not_exist) {
      printf(\"empty\");
      return;
   }

   unsigned idx = data_start;
   do {
      $data_element &element = data[idx];

";
print_type("      ",$type,"(&element.data)");
print
"      idx = element.next_idx;
   } while(idx != c_idx_not_exist);
}

";

# *** hash_array def functions ***

if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_body_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_body_load_file();
}
}
}
}

# *** container array definition ***

$array_struct_postfix = "_a";
@array_struct_defined = ();

sub make_array_struct_definition {

$type = edit_type($m_types[0]);

$type_type = get_type_type($type);
$type_name = get_type_name($type);

foreach $_defined (@array_struct_defined) {
   if ($_defined eq $type_name) {
      die "Err. already defined array structure of type $type_name.";
   }
}

push @array_struct_defined,$type_name;

$struct_name = $type_name . $array_struct_postfix;
push @dynamic_types,$struct_name;
push @flushable_types,$struct_name;

print 
"// --- struct $struct_name definition ---

";
if (@m_names ne 0) {
foreach $_m_name (@m_names) {
push @type_abbreviations,$_m_name;
push @abbreviated_types,$struct_name;
print 
"typedef struct $struct_name $_m_name;
";
}
print 
"
";
}

print
"struct $struct_name
{
   unsigned size;
   unsigned used;
   $type *data;

   inline void init();
   inline void init_size(unsigned a_size);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print 
"   inline void clear();
   inline void set(unsigned a_used,$type *a_data);
";
}
else {
print
"   void clear();
   void set(unsigned a_used,$type *a_data);
";   
}
print 
"   inline void flush();
";
if (!($type_type & $t_m_flushable)) {
print
"   inline void flush_all();
";
}
else {
print
"   void flush_all();
";
}
print
"   inline void swap($struct_name &second);
   inline $type &operator[](unsigned a_idx);
   inline $type &__at(unsigned a_idx);
   inline $type &__tail(unsigned a_idx);
   inline $type &__last();
   inline $type &__pop();
";
if (($type_type & $t_m_type) eq $t_basic) {
print
"   inline void push($type a_value);
";
}
else {
print
"   inline void push($type &a_value);
";
}
print
"   inline void push_blank();
   inline void push_clear();
   inline $type &pop();
   inline $type &last();
   void copy_resize(unsigned a_size);
";
if ($type eq "char") {
print
"   inline void fill($type a_value);
";
}
else {
if (($type_type & $t_m_type) eq $t_basic) {
print
"   void fill($type a_value);
";
}
else {
print
"   void fill($type &a_value);
";
}
}
if (($type_type & $t_m_type) eq $t_basic) {
print
"   unsigned get_idx($type a_value);
";
}
else {
print
"   unsigned get_idx($type &a_value);
";
}
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   inline $struct_name &operator=($struct_name &src);
   inline bool operator==($struct_name &second);
   inline unsigned save_size();
   inline void save_to(char **b_ptr);
   inline void load_from(char **b_ptr);
";
}
else {
print
"   $struct_name &operator=($struct_name &src);
   bool operator==($struct_name &second);
   unsigned save_size();
   void save_to(char **b_ptr);
   void load_from(char **b_ptr);
";
}
print
"   void print();
";
if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_definition_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_definition_load_file();
}
}
}
if (@m_functions ne 0) {
foreach $_m_function (@m_functions) {
print
"   $_m_function
";
}
}
print
"};

";

print
"// --- struct $struct_name method definition ---

";

# *** array init method ***

print
"inline void $struct_name\:\:init()
{
   size = 0;
   used = 0;
   data = NULL;
}

";

# *** array init_size method ***

print
"inline void $struct_name\:\:init_size(unsigned a_size)
{
   init();
   copy_resize(a_size);
}

";

# *** array clear method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print 
"inline void $struct_name\:\:clear()
";
}
else {
print
"void $struct_name\:\:clear()
";
}
print
"{
   if (data != NULL) {
";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"      $type *ptr = data;
      $type *ptr_end = ptr + size;

      do {
	 ptr->clear();
      } while(++ptr < ptr_end);

";
}
print
"      cfree(data);
   }

   init();
}

";

# *** array set method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline void $struct_name\:\:set(unsigned a_used,$type *a_data)
";
}
else {
print
"void $struct_name\:\:set(unsigned a_used,$type *a_data)
";
}
print
"{
   clear();
   if (a_used == 0) return;

   debug_assert(a_data != NULL);
   copy_resize(a_used);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"
   memcpy(data,a_data,a_used*sizeof($type));
";
}
else {
print
"
   $type *ptr = data;
   $type *ptr_end = ptr + a_used;
   $type *a_ptr = a_data;

   do {
      *ptr = *a_ptr;
   } while(++a_ptr,++ptr < ptr_end);
";
}
print
"   used = a_used;
}

";

# *** array flush method ***

print
"inline void $struct_name\:\:flush()
{
   copy_resize(used);
}

";

# ***  array flush_all method ***

if (!($type_type & $t_m_flushable)) {
print
"inline void $struct_name\:\:flush_all()
";
}
else {
print
"void $struct_name\:\:flush_all()
";
}
print
"{
";
if ($type_type & $t_m_flushable) {
print
"   if (used == 0) return;

   $type *ptr = data;
   $type *ptr_end = ptr + used;

   do {
      ptr->flush_all();
   } while(++ptr < ptr_end);

";
}
print
"   copy_resize(used);
}

";

# *** array swap method ***

print
"inline void $struct_name\:\:swap($struct_name &second)
{
   unsigned tmp_size = size;
   size = second.size;
   second.size = tmp_size;

   unsigned tmp_used = used;
   used = second.used;
   second.used = tmp_used;

   $type *tmp_data = data;
   data = second.data;
   second.data = tmp_data;
}

";

# *** array operator[] method ***

print
"inline $type &$struct_name\:\:operator[](unsigned a_idx)
{
   debug_assert(a_idx < used);
   return data[a_idx];
}

";

# *** array unsafe __at method ***

print
"inline $type &$struct_name\:\:__at(unsigned a_idx)
{
   return data[a_idx];
}

";

# *** array unsafe __tail method ***

print
"inline $type &$struct_name\:\:__tail(unsigned a_idx)
{
   return data[used - 1 - a_idx];
}

";

# *** array unsafe __last method ***

print
"inline $type &$struct_name\:\:__last()
{
   return data[used - 1];
}

";

# *** array unsafe __pop method ***

print
"inline $type &$struct_name\:\:__pop()
{
   return data[--used];
}

";

# *** array push method ***

if (($type_type & $t_m_type) eq $t_basic) {
print
"inline void $struct_name\:\:push($type a_value)
";
}
else {
print
"inline void $struct_name\:\:push($type &a_value)
";
}
print
"{
   if (used >= size) {
      copy_resize(size + c_array_add);
   }
   
   data[used++] = a_value;
}

";

# *** array push_blank method ***

print
"inline void $struct_name\:\:push_blank()
{
   if (used >= size) {
      copy_resize(size + c_array_add);
   }

   used++;
}

";

# *** array push_clear method ***

print
"inline void $struct_name\:\:push_clear()
{
   if (used >= size) {
      copy_resize(size + c_array_add);
   }

";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"   data[used].clear();

";
}
print
"   used++;
}

";

# *** array pop method ***

print
"inline $type &$struct_name\:\:pop()
{
   debug_assert(used > 0);
   return data[--used];
}

";

# *** array last method ***

print
"inline $type &$struct_name\:\:last()
{
   debug_assert(used != 0);
   return data[used - 1];
}

";

# *** array copy_resize method ***

print
"void $struct_name\:\:copy_resize(unsigned a_size)
{
   debug_assert(a_size >= used);

   $type *n_data;

   if (a_size == 0) {
      n_data = NULL;
   }
   else {
      n_data = ($type *)cmalloc(a_size*sizeof($type));
";
if (($type_type & $t_m_type) eq $t_dynamic) {
print 
"
      if (a_size > used) {
	 $type *ptr = n_data + used;
	 $type *ptr_end = n_data + a_size;

	 do {
	    ptr->init();
	 } while(++ptr < ptr_end);
      }
";
}
print
"   }

   if (used != 0) {
      memcpy(n_data,data,used*sizeof($type));
   }
";
if (($type_type & $t_m_type) eq $t_dynamic) {
print 
"
   if (size > used) {
      $type *ptr = data + used;
      $type *ptr_end = data + size;

      do {
	 ptr->clear();
      } while(++ptr < ptr_end);
   }
";
}
print
"
   if (size != 0) {
      cfree(data);
   }

   data = n_data;
   size = a_size;
}

";

# *** array fill method ***

if ($type eq "char") {
print
"inline void $struct_name\:\:fill($type a_value)
";
}
else {
if (($type_type & $t_m_type) eq $t_basic) {
print
"void $struct_name\:\:fill($type a_value)
";
}
else {
print
"void $struct_name\:\:fill($type &a_value)
";
}
}
print
"{
   if (size == 0) return;
";
if ($type eq "char") {
print 
"
   memset(data,a_value,size);
";
}
else {
print
"
   $type *ptr = data;
   $type *ptr_end = ptr + size;

   do {
      *ptr = a_value;
   } while(++ptr < ptr_end);
";
}
print
"
   used = size;
}

";

# *** array get_idx method ***

if (($type_type & $t_m_type) eq $t_basic) {
print
"unsigned $struct_name\:\:get_idx($type a_value)
";
}
else {
print
"unsigned $struct_name\:\:get_idx($type &a_value)
";
}
print
"{
   if (used == 0) return c_idx_not_exist;

   $type *ptr = data;
   $type *ptr_end = ptr + used;

   do {
      if (*ptr == a_value) {
	 return ptr - data;
      }
   } while(++ptr < ptr_end);

   return c_idx_not_exist;
}

";

# *** array operator= method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline $struct_name &$struct_name\:\:operator=($struct_name &src)
";
}
else {
print
"$struct_name &$struct_name\:\:operator=($struct_name &src)
";
}
print
"{
   clear();

   if (src.used == 0) return *this;
   
   copy_resize(src.used);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   memcpy(data,src.data,src.used*sizeof($type));
";
}
else {
print
"
   $type *ptr = data;
   $type *s_ptr = src.data;
   $type *s_ptr_end = s_ptr + src.used;

   do {
      *ptr = *s_ptr;
   } while(++ptr,++s_ptr < s_ptr_end);
";
}
print
"
   used = src.used;
   return *this;
}

";

# *** array operator== method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline bool $struct_name\:\:operator==($struct_name &second)
";
}
else {
print
"bool $struct_name\:\:operator==($struct_name &second)
";
}
print
"{
   if (used != second.used) return false;
   if (used == 0) return true;
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"
   return (memcmp(data,second.data,used*sizeof($type)) == 0);
";
}
else {
print
"
   $type *ptr = data;
   $type *ptr_end = ptr + used;
   $type *s_ptr = second.data;

   do {
      if (!(*ptr == *s_ptr)) {
	 return false;
      }
   } while(++s_ptr,++ptr < ptr_end);

   return true;
";
}
print
"}

";

# *** array save_size method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline unsigned $struct_name\:\:save_size()
";
}
else {
print
"unsigned $struct_name\:\:save_size()
";
}
print
"{
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   return sizeof(unsigned) + used*sizeof($type);
";
}
else {
print
"   if (used == 0) return sizeof(unsigned);
   
   unsigned ret_size = sizeof(unsigned);

   $type *ptr = data;
   $type *ptr_end = ptr + used;

   do {
      ret_size += ptr->save_size();
   } while(++ptr < ptr_end);

   return ret_size;
";
}
print
"}

";

# *** array save_to method ***

if (($type_type & $t_m_type) ne $dynamic_type) {
print
"inline void $struct_name\:\:save_to(char **b_ptr)
";
}
else {
print
"void $struct_name\:\:save_to(char **b_ptr)
";
}
print
"{
   *(unsigned *)*b_ptr = used; *b_ptr += sizeof(unsigned);

   if (used == 0) return;
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"
   unsigned data_size = used*sizeof($type);
   memcpy(*b_ptr,data,data_size);
   *b_ptr += data_size;
"
}
else {
print
"
   $type *ptr = data;
   $type *ptr_end = ptr + used;

   do {
      ptr->save_to(b_ptr);
   } while(++ptr < ptr_end);
"
}
print
"}

";

# *** array load_from method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline void $struct_name\:\:load_from(char **b_ptr)
";
}
else {
print
"void $struct_name\:\:load_from(char **b_ptr)
";
}
print
"{
   clear();

   unsigned tmp_used = *(unsigned *)*b_ptr; *b_ptr += sizeof(unsigned);
   if (tmp_used == 0) return;

   copy_resize(tmp_used);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"
   unsigned data_size = tmp_used*sizeof($type);
   memcpy(data,*b_ptr,data_size);

   *b_ptr += data_size;
   used = tmp_used;
";
}
else {
print
"
   $type *ptr = data;
   $type *ptr_end = ptr + tmp_used;

   do {
      ptr->load_from(b_ptr);
   } while(++ptr < ptr_end);

   used = tmp_used;
";
}
print
"}

";

# *** array print method ***

print
"void $struct_name\:\:print()
{
   if (used == 0) {
      printf(\"empty\");
      return;
   }

   $type *ptr = data;
   $type *ptr_end = ptr + used;

   do {
";
print_type("      ",$type,ptr);
print
"   putchar(\'\\n\');
   } while(++ptr < ptr_end);
}

";

# *** array def functions ***

if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_body_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_body_load_file();
}
}
}
}

# *** container queue definition ***

$queue_struct_postfix = "_q";
@queue_struct_defined = ();

sub make_queue_struct_definition {

$type = edit_type($m_types[0]);

$type_type = get_type_type($type);
$type_name = get_type_name($type);

foreach $_defined (@queue_struct_defined) {
   if ($_defined eq $type_name) {
      die "Err. already defined queue structure of type $type_name.";
   }
}

push @queue_struct_defined,$type_name;

$struct_name = $type_name . $queue_struct_postfix;
push @dynamic_types,$struct_name;
push @flushable_types,$struct_name;

print
"// --- struct $struct_name definition ---

";
if (@m_names ne 0) {
foreach $_m_name (@m_names) {
push @type_abbreviations,$_m_name;
push @abbreviated_types,$struct_name;
print 
"typedef struct $struct_name $_m_name;
";
}
print 
"
";
}

print
"struct $struct_name
{
   unsigned size;
   unsigned used;
   unsigned begin;
   $type *data;

   inline void init();
   inline void init_size(unsigned a_size);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   inline void clear();
";
}
else {
print
"   void clear();
";
}
print
"   inline void flush();
";
if (!($type_type & $t_m_flushable)) {
print
"   inline void flush_all();
";
}
else {
print
"   void flush_all();
";
}
print
"   inline void swap($struct_name &second);
";
if (($type_type & $t_m_type) eq $t_basic) {
print
"   inline void insert($type a_value);
";
}
else {
print
"   inline void insert($type &a_value);
";
}
print
"   inline void insert_blank();
   inline $type &next();
   inline $type &last();
   void copy_resize(unsigned a_size);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   inline $struct_name &operator=($struct_name &src);
";
}
else {
print
"   $struct_name &operator=($struct_name &src);
";
}
print
"   bool operator==($struct_name &second);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   inline unsigned save_size();
   inline void save_to(char **b_ptr);
   inline void load_from(char **b_ptr);
";
}
else {
print
"   unsigned save_size();
   void save_to(char **b_ptr);
   void load_from(char **b_ptr);
";
}
print
"   void print();
";
if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_definition_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_definition_load_file();
}
}
}
if (@m_functions ne 0) {
foreach $_m_function (@m_functions) {
print
"   $_m_function
";
}
}
print
"};

";

print
"// --- struct $struct_name method definition ---

";

# *** queue init method ***

print
"inline void $struct_name\:\:init()
{
   size = 0;
   used = 0;
   begin = 0;
   data = NULL;
}

";

# *** queue init_size method ***

print
"inline void $struct_name\:\:init_size(unsigned a_size)
{
   init();
   copy_resize(a_size);
}

";

# *** queue clear method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline void $struct_name\:\:clear()
";
}
else {
print
"void $struct_name\:\:clear()
";
}
print
"{
   if (data != NULL) {
";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"      $type *ptr = data;
      $type *ptr_end = ptr + size;

      do {
	 ptr->clear();
      } while(++ptr < ptr_end);

";
}
print
"      cfree(data);
   }

   init();
}

";

# *** queue flush method ***

print
"inline void $struct_name\:\:flush()
{
   copy_resize(used);
}

";

# *** queue flush_all method ***

if (!($type_type & $t_m_flushable)) {
print
"inline void $struct_name\:\:flush_all()
";
}
else {
print
"void $struct_name\:\:flush_all()
";
}
print
"{
   copy_resize(used);
";
if ($type_type & $t_m_flushable) {
print
"
   if (used == 0) return;

   $type *ptr = data;
   $type *ptr_end = ptr + used;

   do {
      ptr->flush_all();
   } while(++ptr < ptr_end);
";
}
print
"}

";

# *** queue swap method ***

print
"inline void $struct_name\:\:swap($struct_name &second)
{
   unsigned tmp_size = size;
   size = second.size;
   second.size = tmp_size;

   unsigned tmp_used = used;
   used = second.used;
   second.used = tmp_used;

   unsigned tmp_begin = begin;
   begin = second.begin;
   second.begin = tmp_begin;

   $type *tmp_data = data;
   data = second.data;
   second.data = tmp_data;
}

";

# *** queue insert method ***

if (($type_type & $t_m_type) eq $t_basic) {
print
"inline void $struct_name\:\:insert($type a_value)
";
}
else {
print
"inline void $struct_name\:\:insert($type &a_value)
";
}
print
"{
   if (used >= size) {
      copy_resize(size + c_array_add);
   }

   if (begin + used >= size) {
      data[begin + used - size] = a_value;
   }
   else {
      data[begin + used] = a_value;
   }

   used++;
}

";

# *** queue insert_blank method ***

print
"inline void $struct_name\:\:insert_blank()
{
   if (used >= size) {
      copy_resize(size + c_array_add);
   }

   used++;
}

";

# *** queue next method ***

print
"inline $type &$struct_name\:\:next()
{
   debug_assert(used > 0);

   unsigned ret_idx = begin;

   if (++begin >= size) {
      begin = 0;
   }

   used--;

   return data[ret_idx];
}

";

# *** queue last method ***

print
"inline $type &$struct_name\:\:last()
{
   debug_assert(used > 0);

   if (begin + (used - 1) >= size) {
      return data[begin + (used - 1) - size];
   }
   else {
      return data[begin + (used - 1)];
   }
}

";

# *** queue copy_resize method ***

print
"void $struct_name\:\:copy_resize(unsigned a_size)
{
   debug_assert(a_size >= used);

   $type *n_data;

   if (a_size == 0) {
      n_data = NULL;
   }
   else {
      n_data = ($type *)cmalloc(a_size*sizeof($type));
";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"
      if (a_size > used) {
	 $type *ptr = n_data + used;
	 $type *ptr_end = n_data + a_size;

	 do {
	    ptr->init();
	 } while(++ptr < ptr_end);
      }
";
}
print
"   }

   if (used != 0) {
      unsigned fir_cnt;
      unsigned sec_cnt;

      if (begin + used > size) {
	 sec_cnt = begin + used - size;
	 fir_cnt = used - sec_cnt;
      }
      else {
	 fir_cnt = used;
	 sec_cnt = 0;
      }

      memcpy(n_data,data + begin,fir_cnt*sizeof($type));
   
      if (sec_cnt != 0) {
	 memcpy(n_data + fir_cnt,data,sec_cnt*sizeof($type));
      }
   }

   if (data != NULL) {
";
if (($type_type & $t_m_type) eq $t_dynamic) {
print
"      $type *ptr;
      $type *ptr_end;
      $type *s_ptr;

      if (begin + used >= size) {
	 ptr = data + (begin + used - size);
	 ptr_end = data + begin;
	 s_ptr = NULL;
      }
      else {
	 ptr = data;
	 ptr_end = data + begin;
	 s_ptr = ptr_end + used;
      }

      if (ptr < ptr_end) {
	 do {
	    ptr->clear();
	 } while(++ptr < ptr_end);
      }

      if (s_ptr != NULL) {
	 $type *s_ptr_end = data + size;
	 do {
	    s_ptr->clear();
	 } while(++s_ptr < s_ptr_end);
      }

";
}
print
"      cfree(data);
   }

   data = n_data;
   size = a_size;
   begin = 0;
}

";

# *** queue operator= method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline $struct_name &$struct_name\:\:operator=($struct_name &src)
";
}
else {
print
"$struct_name &$struct_name\:\:operator=($struct_name &src)
";
}
print
"{
   clear();

   if (src.used == 0) return *this;

   copy_resize(src.used);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"
   unsigned fir_cnt;
   unsigned sec_cnt;

   if (src.begin + src.used > src.size) {
      sec_cnt = src.begin + src.used - src.size;
      fir_cnt = src.used - sec_cnt;
   }
   else {
      fir_cnt = src.used;
      sec_cnt = 0;
   }

   memcpy(data,src.data + src.begin,fir_cnt*sizeof($type));

   if (sec_cnt != 0) {
      memcpy(data + fir_cnt,src.data,sec_cnt*sizeof($type));
   }
";
}
else {
print
"
   unsigned sec_cnt;
   $type *ptr = data;
   $type *s_ptr = src.data + src.begin;
   $type *s_ptr_end;

   if (src.begin + src.used > src.size) {
      s_ptr_end = src.data + src.size;
      sec_cnt = src.begin + src.used - src.size;
   }
   else {
      s_ptr_end = s_ptr + src.used;
      sec_cnt = 0;
   }

   do {
      *ptr = *s_ptr;
   } while(++ptr,++s_ptr < s_ptr_end);

   if (sec_cnt != 0) {
      s_ptr = src.data;
      s_ptr_end = s_ptr + sec_cnt;

      do {
	 *ptr = *s_ptr;
      } while(++ptr,++s_ptr < s_ptr_end);
   }
"
}
print
"
   used = src.used;
   return *this;
}

";

# *** queue operator== method ***

print
"bool $struct_name\:\:operator==($struct_name &second)
{
   if (used != second.used) return false;
   if (used == 0) return true;
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"
   bool _break;
   bool s_break;
   unsigned pos;
   unsigned pos_end;
   unsigned s_pos;

   _break = (begin + used > size);
   s_break = (second.begin + second.used > second.size);
   pos = begin;
   s_pos = second.begin;

   if (_break) {
      pos_end = begin + used - size;
   }
   else {
      pos_end = begin + used;
   }

   do {
      if (_break) {
	 unsigned offset = size - pos;
	 
	 if (s_break) {
	    unsigned s_offset = second.size = s_pos;

	    if (offset < s_offset) {
	       if (memcmp(data + pos,second.data + s_pos,offset*sizeof($type)) != 0) {
		  return false;
	       }

	       s_pos += offset;
	       pos = 0;
	       _break = false;
	    }
	    else {
	       if (memcmp(data + pos,second.data + s_pos,s_offset*sizeof($type)) != 0) {
		  return false;
	       }

	       if (pos += s_offset >= size) {
		  pos = 0;
		  _break = false;
	       }
	       s_pos = 0;
	       s_break = false;
	    }
	 }
	 else {
	    if (memcmp(data + pos,second.data + s_pos,offset*sizeof($type)) != 0) {
	       return false;
	    }
	    s_pos += offset;
	    pos = 0;
	    _break = false;
	 }
      }
      else {
	 if (s_break) {
	    unsigned s_offset = second.size - s_pos;

	    if (memcmp(data + pos,second.data + s_pos,s_offset*sizeof($type)) != 0) {
	       return false;
	    }
	    pos += s_offset;
	    s_pos = 0;
	    s_break = false;
	 }
	 else {
	    if (memcmp(data + pos,second.data + s_pos,(pos_end - pos)*sizeof($type)) != 0) {
	       return false;
	    }
	    else {
	       return true;
	    }
	 }
      }
   } while(1);
";
}
else {
print
"
   $type *ptr = data + begin;
   $type *ptr_break = data + size;
   $type *ptr_end;
   $type *s_ptr = second.data + second.begin;
   $type *s_ptr_break = second.data + second.size;

   if (begin + used > size) {
      ptr_end = data + begin + used - size;
   }
   else {
      ptr_end = ptr + used;
   }

   do {
      if (!(*ptr == *s_ptr)) {
	 return false;
      }
      
      if (++ptr >= ptr_break) {
	 ptr = data;
      }

      if (++s_ptr >= s_ptr_break) {
	 s_ptr = second.data;
      }

   } while(ptr != ptr_end);

   return true;
";
}
print
"}

";

# *** queue save_size method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline unsigned $struct_name\:\:save_size()
";
}
else {
print
"unsigned $struct_name\:\:save_size()
";
}
print
"{
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   return sizeof(unsigned) + used*sizeof($type);
";
}
else {
print
"   if (used == 0) return sizeof(unsigned);

   unsigned ret_size = sizeof(unsigned);
   unsigned sec_cnt;
   $type *ptr = data + begin;
   $type *ptr_end;

   if (begin + used > size) {
      ptr_end = data + size;
      sec_cnt = begin + used - size;
   }
   else {
      ptr_end = ptr + used;
      sec_cnt = 0;
   }

   do {
      ret_size += ptr->save_size();
   } while(++ptr < ptr_end);

   if (sec_cnt != 0) {
      ptr = data;
      ptr_end = ptr + sec_cnt;

      do {
	 ret_size += ptr->save_size();
      } while(++ptr < ptr_end);
   }

   return ret_size;
";
}
print
"}

";

# *** queue save_to method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline void $struct_name\:\:save_to(char **b_ptr)
";
}
else {
print
"void $struct_name\:\:save_to(char **b_ptr)
";
}
print
"{
   *(unsigned *)*b_ptr = used; *b_ptr += sizeof(unsigned);
   if (used == 0) return;
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"
   unsigned fir_cnt;
   unsigned sec_cnt;

   if (begin + used > size) {
      sec_cnt = begin + used - size;
      fir_cnt = used - sec_cnt;
   }
   else {
      fir_cnt = used;
      sec_cnt = 0;
   }

   memcpy(*b_ptr,data + begin,fir_cnt*sizeof($type));
   *b_ptr += fir_cnt*sizeof($type);

   if (sec_cnt != 0) {
      memcpy(*b_ptr,data,sec_cnt*sizeof($type));
      *b_ptr += sec_cnt*sizeof($type);
   }
";
}
else {
print
"
   unsigned sec_cnt;
   $type *ptr = data + begin;
   $type *ptr_end;

   if (begin + used > size) {
      sec_cnt = begin + used - size;
      ptr_end = data + size;
   }
   else {
      ptr_end = ptr + used;
      sec_cnt = 0;
   }

   do {
      ptr->save_to(b_ptr);
   } while(++ptr < ptr_end);

   if (sec_cnt != 0) {
      ptr = data;
      ptr_end = ptr + sec_cnt;

      do {
	 ptr->save_to(b_ptr);
      } while(++ptr < ptr_end);
   }
";
}
print
"}

";

# *** queue load_from method ***

if (($type_type & $t_m_type) ne $t_dynamic) {
print
"inline void $struct_name\:\:load_from(char **b_ptr)
";
}
else {
print
"void $struct_name\:\:load_from(char **b_ptr)
";
}
print
"{
   clear();

   unsigned tmp_used = *(unsigned *)*b_ptr; *b_ptr += sizeof(unsigned);
   if (tmp_used == 0) return;

   copy_resize(tmp_used);
";
if (($type_type & $t_m_type) ne $t_dynamic) {
print
"   memcpy(data,*b_ptr,tmp_used*sizeof($type));
";
}
else {
print
"
   $type *ptr = data;
   $type *ptr_end = ptr + tmp_used;

   do {
      ptr->load_from(b_ptr);
   } while(++ptr < ptr_end);
";
}
print
"
   used = tmp_used;
   begin = 0;
}

";

# *** queue print method ***

print
"void $struct_name\:\:print()
{
   if (used == 0) {
      printf(\"empty\");
      return;
   }

   unsigned sec_cnt;
   $type *ptr = data + begin;
   $type *ptr_end;

   if (begin + used > size) {
      sec_cnt = begin + used - size;
      ptr_end = data + size;
   }
   else {
      ptr_end = ptr + used;
      sec_cnt = 0;
   }

   do {
";
print_type("      ",$type,ptr);
print
"   } while(++ptr < ptr_end);

   if (sec_cnt != 0) {
      ptr = data;
      ptr_end = ptr + sec_cnt;

      do {
";
print_type("         ",$type,ptr);
print
"      } while(++ptr < ptr_end);
   }
}

";

# *** queue def functions ***

if (@m_fundefs ne 0) {
foreach $_m_fundef (@m_fundefs) {
if ($_m_fundef eq $fd_save_file) {
fd_body_save_file();
}
elsif ($_m_fundef eq $fd_load_file) {
fd_body_load_file();
}
}
}
}

sub process_match {
   $define_string = $_[0];

   @m_names = ();
   @m_types = ();
   @m_variables = ();
   @m_functions = ();
   @m_fundefs = ();

   $define_end = 0;
   do {
      $line = <FILE>;

      if ($line=~m/%%end/) {
	 $define_end = 1;
      }
      else {
	 $line=~m/".*"/;
	 @string_split = split "\"",$&;

	 if ($line=~m/%name/) {
	    push @m_names,$string_split[1];
	 }
	 elsif ($line=~m/%type/) {
	    push @m_types,$string_split[1];
	 }
	 elsif ($line=~m/%variable/) {
	    push @m_variables,$string_split[1];
	 }
	 elsif ($line=~m/%function/) {
	    push @m_functions,$string_split[1];
	 }
	 elsif ($line=~m/%fundef/) {
	    push @m_fundefs,$string_split[1];
	 }
      }
   } while(!$define_end);

   if ($define_string=~m/double_array/) {
      if (@m_types < 1) {
	 die "Err. structure double_array need at least one type.";
      }

      make_double_array_struct_definition();
   }
   elsif ($define_string=~m/hash_array/) {
      if (@m_types < 1) {
	 die "Err. structure hash_array need at least one type.";
      }

      make_hash_array_struct_definition();
   }
   elsif ($define_string=~m/array/) {
      if (@m_types < 1) {
	 die "Err. structure array need at least one type.";
      }

      make_array_struct_definition();
   }
   elsif ($define_string=~m/queue/) {
      if (@m_types < 1) {
	 die "Err. structure queue need at least one type.";
      }

      make_queue_struct_definition();
   }
   elsif ($define_string=~m/map/) {
      if (@m_types < 2) {
	 die "Err. structure map need two types.";
      }

      make_map_struct_definition();
   }
   elsif ($define_string=~m/set/) {
      if (@m_types < 1) {
	 die "Err. structure set need at least one type.";
      }
      if (@m_types ne @m_variables) {
	 die "Err. structure set need same count of types and variables.";
      }

      make_set_struct_definition();
   }
   else {
      die "Err. unknown structure type in $_[0].";
   }
}

sub process_file {
   $file_name = $_[0];

   open FILE,$file_name;
   while (<FILE>) {
      if ($_=~m/%%define/) {
	 process_match($_);
      }
      else {
	 print $_;
      }
   }
   close FILE;
}

process_file("mid_main.cc");

