
/*
 * author Jiří Zuzaňák
 * institution BUT FIT UPGM
 */

// - datovy typ instrukce v semantickem bytovem kodu -

typedef unsigned short t_si_type;

// --- struct string_s definition ---

/*
 * struktura implementujici retezec, zapsana takovym zpusobem,
 * aby se mohla pouzit v definicich objektu skriptu process.pl
 */
struct string_s
{
   unsigned size;
   char *data;

   inline void init();
   inline void clear();
   inline void create(unsigned a_length);
   inline void set(unsigned a_length,char *a_data);
   void set_format(unsigned a_length,char *a_data);
   void set_format_parm(unsigned a_length,char *a_data,unsigned a_parm_cnt,unsigned *params);
   inline void conc_set(unsigned a_flength,char *a_fdata,unsigned a_slength,char *a_sdata);
   inline void flush_all();
   inline void swap(string_s &second);
   inline char &operator[](unsigned a_idx);
   inline string_s &operator=(string_s &src);
   inline bool operator==(string_s &second);
   inline bool compare_char_ptr(unsigned a_length,char *a_data);
   inline unsigned save_size();
   inline void save_to(char **b_ptr);
   inline void load_from(char **b_ptr);
   inline void print();
   inline bool load_text_file(char *file);
   inline bool save_text_file(char *file);
};

inline void string_s::init()
{
   size = 0;
   data = NULL;
}

inline void string_s::clear()
{
   if (data != NULL) {
      cfree(data);
   }

   init();
}

inline void string_s::create(unsigned a_length)
{
   clear();
   data = (char *)cmalloc((a_length + 1)*sizeof(char));

   data[a_length] = '\0';
   size = a_length + 1;
}

inline void string_s::set(unsigned a_length,char *a_data)
{
   clear();
   data = (char *)cmalloc((a_length + 1)*sizeof(char));

   if (a_length != 0) {
      memcpy(data,a_data,a_length*sizeof(char));
   }

   data[a_length] = '\0';
   size = a_length + 1;
}

inline void string_s::conc_set(unsigned a_flength,char *a_fdata,unsigned a_slength,char *a_sdata)
{
   clear();
   data = (char *)cmalloc((a_flength + a_slength + 1)*sizeof(char));

   if (a_flength != 0) {
      memcpy(data,a_fdata,a_flength*sizeof(char));
   }

   if (a_slength != 0) {
      memcpy(data + a_flength,a_sdata,a_slength*sizeof(char));
   }

   data[a_flength + a_slength] = '\0';
   size = a_flength + a_slength + 1;
}

inline void string_s::flush_all()
{
}

inline void string_s::swap(string_s &second)
{
   unsigned tmp_size = size;
   size = second.size;
   second.size = tmp_size;

   char *tmp_data = data;
   data = second.data;
   second.data = tmp_data;
}

inline char &string_s::operator[](unsigned a_idx)
{
   debug_assert(a_idx < size);
   return data[a_idx];
}

inline string_s &string_s::operator=(string_s &src)
{
   clear();
   if (src.size == 0) return *this;
   data = (char *)cmalloc(src.size*sizeof(char));
   memcpy(data,src.data,src.size*sizeof(char));
   size = src.size;

   return *this;
}

inline bool string_s::operator==(string_s &second)
{
   if (size != second.size) return false;
   if (size == 0) return true;
   return (memcmp(data,second.data,size*sizeof(char)) == 0);
}

inline bool string_s::compare_char_ptr(unsigned a_length,char *a_data)
{
   if (size != a_length + 1) return false;
   if (a_length == 0) return true;
   return (memcmp(data,a_data,a_length*sizeof(char)) == 0);
}

inline unsigned string_s::save_size()
{
   return sizeof(unsigned) + size*sizeof(char);
}

inline void string_s::save_to(char **b_ptr)
{
   *(unsigned *)*b_ptr = size; *b_ptr += sizeof(unsigned);

   if (size == 0) return;
   memcpy(*b_ptr,data,size*sizeof(char));
   *b_ptr += size*sizeof(char);
}

inline void string_s::load_from(char **b_ptr)
{
   clear();
   size = *(unsigned *)*b_ptr; *b_ptr += sizeof(unsigned);

   unsigned data_size = size*sizeof(char);
   data = (char *)cmalloc(data_size);
   memcpy(data,*b_ptr,data_size);
   *b_ptr += data_size;
}

inline void string_s::print()
{
   if (size == 0) return;
   printf("%s",data);
}

inline bool string_s::load_text_file(char *file)
{
   if (file == NULL) return false;
   FILE *f = fopen(file,"rb");
   if (f == NULL) return false;

   fseek(f,0,SEEK_END);
   unsigned file_size = ftell(f);
   fseek(f,0,SEEK_SET);

   clear();
   data = (char *)cmalloc((file_size + 1)*sizeof(char));
   fread(data,file_size,1,f);
   fclose(f);

   data[file_size] = '\0';
   size = file_size + 1;

   return true;
}

inline bool string_s::save_text_file(char *file)
{
   if (file == NULL) return false;
   FILE *f = fopen(file,"wb");
   if (f == NULL) return false;

   if (size > 1) {
      fwrite(data,size - 1,1,f);
   }

   fclose(f);
   return true;
}

